#ifndef	NOIDENT
#ident	"@(#)FlatP.h	1.2 SMI"
#ident	"@(#)flat:FlatP.h	1.12"
#endif

#ifndef _OL_FLATP_H
#define _OL_FLATP_H

/*
 ************************************************************************	
 * Description:
 *	This is the flat container's private header file.
 ************************************************************************	
 */

#include <Xol/PrimitiveP.h>
#include <Xol/Flat.h>

/*
 ************************************************************************	
 * Define structures and names/types needed to support flat widgets
 ************************************************************************	
 */

			/* Define a structure that holds the flat
			 * resources for the container			*/

typedef struct {
	Cardinal	size;		/* size of resource's data	*/
	Cardinal	offset;		/* offset into structure	*/
	XrmQuark	quark;		/* quarkified name		*/
} OlFlatResource, *OlFlatResourceList;

			/* Define a structure that's used to extract
			 * information from the application's list	*/
typedef struct {
	OlFlatResourceList	xlist;	/* extractor list		*/
	Cardinal		num_xlist; /* number of items to extract*/
} OlFlatExtractInfo;

			/* Create a structure for holding transient
			 * geometry information				*/

typedef struct {
	Cardinal	rows;		/* number of rows		*/
	Cardinal	cols;		/* number of columns		*/
	Position	x_offset;	/* x coordinate of first item	*/
	Position	y_offset;	/* y coordinate of first item	*/
	Dimension	bounding_width;	/* width tightly bounding items	*/
	Dimension	bounding_height;/* height tightly bounding items*/
	Dimension *	col_widths;	/* array of column widths	*/
	Dimension *	row_heights;	/* array of row heights		*/
} OlFlatGeometry;

			/* Define a structure used to Draw Sub-objects	*/

typedef struct {
	Drawable	drawable;	/* place to draw	*/
	Screen *	screen;		/* drawable's screen	*/
	Pixel		background;	/* drawable's backgrnd	*/
	Position	x;		/* item's X coordinate	*/
	Position	y;		/* item's Y coordinate	*/
	Dimension	width;		/* item's width		*/
	Dimension	height;		/* item's height	*/
	Boolean		clipped;	/* is it clipped ?	*/
	Dimension	clip_width;	/* clip width		*/
	Dimension	clip_height;	/* clip height		*/
} OlFlatDrawInfo;

				/* Define a structure to cache information
				 * on a per screen basis		*/

typedef struct _OlFlatScreenCache {
	Screen *		screen;		/* screen id		*/
	struct _OlFlatScreenCache * next;	/* next node in list	*/
	XFontStruct *		font;		/* OPEN LOOK Font	*/
	XFontStruct *		button_font;	/* OPEN LOOK Font	*/
	char *			arrow_string;	/* Truncation arrow	*/
	Pixmap			stipple;	/* Stippled pixmap	*/
	Dimension		arrow_width;	/* in pixels		*/
	Dimension		arrow_padding;	/* points cvted to pixels*/
	Dimension		stroke_width;	/* brush stroke width	*/
	GC			default_gc;	/* default GC		*/
	GC			scratch_gc;	/* scratch GC		*/
	Cardinal		count;		/* reference count	*/
} OlFlatScreenCache, *OlFlatScreenCacheList;

#define OL_JUST_LOOKING	1		/* OlFlatScreenManager flag	*/
#define OL_ADD_REF	2		/* OlFlatScreenManager flag	*/
#define OL_DELETE_REF	3		/* OlFlatScreenManager flag	*/

			/* Define a structure that handles the label
			 * drawing for all flat widgets.		*/
typedef struct {
	Pixel		foreground;		/* label foreground	*/
	Pixel		background;		/* label background	*/
	XFontStruct *	label_font;		/* text label font	*/
	GC		label_gc;		/* GC or NULL		*/
	GC		stipple_gc;		/* GC or NULL		*/
	OlDefine	label_justify;		/* label justification	*/
	String		label;			/* text label		*/
	XImage *	image;			/* XImage label		*/
	Pixmap		pixmap;			/* Pixmap label		*/
	Boolean		sensitive;		/* sensitive label ?	*/
	Boolean		tile;			/* tile the label ?	*/
	OlFlatScreenCache *sc;			/* Pointer to Cache	*/
} OlFlatLabelInfo;

			/* Define new function pointers for the flat
			 * widget class.				*/

typedef void	(*OlFlatAnalyzeItemsProc)();
/*	Widget		w;		container widget id		*/

typedef void	(*OlFlatDrawItemProc)();
/*	Widget		w;		container widget id		*/
/*	Cardinal	index;		Item to draw			*/
/*	FlatItem	item;		expanded item			*/
/*	OlFlatDrawInfo *draw_info;	Information used in drawing	*/

typedef void	(*OlFlatExpandItemProc)();
/*	Widget		w;		flat widget container		*/
/*	Cardinal	index;		item to be expanded		*/
/*	FlatItem	item;		expanded item			*/

typedef void (*OlFlatGetDrawInfoProc)();
/*	Widget		w;		Flat widget type		*/
/*	Cardinal	index;		Item to get drawing info. for	*/
/*	OlFlatDrawInfo *draw_info;	Information used in drawing	*/

typedef Cardinal	(*OlFlatGetIndexFunc)();
/*	Widget		w;			Flat widget type	*/
/*	Position	x;			X source location	*/
/*	Position	y;			Y source location	*/
/*	Boolean		ignore_sensitivity;	don't look at sensitivity*/

typedef void	(*OlFlatGetItemHookProc)();
/*	Widget		widget;		flat widget container		*/
/*	Cardinal	index;		item to be expanded		*/
/*	FlatItem	item;		expanded item			*/
/*	ArgList		args;		item Args			*/
/*	Cardinal *	num_args;	num item Args			*/

typedef void	(*OlFlatInitializeItemProc)();
/*	Widget		w;		Flat widget container id	*/
/*	Cardinal	index;		item to be checked		*/
/*	FlatItem	request;	expanded requested item		*/
/*	FlatItem	new;		expanded new item		*/

typedef void	(*OlFlatItemDimensionsProc)();
/*	Widget		w;		Flat widget container id	*/
/*	Cardinal	item_index;	item to be sized		*/
/*	FlatItem	item;		expanded item			*/
/*	Dimension *	width;		returned width			*/
/*	Dimension *	height;		returned height			*/

typedef void	(*OlFlatItemsTouchedProc)();
/*	Widget		w;		flat widget container		*/

typedef void	(*OlFlatLayoutProc)();
/*	Widget		w;		Flat widget container id	*/
/*	Dimension *	return_width;	returned width			*/
/*	Dimension *	return_height;	returned height			*/

typedef Boolean	(*OlFlatSetItemHookFunc)();
/*	Widget		w;		flat widget container		*/
/*	Cardinal	item_index;	item being modified		*/
/*	FlatItem	current;	item application has		*/
/*	FlatItem	request;	item application wants		*/
/*	FlatItem	new;		item application gets		*/
/*	ArgList		args;		item Args			*/
/*	Cardinal *	num_args;	num item Args			*/

			/* Define inheritance procedures for the class	*/

typedef OlDefine	OlFlatLayoutContainerId;

#define XtInheritFlatDrawItem		((OlFlatDrawItemProc) _XtInherit)
#define XtInheritFlatGetDrawInfo	((OlFlatGetDrawInfoProc)_XtInherit)
#define XtInheritFlatGetIndex		((OlFlatGetIndexFunc) _XtInherit)
#define XtInheritFlatItemDimensions	((OlFlatItemDimensionsProc) _XtInherit)
#define XtInheritFlatLayout		((OlFlatLayoutProc) _XtInherit)
#define XtInheritFlatLayoutContainer	((OlFlatLayoutContainerId)OL_IGNORE)

/*
 ************************************************************************	
 * Declare external routines and macros used by the flat widgets.
 * External routines exist for chained class procedures only.
 * Macros exist for non-chained class procedures, they provide a
 * way of checking the number of parameters when using a pointers
 * to functions.
 ************************************************************************	
 */

extern void
_OlDoGravity();
/*	int		gravity;	 the desired gravity		*/
/*	Dimension	c_width;	 container's width		*/
/*	Dimension	c_height;	 container's height		*/
/*	Dimension	width;		 interior item width		*/
/*	Dimension	height;		 interior item height		*/
/*	Position *	x;		 returned x offset		*/
/*	Position *	y;		 returned y offset		*/

extern Boolean
_OlFlatDoLayout();
/*	Widget		w;		flat widget container		*/
/*	WidgetClass	wc;		subclass's id			*/

extern void
_OlFlatDrawLabel();
/*	OlFlatDrawInfo *	di;	drawable's information		*/
/*	OlFlatLabelInfo *	li;	label's information		*/

extern void
_OlFlatExpandItem();
/*	Widget		w;		flat widget container		*/
/*	Cardinal	index;		item to be expanded		*/
/*	FlatItem	dest;		Destination base address	*/

extern void
_OlFlatInitializeItems();
/*	Widget		w;		flat widget container		*/
/*	ArgList		args;		container Args			*/
/*	Cardinal	num_args;	num container Args		*/

extern void
_OlFlatPreviewItem();
/*	Widget		w;		container widget id		*/
/*	Cardinal	index;		Item to draw			*/
/*	Widget		preview;	destination preview widget	*/
/*	Cardinal	preview_index;	sub-object within preview widget*/

extern void
_OlFlatRefreshItem();
/*	Widget		w;		container widget id		*/
/*	Cardinal	index;		Item to draw			*/
/*	Boolean		clear_area;	should the area be cleared?	*/

extern OlFlatScreenCache *
_OlFlatScreenManager();
/*	Widget		w;		any widget id			*/
/*	OlDefine	flag;		OlFlatScreenManager flag	*/

#define OL_FLATCLASS(w,field) \
			(((FlatWidgetClass)XtClass(w))->flat_class.field)

#define _OlFlatDrawItem(w,i,ei,d)	(*OL_FLATCLASS(w,draw_item))(w,i,ei,d)
#define _OlFlatGetDrawInfo(w,i,d)	(*OL_FLATCLASS(w,get_draw_info))(w,i,d)
#define _OlFlatGetIndex(w,x,y,f)	(*OL_FLATCLASS(w,get_index))(w,x,y,f)
#define _OlFlatItemDimensions(w,i,ei,iw,ih) \
			(* OL_FLATCLASS(w,item_dimensions))(w,i,ei,iw,ih)
#define _OlFlatLayout(w,x,y)		(*OL_FLATCLASS(w,layout))(w,x,y)

/*
 ************************************************************************	
 * Define Widget Class Part and Class Rec
 ************************************************************************	
 */

				/* Define new fields for the class part	*/

typedef struct _FlatClassPart {
    OlDefine			container_type;	/* The container's id	*/
    XtResourceList		item_resources;	/* sub-object resources	*/
    Cardinal			num_item_resources;/* number of item rsc*/
    XrmQuarkList		quarked_items;	/* quarked names	*/
    Cardinal			part_size;	/* this item's part size*/
    Cardinal			part_offset;	/* item's default part	*/
    Cardinal			part_in_rec_offset;/* part's record offset*/
    Cardinal			rec_size;	/* Full Record size	*/
    OlFlatAnalyzeItemsProc	analyze_items;	/* check all items	*/
    OlFlatDrawItemProc		draw_item;	/* subclass's draw proc.*/
    OlFlatExpandItemProc	expand_item;	/* supplemental expansion*/
    OlFlatGetDrawInfoProc	get_draw_info;	/* returns drawing info	*/
    OlFlatGetIndexFunc		get_index;	/* gets an item's index	*/
    OlFlatGetItemHookProc	get_item_hook;	/* queries an item	*/
    OlFlatInitializeItemProc	initialize_item;/* checking a new item	*/
    OlFlatItemDimensionsProc	item_dimensions;/* item's width/height	*/
    OlFlatItemsTouchedProc	items_touched;	/* for new lists	*/
    OlDefine			layout_container;/* Flat to do layout	*/
    OlFlatLayoutProc		layout;		/* sub-object layout	*/
    OlFlatSetItemHookFunc	set_item_hook;	/* updates an item	*/
    XtPointer			extension;	/* future extensions	*/
} FlatClassPart;

				/* Full class record declaration 	*/

typedef struct _FlatClassRec {
    CoreClassPart	core_class;
    PrimitiveClassPart	primitive_class;
    FlatClassPart	flat_class;
} FlatClassRec;

				/* External class record declaration	*/

extern FlatClassRec		flatClassRec;

/*
 ************************************************************************	
 * Define Widget Instance Structure
 ************************************************************************	
 */

				/* Define Expanded sub-object instance	*/

typedef struct {
	Boolean		managed;	/* is this item managed ?	*/
	Boolean		mapped_when_managed; /* mapped when managed ?	*/
	Boolean		sensitive;	/* is this item sensitive ?	*/
	Boolean		ancestor_sensitive; /* ancestor sensitive ?	*/
	Pixel		background;	/* Background color		*/
	Pixmap		background_pixmap; /* Background pixmap		*/
	XtPointer	user_data;	/* application data hook	*/
} FlatItemPart;

			/* Item's Full Instance record declaration	*/

typedef struct {
	FlatItemPart	flat;
} FlatItemRec, *FlatItem;


			/* Define new fields for the instance part	*/

typedef struct _FlatPart {
						/* layout resources	*/

	Dimension	h_pad;		/* horizontal margin padding	*/
	Dimension	h_space;	/* internal horizontal padding	*/
	Dimension	item_max_height;/* item's maximum allow. height	*/
	Dimension	item_max_width;	/* item's maximum allow. width	*/
	Dimension	item_min_height;/* item's minimum allow. height	*/
	Dimension	item_min_width;	/* item's minimum allow. width	*/
	Dimension	overlap;	/* overlap of each item		*/
	Dimension	v_pad;		/* vertical margin padding	*/
	Dimension	v_space;	/* internal vertical padding	*/
	int		gravity;	/* gravity for sub-object group	*/
	int		item_gravity;	/* gravity for each sub-object	*/
	int		measure;	/* layout related dimension	*/
	OlDefine	layout_height;	/* height boundary constraint	*/
	OlDefine	layout_type;	/* type of desired layout	*/
	OlDefine	layout_width;	/* width boundary constraint	*/
	OlDefine	same_height;	/* forcing of sub-object height	*/
	OlDefine	same_width;	/* forcing of sub-object width	*/

					/* Sub-object related resources	*/

	XtPointer	items;		/* sub-object list		*/
	String *	item_fields;	/* array of fields in item list	*/
	Cardinal	num_item_fields;/* Number of fields per item	*/
	Cardinal	num_items;	/* number of sub-object items	*/
	Cardinal	num_managed;	/* Number of managed items	*/
	Cardinal *	managed_items;	/* list of managed items	*/
	
							/* Private data	*/

	Boolean		relayout_hint;	/* flag suggesting a relayout	*/
	Boolean		items_touched;	/* application touched items	*/
	OlFlatExtractInfo extract_info; /* resource extraction info.	*/
	OlFlatGeometry	item_geom;	/* calculated item geometry	*/
	FlatItemPart	item_part;	/* default item attributes	*/
} FlatPart;

				/* Full instance record declaration	*/

typedef struct _FlatRec {
    CorePart		core;
    PrimitivePart	primitive;
    FlatPart		flat;
} FlatRec;

#endif /* _OL_FLATP_H */
