#ifndef	NOIDENT
#ident	"@(#)OpenLookP.h	1.2 SMI"
#ident	"@(#)oltemporary:OpenLookP.h	1.5"
#endif

#ifndef __OpenLookP_h__
#define __OpenLookP_h__

/*
 *************************************************************************
 *
 * Description:
 * 		This is the include file for the OPEN LOOK (TM - AT&T)
 *		routines which are private to the toolkit.
 * 
 **************************file*header************************************
 */

#include <Xol/OpenLook.h>

/*
 ************************************************************************
 *	Define True Constant Tokens
 *		Tokens that appear in this section should not have
 *		their values changed.
 ************************************************************************
 */

#define OL_VERSION		2
#define OL_REVISION		0
#define OlVersion		(OL_VERSION * 1000 + OL_REVISION)

#define OL_DEFAULT_POINT_SIZE	12
#define OL_DEFAULT_FONT_NAME	"lucida"

#define OL_NO_ITEM		(~0)
#define OL_MAX_VIRTUAL_MAPPINGS	6

/*************************************************************************
 *
 *	Macros
 */

#define Max(x, y)	(((x) > (y)) ? (x) : (y))
#define Min(x, y)	(((x) < (y)) ? (x) : (y))
#define AssignMax(x, y)  if ((y) > (x)) x = (y)
#define AssignMin(x, y)  if ((y) < (x)) x = (y)


						/* String length macro	*/

#define _OlStrlen(string)	((string) ? strlen((const char *)(string)) : 0)

				/* geometry-related convenience macros	*/

#define _OlScreenWidth(w)	(WidthOfScreen(XtScreenOfObject(w)))
#define _OlScreenHeight(w)	(HeightOfScreen(XtScreenOfObject(w)))
#define _OlWidgetWidth(w)	((w)->core.width + 2*(w)->core.border_width)
#define _OlWidgetHeight(w)	((w)->core.height + 2*(w)->core.border_width)

#ifndef	XA_CLIPBOARD
#define XA_CLIPBOARD(d)	XInternAtom(d, "CLIPBOARD", False)
#endif

			/* Define Mask Arg List structure and macro	*/

typedef struct {
	String		name;		/* resource name */
	XtArgVal	value;		/* resource value */
	int		rule;		/* comparision rule (see below) */

} MaskArg, *MaskArgList;

#define _OlSetMaskArg(m_arg, n, v, r) \
	((m_arg).name = (String) (n), (m_arg).value = (XtArgVal) (v), \
	(m_arg).rule = (int) (r))


/*************************************************************************
 *
 *	typedef's, enum's, struct's
 */

typedef struct {
	Cardinal	offset;
	String		name;
	XrmQuark	quark;
	OlDefine	type;
	OlDefine	direction;
} OlUnitType, *OlUnitTypeList;

typedef struct {
	Cardinal	id_type;
	XtPointer	id;
	String		tag;
	OlDefine	source_type;
	XtPointer	source;
} OlHelpInfo, *OlHelpInfoList;

typedef struct {
	int	place_holder;
} OlDynamicResource, *OlDynamicResourceList;

					/* Virtual mapping structure	*/

typedef struct {
	OlDefine	type;		/* type flag			*/
	unsigned int	modifiers;	/* modifier mask		*/
	unsigned int	detail;		/* mapping detail, e.g., keycode*/
	String		composed;	/* Name of virtual key that
					 * was used to compose this one	*/
} _OlVirtualMapping;

				/* ShellBehavior typedef used as argument to
				 * the XtNshellBehavior resource
				 */
typedef int	ShellBehavior;

#define OtherBehavior		0 /* Application-defined shell behavior */
#define BaseWindow		1 /* Shell is a plain base window */
#define PopupWindow		2 /* Shell is a plain, unpinned popup */
#define PinnedWindow		3 /* Shell is pinned up */
#define PinnedMenu		4 /* Shell is pinned menu window */
#define PressDragReleaseMenu	5 /* Menu shell in press-drag-release mode */
#define StayUpMenu		6 /* Menu shell in stay-up mode */
#define UnpinnedMenu		7 /* Shell is unpinned menu window */


/*************************************************************************
 *
 *	Private extern declarations
 */

extern Display *	toplevelDisplay;
extern XrmName		_OlApplicationName;

extern void
_OlAddVirtualMappings OL_ARGS((			
	String		/* mappings;	virtual token mappings	*/
));

extern void
_OlAppAddVirtualMappings OL_ARGS((			
	Widget,		/* widget;	widget id or NULL	*/
	String		/* mappings;	virtual token mappings	*/
));

	/* Beep the display only if Settings in Workspace permit it	*/
extern void
_OlBeepDisplay OL_ARGS((
	Widget,		/* widget;	- widget wanting beep	*/
	Cardinal	/* count;	- number of beeps	*/
));

		/* Composes an Arg list from a source Arg list and a
		 * source MaskArg list.					*/
extern void
_OlComposeArgList OL_ARGS((
	ArgList,	/* args;		- source Arg List	*/
	Cardinal,	/* num_args;		- number of source Args	*/
	MaskArgList,	/* mask_args;		- mask Arg List		*/
	Cardinal,	/* mask_num_args;	- number of mask Args	*/
	ArgList *,	/* dest_args;		- destination Arg List	*/
	Cardinal *	/* dest_num_args;	- number of dest Args	*/
));

			/* Copy an XtArgVal value into a destination of
			 * an arbitrary size.				*/
extern void
_OlCopyFromXtArgVal OL_ARGS((
	XtArgVal,		/* source;			*/
	char *,			/* destination;			*/
	Cardinal		/* size;			*/
));

			/* Copy information located at some address into
			 * an XtArgVal.					*/
extern void
_OlCopyToXtArgVal OL_ARGS((
	char *,		/* source;			*/
	XtArgVal *,	/* destination;			*/
	Cardinal	/* size;			*/
));

			/* Clear the background of a widget or a gadget	*/
extern void
_OlClearWidget OL_ARGS((
	Widget,		/* widget;	- widget/gadget id to clear	*/
	Boolean		/* exposures;	- generate exposure event ?	*/
));

				/* Converts string to XFontStruct for
				 * windowed/windowless objects		*/
extern void
_OlCvtStringToFontStruct OL_ARGS((
	XrmValue *,	/* args;	arguments needed for conversion	*/
	Cardinal *,	/* num_args;	the number of converion args	*/
	XrmValue *,	/* fromVal;	value to convert		*/
	XrmValue *	/* toVal;	returned converted value	*/
));

				/* Converts string to Gravity resource	*/
extern void
_OlCvtStringToGravity OL_ARGS((
	XrmValue *,	/* args;	arguments needed for conversion	*/
	Cardinal *,	/* num_args;	the number of converion args	*/
	XrmValue *,	/* fromVal;	value to convert		*/
	XrmValue *	/* toVal;	returned converted value	*/
));

				/* Converts string to OlDefine resource	*/
extern void
_OlCvtStringToOlDefine OL_ARGS((
	XrmValue *,	/* args;	arguments needed for conversion	*/
	Cardinal *,	/* num_args;	the number of converion args	*/
	XrmValue *,	/* fromVal;	value to convert		*/
	XrmValue *	/* toVal;	returned converted value	*/
));

				/* Converts string to Pixel for
				 * windowed/windowless objects		*/
extern void
_OlCvtStringToPixel OL_ARGS((
	XrmValue *,	/* args;	arguments needed for conversion	*/
	Cardinal *,	/* num_args;	the number of converion args	*/
	XrmValue *,	/* fromVal;	value to convert		*/
	XrmValue *	/* toVal;	returned converted value	*/
));

			/* Dump the virtual buttons/keys to a file	*/
extern void
_OlDumpVirtualMappings();
	/* FILE *		fp;		- destination file pointer	*/
	/* Boolean		long_form;	- True for long listing		*/

				/* Free font created by _OlGetFont() 	*/
extern void
_OlFreeFont OL_ARGS((
	Display *,	/* display;	- font's display		*/
	XFontStruct *	/* font_struct;	- XFontStruct returned by _OlGetFont*/
));

			/* Free XImages created with _OlGetImage()	*/
extern void
_OlFreeImage OL_ARGS((
	XImage *	/* ximage;	- XImage returned by _OlGetImage*/
));

		/* to retrieve appl title */
extern String
_OlGetApplicationTitle OL_ARGS((
	Widget		/* widget;	any widget in application	*/
));

	/* Font Loading & caching */
extern XFontStruct *
_OlGetFont OL_ARGS((
	Screen *,	/* screen;	- intended viewing screen id	*/
	int,		/* point_size;	- font point_size 		*/
	char *		/* name;	- raw font name or NULL		*/
));

	/* Image creation & caching */
extern XImage *
_OlGetImage OL_ARGS((
	Screen *,	/* screen;	- intended viewing screen id	*/
	int,		/* point_size;	- point_size of information	*/
	char *		/* name;	- raw bitmap name		*/
));

					/* single click damping factor	*/
extern Cardinal
_OlGetMouseDampingFactor OL_ARGS((
	Widget		/* widget;	widget id or NULL		*/
));

	/* Search up a widget tree until the shell is found		*/
extern Widget
_OlGetShellOfWidget OL_ARGS((
	Widget		/* widget;	start search here	*/
));

				/* Get the id of the Troll Widget	*/
extern Widget
_OlGetTrollWidget OL_NO_ARGS();

		/* Get the mappings for a virtual button or virtual key	*/
extern Cardinal
_OlGetVirtualMappings OL_ARGS((
	String,			/* name;	- virtual button name	*/
	_OlVirtualMapping *,	/* mapping_list;- application-supplied
						  array to fill in	*/
	Cardinal		/* list_length;	- number of array elements*/
));

	/* Checks to see if a widget can grab the pointer */ 
extern Boolean
_OlGrabPointer OL_ARGS((
	Widget,		/* widget; 	- widget requesting grab	*/
	Bool,		/* event_mask;	- */
	unsigned int,	/* event_mask;	- */
	int,		/* pointer_mode; - */
	int,		/* keyboard_mode; - */
	Window,		/* confine_to; - */
	Cursor,		/* cursor; - */
	Time		/* time; - */
));

						/* Grab the server	*/
extern Boolean
_OlGrabServer OL_ARGS((
	Widget		/* widget; 	- widget requesting grab	*/
));

			/* Initialize the virtual button/key code	*/
extern void
_OlInitVirtualMappings OL_ARGS((
	Widget		/* widget;		application widget	*/
));

	/* Sees if a modifier mask and button number is a virtual
	 * button mapping						*/
extern Boolean
_OlIsVirtualButton OL_ARGS((
	String,		/* name;    	- virtual button name		*/
	unsigned int,	/* state;      	- modifier mask			*/
	unsigned int,	/* button;     	- button number or zero		*/
	Boolean		/* exclusive;  	- True for perfect match	*/
));

	/* Sees if an XEvent is either a virtual button or virtual Key
	 * event.							*/
extern Boolean
_OlIsVirtualEvent OL_ARGS((
	char *,		/* name;    	- Virtual Button/key name	*/
	XEvent *,	/* xevent;	- XEvent to be checked		*/
	Boolean		/* exclusive;  	- True for perfect match	*/
));

	/* Sees if a modifier mask and a keycode is a virtual key
	 * mapping						*/
extern Boolean
_OlIsVirtualKey OL_ARGS((
	String,		/* name;    	- virtual key name		*/
	unsigned int,	/* state;      	- modifier mask			*/
	KeyCode,	/* keycode;     - button from XEvent or NULL	*/
	Boolean		/* exclusive;  	- True for perfect match	*/
));

	/* Expose the event handler that pops up the help widget	*/
extern void
_OlPopupHelpTree OL_ARGS((
	Widget,		/* w;		- shell widget id		*/
	XtPointer,	/* client_data;	- unused			*/
	XEvent *	/* xevent;	- help XEvent message		*/
));

		/* Add an OlDefine type to it's converter's database	*/
extern void
_OlRegisterOlDefineType OL_ARGS((
	XtAppContext,	/* app_context;	Application Context or NULL	*/
	String,		/* name;	OlDefine's name			*/
	OlDefine	/* value;	OlDefine's value		*/
));

extern Boolean
_OlSelectDoesPreview OL_ARGS((
	Widget		/* widget;	widget id or NULL	*/
));

					/* to store appl title	*/
extern void
_OlSetApplicationTitle OL_ARGS((
	String		/* title;	/* set title to this	*/
));

	/* Set a menu's default	*/
extern void
_OlSetMenuDefault OL_ARGS((
	Widget		/* widget;	- widget wanting to be default	*/
));

	/* append Atom to WM_PROTOCOLS property of toplevel window */
extern void
_OlSetWMProtocol OL_ARGS((
	Display *,	/* display;	display			*/
	Window,		/* window;	window (toplevel)	*/
	Atom		/* atom;	atom to append		*/
));

					/* Used to ungrab the pointer	*/
extern void
_OlUngrabPointer OL_ARGS((
	Widget		/* widget;	- widget wishing pointer ungrab	*/
));

					/* Used to ungrab the server	*/
extern void
_OlUngrabServer OL_ARGS((
	Widget		/* widget;	- widget wishing server ungrab	*/
));

				/* get gadget child of composite	*/
extern Widget
_OlWidgetToGadget OL_ARGS((
	Widget,			/* w;	Composite		*/
	Position,		/* x;	Position over Composite	*/
	Position		/* y;	Position over Composite	*/
));

#endif /* __OpenLookP_h__ */
