#ifndef NOIDENT
#ident	"@(#)s_sampler.c	1.4 SMI"
#endif

/************************************************************************/
/*	OPEN LOOK WIDGET SAMPLER : prototype widgets and use of form	*/
/*									*/
/*	Copyright (c) 1989 AT&T						*/
/*	Copyright (c) 1988 Hewlett-Packard Company 			*/
/*	Copyright (c) 1988 Massachusetts Institute of Technology	*/
/************************************************************************/

#include <stdio.h>

#include <X11/Intrinsic.h>		/* required for OPEN LOOK */
#include <X11/StringDefs.h>
#include <Xol/OpenLook.h>

#include <Xol/AbbrevMenu.h>		/* one per widget used */
#include <Xol/BulletinBo.h>
#include <Xol/Button.h>
#include <Xol/Caption.h>
#include <Xol/CheckBox.h>
#include <Xol/ControlAre.h>
#include <Xol/Exclusives.h>
#include <Xol/FooterPane.h>
#include <Xol/Form.h>
#include <Xol/MenuButton.h>
#include <Xol/Nonexclusi.h>
#include <Xol/Notice.h>
#include <Xol/OblongButt.h>
#include <Xol/PopupWindo.h>
#include <Xol/RectButton.h>
#include <Xol/Scrollbar.h>
#include <Xol/ScrolledWi.h>
#include <Xol/ScrollingL.h>
#include <X11/Shell.h>
#include <Xol/Slider.h>
#include <Xol/StaticText.h>
#include <Xol/Stub.h>
#include <Xol/Text.h>
#include <Xol/TextField.h>

#define NCOLORS 16
#define NCASES 10
#define NDATA 2 
#define XOFFSET 15
#define YOFFSET 20

static void GetColors();		/* forward declarations */
static void SetPosition();
static void DrawAndPrintCB();
static void SimpleEventHandler();

XtPointer stubCBData[NDATA];

static XColor	this_red, exact_red, this_blue, exact_blue, 
	this_purple,exact_purple, this_green, exact_green, 
	this_yellow,exact_yellow, this_orange,exact_orange,
	this_white,exact_white,this_skyblue,exact_skyblue;

static int i, n[NCASES];

static Display	*display;
Widget		generic_widget;
static Widget	toplevel,
		form,
		abbmenubutton,abbmenupane,
		controlarea,cabutton,
		exclusives,
		bulletinboard,
		caption,capbutton,
		footerpanel,
		menubutton,menupane,mexclusives,
		nonexclusives,nebutton,
		noticeshell,noticebox,noticetext,
		popupshell,popupbutton,popupca1,popupca2,popupfooter,
		scrollbar,
		scrolledwindow,
		slbutton1,slbutton2,scrollinglist,
		slider,
		stub,
		statictext,
		text,
		textfield,textfieldprompt,
		titlebutton;

/* R E S O U R C E S  A N D  C A L L B A C K S  F O R  W I D G E T S */

/* GENERIC RESOURCES AND CALLBACK FOR SIMPLE WIDGET CASES */

static Arg genericARGS[] = {
					/* form attachment resources only */
	{ XtNxRefWidget, NULL },
	{ XtNyRefWidget, NULL },
	{ XtNxOffset,(XtArgVal) XOFFSET },
	{ XtNyOffset,(XtArgVal) YOFFSET },
	{ XtNxAddWidth,(XtArgVal) TRUE },
	{ XtNyAddHeight,(XtArgVal) TRUE },
};

/*** Each widget passes its index as clientData and thus maps to 
   its own printf statement ***/

static void genericCB(widget,clientData,callData)
	Widget widget;
	XtPointer clientData, callData;
{
	int n = * (int *)clientData;

	switch(n) {
		case 0:
			printf("CONTROLAREA callback for CONTROL button \n");
			break;
		case 1:
			printf("CONTROLAREA callback for AREA button \n");
			break;
		case 2:
			printf("NONEXCLUSIVES callback \n");
			break;
		case 3:
			printf("POPUP apply callback \n");
			break;
		case 4:
			printf("POPUP setdefaults callback \n");
			break;
		case 5:
			printf("POPUP reset callback \n");
			break;
		case 6:
			printf("POPUP resetfactory callback \n");
			break;
	}
}

/* CHECKBOX */

static void checkboxCB(widget,clientData,callData)
	Widget widget;
	XtPointer clientData, callData;
{
	Arg arg;
						/* get the current value */
	XtSetArg(arg,XtNposition,(XtArgVal) 0);
	XtGetValues(widget,&arg,1);
						/* toggle it */
	if(arg.value == OL_LEFT) {
		XtSetArg(arg,XtNposition,OL_RIGHT);
		XtSetValues(widget,&arg,1);
	}
	else if(arg.value == OL_RIGHT) {
		XtSetArg(arg,XtNposition,OL_LEFT);
		XtSetValues(widget,&arg,1);
	}
}

/* EXCLUSIVES */

static void exclusivesCB(widget,clientData,callData)
	Widget widget;
	XtPointer clientData, callData;

{
	Arg arg;
	int button;
	Widget parent = XtParent(widget);

	button = * (int *)clientData;
	printf("EXCLUSIVES callback for button %d\n",button);

					/* get the current layout */
	XtSetArg(arg,XtNlayoutType,(XtArgVal) 0);
	XtGetValues(parent,&arg,1);

						/* toggle it */
	if(button == 2 && arg.value == OL_FIXEDROWS) {
		XtUnmapWidget(parent);		/* makes display better */
		XtSetArg(arg,XtNlayoutType,OL_FIXEDCOLS);
		XtSetValues(parent,&arg,1);
		XtMapWidget(parent); 
	}
	else if(button == 3 && arg.value == OL_FIXEDCOLS) {
		XtUnmapWidget(parent);		/* makes display better */
		XtSetArg(arg,XtNlayoutType,OL_FIXEDROWS);
		XtSetValues(parent,&arg,1);
		XtMapWidget(parent); 
	}
}

/* MENU */

static void menuSelectCB(widget, clientData, callData)
	Widget widget;
	XtPointer clientData, callData;
{
	printf("Button %d selected\n", *clientData);
}

static void menuUnselectCB(widget, clientData, callData)
	Widget widget;
	XtPointer clientData, callData;
{
	printf("Button %d unselected\n", *clientData);
}

/* NONEXCLUSIVES */

static void nonexclusivesCB(widget,clientData,callData)
	Widget widget;
	XtPointer clientData, callData;
{
	Widget parent = (Widget) clientData;
	Arg arg;

				/* if two buttons, create a third */
	if(nebutton == (Widget) 0) {
		nebutton = XtCreateManagedWidget
			("FEWER",rectButtonWidgetClass,parent,NULL,0);
	XtAddCallback(nebutton,XtNselect,
				nonexclusivesCB,(XtPointer) nonexclusives);

				/* scrollbar to right of nonexclusives now */
		XtSetArg(arg,XtNxRefWidget,(XtArgVal) nonexclusives);
		XtSetValues(scrollbar,&arg,1);
	}
				/* if all three, delete the third */
	else {
		XtDestroyWidget(nebutton);
		nebutton = (Widget) 0;

				/* scrollbar to right of scrollinglist now */
		XtSetArg(arg,XtNxRefWidget,(XtArgVal) scrollinglist);
		XtSetValues(scrollbar,&arg,1);
	}
}

/* NOTICE */

static Arg noticeARGS[] = {
	{XtNtextArea, (XtArgVal)&noticetext},
	{XtNcontrolArea, (XtArgVal)&noticebox},
};

static void noticeCB1(w, callData,clientData) 
	Widget w;	/* w = emanating button: where notice does popup */
	XtPointer callData, clientData;
{
	Arg arg;

	XtSetArg(arg, XtNemanateWidget, (XtArgVal)w);
	XtSetValues(noticebox, &arg, 1);
			/* freeze application until user makes choice */
	XtPopup(noticeshell, XtGrabExclusive); 
}

static void noticeCB2(w, client_data, call_data)
	Widget w;
	XtPointer client_data, call_data;
{
	printf("ADIOS !!! \n");
	exit(0);
}

/* POPUP */

static XtCallbackRec popup_applyCBR[] = {
	{ genericCB, (XtPointer) &n[3] },
	{ NULL, NULL },
};

static XtCallbackRec popup_setdefaultsCBR[] = {
	{ genericCB, (XtPointer) &n[4] },
	{ NULL, NULL },
};

static XtCallbackRec popup_resetCBR[] = {
	{ genericCB, (XtPointer) &n[5] },
	{ NULL, NULL },
};

static XtCallbackRec popup_resetfactoryCBR[] = {
	{ genericCB, (XtPointer) &n[6] },
	{ NULL, NULL },
};

static void popupCB1(w,clientData,callData)
	Widget w;
	XtPointer clientData, callData;
{
	XtPopup(clientData,XtGrabNone);
}

/* SCROLLINGLIST */

static OlListToken  (*scrollinglistADDfn)();
static OlListItem sl_listitem[10];

static void scrollinglistCB(w,clientData,callData)
	Widget w;
	XtPointer clientData;
	OlListToken callData;
{
	OlListToken token = callData;
	OlListItem *selected_listitem;
	Arg arg;

		/* this supplied macro identifies the item selected */

	selected_listitem = OlListItemPointer(token);

	printf("SCROLLING LIST ITEM SELECTED: %s\n",selected_listitem->label);
}

/* SCROLLBAR */

static Arg scrollbarARGS[] = {

						/* widget resources */
	{ XtNbackground,NULL },
	{ XtNforeground,NULL },
	{ XtNproportionLength, 1 },
						/* form resources */
	{ XtNyResizable,(XtArgVal) TRUE },
	{ XtNyAttachBottom,(XtArgVal)TRUE },
	{ XtNxAttachRight,(XtArgVal) TRUE },
	{ XtNxVaryOffset,(XtArgVal) TRUE },
};

/*** this function is used by all three scrollbars as an economy ***/

static void scrollbarCB(w, clientData, callData)
    Widget w;
    XtPointer clientData;
    OlScrollbarVerify *callData;
{
	int n = *clientData;

	callData->ok = TRUE;

	switch(n) {
		case 0:
			printf("Form");
			break;
		case 1:
			printf("Scrolledwindow Vertical");
			break;
		case 2:
			printf("Scrolledwindow Horizontal");
			break;
	}
    printf( " scrollbar moved to %d%%\n", callData->new_location );
}

/* SCROLLEDWINDOW */

static XtCallbackRec	scrolledwindowCBR1[] =
{
  {scrollbarCB, (XtPointer) &n[1]},
  {NULL, (XtPointer) NULL}
};

static XtCallbackRec	scrolledwindowCBR2[] =
{
  {scrollbarCB, (XtPointer) &n[2]},
  {NULL, (XtPointer) NULL}
};

static char	swstring[] = 

"This text is in a\n\
statictext widget\n\
in a scrolled window.\n\
You can move it up\n\
and down or left\n\
and right with the\n\
two scrollbars.\n";

/* SLIDER */

static void sliderCB(widget, clientData, callData)
	Widget widget;
	XtPointer clientData, callData;
{
	Arg arg;

	arg.value = (XtArgVal) *(int *)callData;  /* slider returns current value */
	XtSetArg(arg,XtNbackground,arg.value);
	XtSetValues(statictext,&arg,1);
	XtSetValues(abbmenubutton,&arg,1);
}

/* TEXTFIELD */

static void textfieldCB(w,clientData,callData)
	Widget w;
	XtPointer clientData;
	OlTextFieldVerify *callData;
{
	Arg arg;

	printf("TEXTFIELD User Input: %s\n",callData->string);
	XtSetArg(arg,XtNstring,(XtArgVal) "");
	XtSetValues(textfield,&arg,1);
}

/* TITLEBUTTON */

static Arg titlebuttonARGS[] = {
						/* widget resources */

	{ XtNbackground, NULL  },
	{ XtNforeground, NULL },
	{ XtNlabelJustify,(XtArgVal) OL_CENTER },

						/* form resources */
	{ XtNxResizable,(XtArgVal) TRUE },		
	{ XtNxAttachRight,(XtArgVal) TRUE },
};

/************************************************************************
 		P R O G R A M M E R   I N T E R F A C E 		

   WIDGET TREE: 

	TOPLEVEL 
	FORM
	OPEN LOOK WIDGETS, ALL CHILDREN OF FORM

/************************************************************************/

int main(argc,argv)
	unsigned int argc;
	char **argv;
{

Arg arg[10];
int i, itoadiff,ncolors1;

/******************** INITIALIZE THE ENVIRONMENT ********************/

toplevel = OlInitialize("OL_DEMO","ol_demo",NULL,0,&argc, argv);
display = XtDisplay(toplevel);
GetColors(display);
for(i=0;i<NCASES;i++)		/* index array for generic callback function */
	n[i] = i;

/*** MAKE ALL THE WIDGETS TO GET WIDGET ID'S; DO PLACEMENT LAST ***/

/*** FORM: must be made first ***/

i = 0;
XtSetArg(arg[i],XtNbackground,(XtArgVal) this_skyblue.pixel);i++;

form = XtCreateManagedWidget("form",formWidgetClass,toplevel,arg,i);

/*** TITLE: use a generic button (OL_LABEL) ***/

titlebuttonARGS[0].value = (XtArgVal) this_skyblue.pixel; 
titlebuttonARGS[1].value = (XtArgVal) this_red.pixel; 

titlebutton = XtCreateManagedWidget("*** OPENLOOK (TM) WIDGETS ***",
	buttonWidgetClass,form, titlebuttonARGS,XtNumber(titlebuttonARGS));

/*** CONTROLAREA: and buttons ***/

controlarea = XtCreateManagedWidget("controlarea",
	controlAreaWidgetClass,form,NULL,0);

generic_widget =
 XtCreateManagedWidget("CONTROL",oblongButtonWidgetClass,controlarea,NULL,0);
 XtAddCallback(generic_widget,XtNselect,genericCB,(XtPointer) &n[0]);

generic_widget =
 XtCreateManagedWidget("AREA",oblongButtonWidgetClass,controlarea,NULL,0);
 XtAddCallback(generic_widget,XtNselect,genericCB,(XtPointer) &n[1]);

i = 0;
XtSetArg(arg[0],XtNbackground,(XtArgVal)this_red.pixel); i++;

cabutton =
 XtCreateManagedWidget("EXIT",oblongButtonWidgetClass,controlarea,arg,1);

/*** NOTICE BOX: attach to the exit button of the control area ***/

	/* attach notice popup callback to "EXIT" controlarea button */

	XtAddCallback(cabutton, XtNselect, noticeCB1,NULL);

	/* create notice popup shell */

	noticeshell = XtCreatePopupShell("notice", noticeShellWidgetClass,
		cabutton, NULL, 0);

	/* get the widget ids of noticebox and textarea of noticebox */

	XtGetValues(noticeshell, noticeARGS, XtNumber(noticeARGS));

	/* add text to text area of noticebox */
 
	i = 0;
	XtSetArg(arg[i],XtNstring,(XtArgVal)"NOTICE WIDGET:\n Confirm EXIT");
	i++;
	XtSetValues(noticetext,arg,i);

	/* add two buttons to noticebox */

	/* this button has an exit() callback */

	XtSetArg(arg[0],XtNbackground,(XtArgVal)this_red.pixel); i++;
	generic_widget = XtCreateManagedWidget("OKAY", 
		oblongButtonWidgetClass,noticebox,arg,i);
	XtAddCallback(generic_widget, XtNselect,noticeCB2,noticebox);

	/* this button is a no-op that merely pops down the notice widget */

	XtCreateManagedWidget("CANCEL",oblongButtonWidgetClass,noticebox,NULL,0);

/*** EXCLUSIVES: with three buttons ***/

exclusives = XtCreateManagedWidget("exclusives",
	exclusivesWidgetClass,form,NULL,0);

i = 0;
XtSetArg(arg[0],XtNlayoutType,(XtArgVal) OL_FIXEDCOLS); i++;
XtSetValues(exclusives,arg,i);

i = 0;
XtSetArg(arg[0],XtNlabel,(XtArgVal) "EXCLUSIVES"); i++;

generic_widget = XtCreateManagedWidget("rbutton",
	rectButtonWidgetClass,exclusives,arg,i);
XtAddCallback(generic_widget,XtNselect,exclusivesCB,(XtPointer) &n[1]);

generic_widget = XtCreateManagedWidget("VERTICAL",
	rectButtonWidgetClass,exclusives,NULL,0);
XtAddCallback(generic_widget,XtNselect,exclusivesCB,(XtPointer) &n[2]);

generic_widget = XtCreateManagedWidget("HORIZONTAL",
	rectButtonWidgetClass,exclusives,NULL,0);
XtAddCallback(generic_widget,XtNselect,exclusivesCB,(XtPointer) &n[3]);

/*** BULLETINBOARD: with a checkbox in it ***/

bulletinboard = XtCreateManagedWidget("bboard",
	bulletinBoardWidgetClass,form,NULL,0);
i = 0;
XtSetArg(arg[i],XtNbackground, (XtArgVal) this_blue.pixel); i++;
XtSetArg(arg[i],XtNborderWidth, (XtArgVal) 2); i++;
XtSetArg(arg[i],XtNborderColor, (XtArgVal) this_yellow.pixel); i++;
XtSetValues(bulletinboard,arg,i);

i = 0;
XtSetArg(arg[i],XtNfontColor, (XtArgVal) this_white.pixel); i++;
XtSetArg(arg[i],XtNforeground, (XtArgVal) this_white.pixel); i++;

generic_widget = XtCreateManagedWidget("CHECKBOX IN BULLETINBOARD",
	checkBoxWidgetClass,bulletinboard, arg,i);

XtAddCallback(generic_widget,XtNselect,checkboxCB,NULL);

/*** TEXT WIDGET ***/

i = 0;
XtSetArg(arg[i],XtNstring, (XtArgVal) "TEXT WIDGET: EDIT ME"); i++;
XtSetArg(arg[i],XtNfontColor, (XtArgVal) this_red.pixel); i++;
XtSetArg(arg[i],XtNheight, (XtArgVal) 22); i++;

text = XtCreateManagedWidget("text",textWidgetClass,form,arg,i);

/*** TEXTFIELD ***/

/* make a label/prompt */

i = 0;
XtSetArg(arg[i],XtNbackground, (XtArgVal) this_blue.pixel); i++; 
XtSetArg(arg[i],XtNforeground, (XtArgVal) this_white.pixel); i++; 
textfieldprompt = XtCreateManagedWidget("TEXTFIELD WIDGET: CLICK & TYPE BELOW",
	buttonWidgetClass,form,arg,i);

i = 0;
XtSetArg(arg[i],XtNfontColor, (XtArgVal) this_blue.pixel); i++; 
XtSetArg(arg[i],XtNinputFocusColor, (XtArgVal) this_blue.pixel); i++; 
textfield = XtCreateManagedWidget("textfield",textFieldWidgetClass,form,arg,i);

/* callback to "read" user input when <return> typed; 
   note that also called when input focus is lost */

XtAddCallback(textfield,XtNverification,textfieldCB,NULL);

/*** FOOTER PANEL: with a slider and static text ***/

footerpanel = XtCreateManagedWidget("footerpanel",
	footerPanelWidgetClass,form,NULL,0);

i = 0;
XtSetArg(arg[i],XtNstring,(XtArgVal) "FOOTER WITH SLIDER & STATIC TEXT (ME)");
i++;

statictext = XtCreateManagedWidget("statictext",
	staticTextWidgetClass,footerpanel,arg,i);

/*
 The slider will be used to change the background of the statictext widget: 
 since there are NCOLORS, the slider range is 0 to (N-1) 
 with a granularity of 1; see the sliderCB function for the rest
 of the code/functionality.
*/

ncolors1 = NCOLORS-1;

i = 0;
XtSetArg(arg[i],XtNorientation, (XtArgVal) OL_HORIZONTAL); i++;
XtSetArg(arg[i],XtNsliderMax, (XtArgVal) ncolors1); i++;
XtSetArg(arg[i],XtNgranularity, (XtArgVal) 1); i++;

slider = XtCreateManagedWidget("slider",sliderWidgetClass,footerpanel,arg,i);
XtAddCallback(slider,XtNsliderMoved,sliderCB,NULL);

/*** NONEXCLUSIVES: with two buttons ***/

i = 0;
XtSetArg(arg[i],XtNlayoutType,(XtArgVal) OL_FIXEDROWS); i++;
XtSetArg(arg[i],XtNmeasure,(XtArgVal) 1); i++;

nonexclusives = XtCreateManagedWidget("nonexclusives",
	nonexclusivesWidgetClass,form,arg,i);

generic_widget = XtCreateManagedWidget("NONEXCLUSIVES SET:",
	rectButtonWidgetClass,nonexclusives,NULL,0);
XtAddCallback(generic_widget,XtNselect,genericCB,(XtPointer) &n[2]);

generic_widget = XtCreateManagedWidget("MORE",
	rectButtonWidgetClass,nonexclusives,NULL,0);
XtAddCallback(generic_widget,XtNselect,nonexclusivesCB,(XtPointer)nonexclusives);

/***
TO MAKE A MENU:

Widget Tree for creating menu:

		form ( parent widget )
			|
		menubutton ( visible/mouse sensitive symbol for menu )
			|
		pane ( used for placement of menu button set )
			|
		exclusives ( button manager for this menu )
			|
		       / \
		     /	   \
	        button1   button2
***/

/*
 Create a menubutton and menu with a pushpin.
*/
	i = 0;
	XtSetArg(arg[i],XtNpushpin,(XtArgVal) OL_OUT); i++;
	XtSetArg(arg[i],XtNlabelType,(XtArgVal) OL_STRING); i++;
	XtSetArg(arg[i],XtNlabelJustify,(XtArgVal) OL_LEFT); i++;
	XtSetArg(arg[i],XtNrecomputeSize,(XtArgVal) TRUE); i++;

	menubutton = XtCreateManagedWidget("MENU BUTTON",
		menuButtonWidgetClass,form,arg,i);

/* 
Get the Widget id of the menupane of the menubutton. 
*/
	i = 0;
	XtSetArg(arg[i], XtNmenuPane,(XtArgVal) &menupane); i++;
	XtGetValues(menubutton, arg, i);

/* 
Make the exclusives widget for the menupane, column format.
*/
	i = 0;
	XtSetArg(arg[i],XtNlayoutType,(XtArgVal) OL_FIXEDCOLS); i++;
	XtSetArg(arg[i],XtNmeasure,(XtArgVal) 1); i++;

	mexclusives = XtCreateManagedWidget("exclusives",
			exclusivesWidgetClass,menupane,arg,i);

/* 
Make two rectangular buttons in the exclusives, 
with select and unselect callbacks.
*/
	generic_widget = XtCreateManagedWidget("ONE",
		rectButtonWidgetClass,mexclusives,NULL,0);
	XtAddCallback(generic_widget,XtNselect,menuSelectCB,(XtPointer) &n[1]);
	XtAddCallback(generic_widget,XtNunselect,menuUnselectCB,(XtPointer)&n[1]);

	generic_widget = XtCreateManagedWidget("TWO",
		rectButtonWidgetClass,mexclusives,NULL,0);
	XtAddCallback(generic_widget,XtNselect,menuSelectCB,(XtPointer) &n[2]);
	XtAddCallback(generic_widget,XtNunselect,menuUnselectCB,(XtPointer)&n[2]);

/*
END OF MENU
*/

/*** ABBREVIATED MENU BUTTON: similar to above ***/

i = 0;
XtSetArg(arg[i], XtNbackground,(XtArgVal) this_green.pixel); i++;

abbmenubutton = XtCreateManagedWidget("abbmenubutton",
	abbrevMenuButtonWidgetClass,form,arg,i);

/*** get the Widget ID of the menupane of the abbreviated menu button ***/

	i = 0;
	XtSetArg(arg[i], XtNmenuPane,(XtArgVal) &abbmenupane); i++;
	XtGetValues(abbmenubutton, arg, i);

	XtCreateManagedWidget("ABBREVIATED MENU BUTTON",
		oblongButtonWidgetClass,abbmenupane,arg,i);

/*** POPUP ***/

popupbutton = XtCreateManagedWidget("POPUP",
		oblongButtonWidgetClass,form,NULL,0);

/* 
   Make the popup shell first:
   NOTE: callbacks must be set for creation of automatic buttons.
   In this example, mapping is to one function but does not have to be. 
*/
	i = 0;
	XtSetArg(arg[i],XtNreset,(XtArgVal) popup_resetCBR); i++;
	XtSetArg(arg[i],XtNapply,(XtArgVal) popup_applyCBR); i++;
	XtSetArg(arg[i],XtNresetFactory,(XtArgVal) popup_resetfactoryCBR);i++;
	XtSetArg(arg[i],XtNsetDefaults, (XtArgVal) popup_setdefaultsCBR);i++;
	popupshell = XtCreatePopupShell("POPUPSHELL", 
		popupWindowShellWidgetClass,popupbutton,arg,i);

/* 
   Add callback to popup button now that we have popupshell widget ID. 
*/
	XtAddCallback(popupbutton, XtNselect, popupCB1, popupshell);

/*
 The popup window automatically makes three children: upper and
 lower control areas and footer: get widget IDs to populate them.
*/
	i = 0;
	XtSetArg(arg[i], XtNupperControlArea, &popupca1); i++;
	XtSetArg(arg[i], XtNlowerControlArea, &popupca2); i++;
	XtSetArg(arg[i], XtNfooterPanel, &popupfooter); i++;
	XtGetValues (popupshell, arg, i);

/* 
  Populate popup upper control area.
*/
	XtSetArg(arg[0],XtNlayoutType,OL_FIXEDROWS);
	XtSetValues(popupca1,arg,1);

	XtCreateManagedWidget("UPPER & LOWER",
		oblongButtonWidgetClass,popupca1,NULL,0);
	XtCreateManagedWidget("CONTROL",
		oblongButtonWidgetClass,popupca1,NULL,0);
	XtCreateManagedWidget("AREAS",
		oblongButtonWidgetClass,popupca1,NULL,0);

/* 
  Populate popup lower controlarea.
*/
	XtCreateManagedWidget("ADDED BUTTON",oblongButtonWidgetClass, 
		popupca2, NULL,0);

/* 
  Add text to the popup footer.
*/
	i = 0;
	XtSetArg(arg[i], XtNborderWidth, 0); i++;
	XtSetArg(arg[i], XtNstring, "FOOTER WIDGET IN POPUP"); i++;

	XtCreateManagedWidget("footer",staticTextWidgetClass,popupfooter,arg,i);

/*
END OF POPUP
*/

/*** CAPTION ***/

i = 0;
XtSetArg(arg[i],XtNposition,(XtArgVal) OL_TOP); i++;
XtSetArg(arg[i],XtNalignment,(XtArgVal) OL_CENTER); i++;

caption = XtCreateManagedWidget("CAPTION WIDGET",
	captionWidgetClass,form,arg,i);

capbutton = XtCreateManagedWidget("*** RAINBOW ***",
	oblongButtonWidgetClass,caption,NULL,0);

/*** STUB WIDGET: this will be used as a drawing board ***/

i = 0;
XtSetArg(arg[i], XtNheight, 60); i++;
XtSetArg(arg[i], XtNwidth, 100); i++;
XtSetArg(arg[i], XtNbackground, this_blue.pixel); i++;

stub = XtCreateManagedWidget("stub",stubWidgetClass,form,arg,i);

	/* now that stub ID set, add callback to caption button 
	/* and an eventhandler to refresh the window when necessary */

 stubCBData[0] = (XtPointer) stub;
 stubCBData[1] = (XtPointer) TRUE; /* this will toggle drawing on and off */
 XtAddCallback(capbutton,XtNselect,DrawAndPrintCB,(XtPointer) stubCBData);

 				/* this will refresh screen only */
 XtAddEventHandler(stub,ExposureMask,FALSE,SimpleEventHandler,NULL);

/*** SCROLLED WINDOW ***/

i = 0;
XtSetArg(arg[i], XtNheight, 150); i++;
XtSetArg(arg[i], XtNwidth, 150); i++;
XtSetArg(arg[i], XtNvSliderMoved, scrolledwindowCBR1); i++;
XtSetArg(arg[i], XtNhSliderMoved, scrolledwindowCBR2); i++;
XtSetArg(arg[i], XtNvStepSize, 20); i++;
XtSetArg(arg[i], XtNhStepSize, 20); i++;

scrolledwindow = XtCreateManagedWidget("scrolledwindow",
	scrolledWindowWidgetClass,form,arg,i);

/*** make a statictext widget in the scrolled window
	to move around with the scrollbars ***/

  i = 0;
  XtSetArg(arg[i], XtNheight, 150); i++;
  XtSetArg(arg[i], XtNstring, swstring); i++;

  XtCreateManagedWidget("statictext",
	staticTextWidgetClass,scrolledwindow,arg,i);

/*** SCROLLINGLIST ***/

/*** use two generic buttons to label the scrollinglist ***/

i = 0;
XtSetArg(arg[i],XtNbackground, (XtArgVal) this_blue.pixel); i++; 
XtSetArg(arg[i],XtNforeground, (XtArgVal) this_white.pixel); i++; 
slbutton1 = XtCreateManagedWidget("SCROLLING",
	buttonWidgetClass,form,arg,i);

i = 0;
XtSetArg(arg[i],XtNbackground, (XtArgVal) this_blue.pixel); i++; 
XtSetArg(arg[i],XtNforeground, (XtArgVal) this_white.pixel); i++; 
slbutton2 = XtCreateManagedWidget("LIST:  ",
	buttonWidgetClass,form,arg,i);

i = 0;
XtSetArg(arg[i], XtNtitle, (XtArgVal) "SCROLLINGLIST"); i++;
XtSetArg(arg[i], XtNviewHeight, (XtArgVal) 7); i++;

scrollinglist = XtCreateManagedWidget("scrollinglist",
	scrollingListWidgetClass,form,arg,i);

/*** get the pointer to the scrollinglist function for adding items ***/

i = 0;
XtSetArg(arg[i], XtNapplAddItem, (XtArgVal) &scrollinglistADDfn); i++;
XtGetValues(scrollinglist,arg,i);

itoadiff = (int) '1' - 1;

for(i=0;i<NCASES;i++) {
	sl_listitem[i].label_type = (OlDefine) OL_STRING;
	sl_listitem[i].label = strcpy(XtMalloc((unsigned)6),"ITEM  ");
	sl_listitem[i].label[5] = n[i] +itoadiff;
	(*scrollinglistADDfn) (scrollinglist,NULL,NULL,sl_listitem[i]); 
}
			/* callback to be invoked when user selects an item */

	XtAddCallback(scrollinglist,XtNuserMakeCurrent,scrollinglistCB,NULL);

/*** SCROLLBAR: attach to the right & bottom of the form ***/

scrollbarARGS[0].value = (XtArgVal) this_orange.pixel; 
scrollbarARGS[1].value = (XtArgVal) this_white.pixel; 

scrollbar = XtCreateManagedWidget("scrollbar",
	scrollbarWidgetClass,form, scrollbarARGS,XtNumber(scrollbarARGS));

XtAddCallback(scrollbar,XtNsliderMoved,scrollbarCB,(XtPointer) &n[0]);

/*** WITH WIDGET ID'S SET, POSITION ALL WIDGETS ON FORM ***/
/*** Note: set every new widget in terms of those already placed on form ***/

genericARGS[2].value = (XtArgVal) 0;
genericARGS[3].value = (XtArgVal) 0;
genericARGS[4].value = (XtArgVal) FALSE;
genericARGS[5].value = (XtArgVal) FALSE;
SetPosition(titlebutton,form,form);	/* titlebutton is special case */
genericARGS[2].value = (XtArgVal) XOFFSET;	/* restore */
genericARGS[3].value = (XtArgVal) YOFFSET;
genericARGS[4].value = (XtArgVal) TRUE;
genericARGS[5].value = (XtArgVal) TRUE;

SetPosition(controlarea,form,titlebutton);
SetPosition(menubutton,form,controlarea);
SetPosition(abbmenubutton,menubutton,controlarea);
SetPosition(bulletinboard,form,menubutton);
SetPosition(text,form,bulletinboard);
SetPosition(textfieldprompt,form,text);

genericARGS[3].value = (XtArgVal) 0;
SetPosition(textfield,form,textfieldprompt); /* another spacial case */
genericARGS[3].value = (XtArgVal) YOFFSET;

SetPosition(footerpanel,form,textfield);
SetPosition(exclusives,form,footerpanel);
SetPosition(nonexclusives,popupbutton,titlebutton);
SetPosition(popupbutton,abbmenubutton,controlarea);
SetPosition(caption,popupbutton,nonexclusives);
SetPosition(stub,popupbutton,caption);
genericARGS[3].value = (XtArgVal) 0;
SetPosition(slbutton1,caption,caption);   /* another special case */
genericARGS[3].value = (XtArgVal) YOFFSET;

genericARGS[3].value = (XtArgVal) 0;
SetPosition(slbutton2,caption,slbutton1); /* another special case */
genericARGS[3].value = (XtArgVal) YOFFSET;
genericARGS[2].value = (XtArgVal) (2 * XOFFSET);	/* restore */

SetPosition(scrolledwindow,popupbutton,stub);
SetPosition(scrollinglist,caption,slbutton2);

genericARGS[3].value = (XtArgVal) 0;
SetPosition(scrollbar,scrollinglist,titlebutton); /* another special case */
genericARGS[3].value = (XtArgVal) YOFFSET;

/* TURN CONTROL OVER TO THE INTRINSICS AND OPEN LOOK */

XtRealizeWidget(toplevel); 
XtMainLoop();

} /* main */

/***************** U T I L I T Y   F U N C T I O N S *****************/

/* to get color values for this display */

static void GetColors(d)	
	Display *d;
{
 XAllocNamedColor(d,DefaultColormap(d,0),"red", &this_red,&exact_red);
 XAllocNamedColor(d,DefaultColormap(d,0),"blue", &this_blue, &exact_blue);
 XAllocNamedColor(d,DefaultColormap(d,0),"purple", &this_purple, &exact_purple);
 XAllocNamedColor(d,DefaultColormap(d,0),"green", &this_green, &exact_green);
 XAllocNamedColor(d,DefaultColormap(d,0),"yellow", &this_yellow, &exact_yellow);
 XAllocNamedColor(d,DefaultColormap(d,0),"orange", &this_orange, &exact_orange);
 XAllocNamedColor(d,DefaultColormap(d,0),"white", &this_white, &exact_white);
 XAllocNamedColor(d,DefaultColormap(d,0),
				"skyblue",&this_skyblue,&exact_skyblue);
}

/*to position widgets on the form */

static void SetPosition(widget, xwidget, ywidget)
Widget widget, xwidget, ywidget;
{
	static int nargs;

	if(nargs == 0)
		nargs = XtNumber(genericARGS);

	genericARGS[0].value = (XtArgVal) xwidget; 
	genericARGS[1].value = (XtArgVal) ywidget; 
	XtSetValues(widget,genericARGS,nargs);
}

/* Xlib examples to utilize stub widget */

static void DrawAndPrintCB(widget,clientData,callData)
	Widget widget;		/* this widget is caption button */
	XtPointer *clientData, callData;
{
	Display *display;
	GC gc[6];
	int i,ix5,finish = 180*64;
	XGCValues values;
	static Boolean rainbow = FALSE;
	Widget dwidget;		/* widget to draw in */
	Boolean toggleflag;	/* toggle or refresh */
	Font fid = (Font)0;
	char *fontname = "*12bluci";
	Window window; 

	dwidget = (Widget) clientData[0];	
	toggleflag = (Boolean) clientData[1];
	display = XtDisplay(dwidget);
	window  = XtWindow(dwidget);

	if(toggleflag) {
		if(rainbow)
			rainbow = FALSE;
		else
			rainbow = TRUE;
	}

	if(rainbow) {
	
	values.line_width = 5;
	values.foreground = this_red.pixel;
	gc[0]= XtGetGC(dwidget,(XtGCMask)(GCForeground | GCLineWidth), &values);
	values.foreground = this_orange.pixel;
	gc[1]= XtGetGC(dwidget,(XtGCMask)(GCForeground | GCLineWidth), &values);
	values.foreground = this_yellow.pixel;
	gc[2]= XtGetGC(dwidget,(XtGCMask)(GCForeground | GCLineWidth), &values);
	values.foreground = this_green.pixel;
	gc[3]= XtGetGC(dwidget,(XtGCMask)(GCForeground | GCLineWidth), &values);
	values.foreground = this_blue.pixel;
	gc[4]= XtGetGC(dwidget,(XtGCMask)(GCForeground | GCLineWidth), &values);
	values.foreground = this_purple.pixel;
	gc[5]= XtGetGC(dwidget,(XtGCMask)(GCForeground | GCLineWidth), &values);

	for(i=0;i<6;i++) {
		ix5 = i*5;
		XDrawArc(display,window,
			gc[i],i*4+10,ix5+5,85-ix5,75-ix5,0,finish);
	}
	for(i=0;i<6;i++)
		XtReleaseGC(dwidget, gc[i]);
	}
	else
		XClearWindow(display,window);

				/* get font for this screen resolution */
	fontname[0] = OlGetResolution(XtScreen(dwidget));
	fid = XLoadFont(display,fontname);
				/* if successful, label the widget */
	if(fid != (Font)0) {
	values.font = fid;
	gc[0] = XtGetGC(dwidget,(XtGCMask) GCFont, &values);
	XDrawString(display,window,gc[0],5,55,"STUB WIDGET",11);
	XtReleaseGC(dwidget, gc[0]);
	XUnloadFont(display,fid);
	}
}

/* event handler used to refresh stub widget contents when redrawing needed */
			
static void SimpleEventHandler(widget,clientData,event)
	Widget widget;
	XtPointer clientData;
	XEvent *event;
{
	static XtPointer eventData[2];	/* use format next callback expects */

	eventData[0] =  (XtPointer) widget;
	eventData[1] = (XtPointer) FALSE;/* refresh picture but do not toggle */
	
	DrawAndPrintCB(widget,eventData,(XtPointer)NULL);
}

