/*
 *
 * lines.c -- a short example using the XView/NeWS library
 * 
 * A PSCANVAS is created, then NeWS PostScript rendering 
 * is used to draw lines
 *
 */
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/win_input.h>
#include <xvps/pscanvas.h>

#include "lines.h"
Frame frame;
Icon icon;
Panel panel;
Panel_item slider, popup;
PScanvas pscanvas;
NeWStoken lines, icon_canvas;

void paint_lines(j)
     float j;
{
  float i;

  clear();
/* setlinewidth(.01);  */
  for ( i=0; i<= 1 ; i= i+ (1/(j/2)))
    {
      set_color(i);
      moveto(0.0,0.0);
      lineto(1.0,i);
      ps_stroke();
      moveto(0.0, 0.0);
      lineto(i,1.0);
      ps_stroke();
    }
  ps_flush_PostScript();
}

void pscanvas_repaint_proc(pscanvas, newstoken, dpy, xid, xrects)
     PScanvas	pscanvas;
     NeWStoken	newstoken;
     Display	*dpy;
     Window	xid;
     Xv_xrectlist *xrects;
{
  xv_set(pscanvas, PSCANVAS_CLIPRECTS, xrects, NULL);
  pscanvas_flip();
  scale_to_fit();
  paint_lines((float) xv_get(slider, PANEL_VALUE));
  ps_clear_cliprects();
}

void slider_proc(item, value, event)
	Panel_item	item;
	int		value;
	Event		*event;
{
  ps_setcanvas(lines);
  pscanvas_flip();
  scale_to_fit();
  paint_lines((float) value);
}

menu_proc(menu, menu_item)
     Menu menu;
     Menu_item menu_item;
{
  char *choice=(char *)xv_get(menu_item, MENU_STRING);
  int size=atoi(choice);

  xv_set(slider,
	 PANEL_MAX_VALUE, size,
	 NULL);
}

event_proc(win, event)
	Xv_Window       win;
	Event          *event;
{
  if (event_action(event) == ACTION_MENU && event_is_down(event))
    {
      Menu menu= (Menu) xv_get(win, XV_KEY_DATA, 100);
      menu_show(menu, win, event, NULL);
    }
}

static initialized=0;
/*
 * Handle icon events.
 */
Notify_value
  icon_proc(win, evnt, arg, type)
	Xv_Window       win;
	Event          *evnt;
	Notify_arg      arg;
	Notify_event_type type;
{

  /* this procedure finds the NeWS canvas for the Icon, first time through */
  if (initialized) ps_setcanvas(icon_canvas);
  else {
    /* Otherwise get the NeWS canvas and store it in "icon_canvas" */
    
    if ((ps_token_from_xid(xv_get(win, XV_XID),&icon_canvas)))
      {
	int x,y;
	ps_setcanvas(icon_canvas);
	ps_getcanvaslocation(icon_canvas, &x, &y);
	circleat(x, y);
	ps_reshapecanvas(icon_canvas);
	initialized=1;
      }
  }
  if (event_id(evnt) == WIN_REPAINT) {
    ps_setcanvas(icon_canvas);
    pscanvas_flip();
    scale_to_fit();
    paint_lines(100.0);
  }

  return (notify_next_event_func(win, evnt, arg, type));
}

main(argc, argv)
	int             argc;
	char          **argv;
{
  xv_init(XV_INIT_ARGS, argc, argv, 0);
  icon=xv_create(XV_NULL, ICON,
		 XV_VISUAL_CLASS, StaticColor,
		 NULL);

  frame=xv_create(NULL, FRAME,
		  FRAME_LABEL, "XView/NeWS Library Example: Lines",
		  FRAME_ICON, icon,
		  NULL);
  panel=xv_create(frame, PANEL,
		  XV_X, 0,
		  XV_Y, 0,
		  XV_HEIGHT, 100,
		  XV_WIDTH, WIN_EXTEND_TO_EDGE,
		  NULL);

  slider=xv_create(panel, PANEL_SLIDER,
	    PANEL_LABEL_STRING, "Number of Lines:",
	    PANEL_VALUE, 10,
	    PANEL_MIN_VALUE, 5,
	    PANEL_MAX_VALUE, 100,
	    PANEL_SLIDER_WIDTH, 300,
	    PANEL_NOTIFY_PROC, slider_proc,
	    NULL);
  window_fit(panel);
  pscanvas=xv_create(frame, PSCANVAS,
	    XV_VISUAL_CLASS, StaticColor,
	    WIN_BELOW, canvas_paint_window(panel),
	    XV_X, 0,
	    PSCANVAS_REPAINT_PROC, pscanvas_repaint_proc,
	    NULL);
  window_fit(pscanvas);
  lines=(NeWStoken)xv_get(pscanvas, PSCANVAS_NEWSTOKEN, NULL);

  popup = (Menu) xv_create(NULL,
			   MENU,
			   XV_LABEL, "Range",
			   MENU_STRINGS,
			      "100", "250", "500", "1000", NULL,
			   MENU_NOTIFY_PROC, menu_proc,
			   NULL);

  xv_set(pscanvas,
	 WIN_CONSUME_EVENTS, WIN_MOUSE_BUTTONS, NULL, NULL);
  xv_set(pscanvas,
	 WIN_EVENT_PROC, event_proc,
	 XV_KEY_DATA, 100, popup,
	 NULL);

  xv_set(slider, 	    PANEL_NOTIFY_LEVEL, PANEL_ALL, 0);

  window_fit(frame);
  notify_interpose_event_func(icon,
			      icon_proc, NOTIFY_SAFE);

  xv_main_loop(frame);
}
