#ifndef lint
static char     sccsid[] = "@(#)xps.c	1.8 90/02/22 Copyright 1989 Sun Micro";
#endif

/*
 * This program was developed by John Matheny and Martin Knutson to
 * demonstrate the ability to use NeWS and the PostScript imaging model from
 * within XView applications.  It is based upon a program by Bruce Schwartz
 * shown at the Spring 1989 Uniforum. This program also demonstrates the
 * ability to "drag" a file from one application and "drop" it on another.
 * This is based upon work by Ian Wallis in Sun's Desktop Tools group.
 * 
 * To use this program, simply type some PostScript language code in the text
 * subwindow.  This code will be sent to the NeWS server and displayed on the
 * XView canvas when the Run button is pressed.  You may also drag a file
 * containing PostScript language code from the file manager and drop it on
 * this program.  The PostScript language code will be displayed in the text
 * subwindow and the image will be displayed on the XView canvas.
 * 
 * Programs that could be developed based on these capabilities include a
 * clipboard browser, a PostScript language document previewer, or a
 * PostScript language editor.
 * 
 * John Matheny, 9 May 1989
 *
 * Converted to use new Xview/NeWS support in Version 2.0
 * New Features:
 *
 * -	Library supports intelligent clipping directly
 * -	Icon shows scaled version when dropped on
 * -	Uses new Drag-and-drop API (xv_decode_drop)
 *
 * Henry Krempel, 16 May 1990
 */

#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/text.h>
#include <xview/xv_xrect.h>
#include <xview/seln.h>
#include <stdio.h>
#include "xps.h"
#include <xvps/pscanvas.h>

/*
 * Base frame.
 */
Frame           base_frame;
Icon            base_icon;
Notify_value    icon_eventproc();

/*
 * Code panel.
 */
Panel           code_panel;
void            run_eventproc();

/*
 * Code text subwindow.
 */
Textsw          code_textsw;
char            code_file[128];
Notify_value    code_eventproc();
void            code_setfile();

/*
 * Image panel.
 */
Panel           image_panel;
void            clear_eventproc();

/*
 * Image canvas.
 */
PScanvas          image_canvas;
char            image_file[128];
NeWStoken      image_paint_window, icon_canvas;
void            image_clear();
Notify_value    image_eventproc();
void            image_paint();
void            image_repaintproc();
void            image_setfile();

/*
 * File to store edited PostScript language code.
 */
char           *tmp_file = "/tmp/pstmp";

main(argc, argv)
	int             argc;
	char          **argv;
{
	xv_init(XV_INIT_ARGS, argc, argv, 0);

	/*
	 * Create base frame and icon.
	 * Note: leave this Icon NULL, we will use PostScript instead
	 *
	 */
	base_icon = xv_create(XV_NULL, ICON, 0);

	base_frame = xv_create(NULL, FRAME,
			       WIN_WIDTH, 500,
			       WIN_HEIGHT, 500,
			       FRAME_LABEL,
			       "XView/NeWS Drag & Drop Demo",
			       FRAME_SHOW_LABEL, TRUE,
			       FRAME_ICON, base_icon,
			       FRAME_NO_CONFIRM, TRUE,
			       0);

	/*
	 * Create code panel and buttons.
	 */
	code_panel = xv_create(base_frame, PANEL,
			       XV_WIDTH, WIN_EXTEND_TO_EDGE,
			       WIN_ROWS, 1,
			       XV_X, 0,
			       XV_Y, 0,
			       0);
	xv_create(code_panel, PANEL_BUTTON,
		  PANEL_LABEL_STRING, "Run",
		  PANEL_NOTIFY_PROC, run_eventproc,
		  0);
	window_fit_height(code_panel);

	/*
	 * Create code text subwindow.
	 */
	code_textsw = xv_create(base_frame, TEXTSW,
				XV_WIDTH, WIN_EXTEND_TO_EDGE,
				XV_X, 0,
				WIN_ROWS, 14,
				WIN_BELOW, code_panel,
				0);

	/*
	 * Create image panel and buttons.
	 */
	image_panel = xv_create(base_frame, PANEL,
				XV_WIDTH, WIN_EXTEND_TO_EDGE,
				WIN_ROWS, 1,
				XV_X, 0,
				WIN_BELOW, code_textsw,
				0);
	xv_create(image_panel, PANEL_BUTTON,
		  PANEL_LABEL_STRING, "Clear",
		  PANEL_NOTIFY_PROC, clear_eventproc,
		  0);
	window_fit_height(image_panel);

	/*
	 * Create image canvas.
	 */
	image_canvas = xv_create(base_frame, PSCANVAS,
				 XV_X, 0,
				 WIN_BELOW, image_panel,
				 WIN_WIDTH, WIN_EXTEND_TO_EDGE,
				 WIN_HEIGHT, WIN_EXTEND_TO_EDGE,
			       /* This attribute identifies that NeWS
				  Color will be used
				  */
			         XV_VISUAL_CLASS, StaticColor,
				 PSCANVAS_REPAINT_PROC, image_repaintproc,
				 PSCANVAS_SYNC, 6,
				 WIN_CONSUME_EVENTS, MS_LEFT,
				 OPENWIN_AUTO_CLEAR, FALSE,
				 LOC_WINENTER, LOC_WINEXIT, ACTION_DRAG_LOAD,
				 0,
				 0);

	xps_init();
	ps_flush_PostScript();

	/*
	 * Enter the notifier.
	 */
	notify_interpose_event_func(base_icon,
				    icon_eventproc, NOTIFY_SAFE);
	notify_interpose_event_func((Xv_Window) xv_get(image_canvas,
						       OPENWIN_NTH_VIEW, 0),
				    image_eventproc, NOTIFY_SAFE);
	notify_interpose_event_func(textsw_first(code_textsw),
				    code_eventproc, NOTIFY_SAFE);
	window_main_loop(base_frame);
}

static initialized=0;
/*
 * Handle icon events.
 */
Notify_value
icon_eventproc(win, evnt, arg, type)
	Xv_Window       win;
	Event          *evnt;
	Notify_arg      arg;
	Notify_event_type type;
{

  /* Check to see if the icon_canvas token has been set,
   * do a setcanvas, if it is.
   */
  if (initialized) ps_setcanvas(icon_canvas);
  else {
    /* Otherwise get the NeWS canvas and store it in "icon_canvas" */
    
    if ((ps_token_from_xid(xv_get(win, XV_XID),&icon_canvas))) initialized=1;
    if (initialized) {
      /* Reshape the Icon into a circle, it might be fun to have it
       * change shape each time a file is dropped on it            
       * (left as an excercise)
       */
      int x, y;
      ps_setcanvas(icon_canvas);
      ps_getcanvaslocation(icon_canvas, &x, &y);
      xps_circle(x, y); /* see xps.cps: makes the appropriate path */
      ps_reshapecanvas(icon_canvas);
    }
    else
      {
	/* if for some reason ps_token_from_xid fails (only if the
	 * XID was wrong, but then we're in trouble anyway.
	 */
	NeWStoken	NeWSwin=xv_get(image_canvas,PSCANVAS_NEWSTOKEN);
	ps_setcanvas(NeWSwin);
      }
  }

  if (event_id(evnt) == WIN_REPAINT) {
    image_paint(NULL);
  }
  /* Handle DROP events */
  else if (event_action(evnt) == ACTION_DRAG_LOAD ||
	   event_action(evnt) == ACTION_DRAG_MOVE ||
	   event_action(evnt) == ACTION_DRAG_COPY) {
    int n;
    char buffer[128];

    n = xv_decode_drop(evnt, buffer, sizeof(buffer));
    if (n == -1)
      {
	fprintf(stderr, "xv_decode_drop failed\n");
      }
    else
      {
	if (n >= sizeof(buffer))
	  {
	    fprintf(stderr, "Data was truncated; %d bytes lost.\n",
		    n - sizeof(buffer) + 1);
	  }
	code_setfile(buffer);
	image_setfile(buffer);
	image_paint(NULL);
      }
  }

  return (notify_next_event_func(win, evnt, arg, type));
}


/*
 * Handle clear button events.
 */
void
clear_eventproc(panel_item, event)
	Panel_item      panel_item;
	Event          *event;
{
	image_clear();
}

/*
 * Handle code text subwindow events.
 */
Notify_value
code_eventproc(win, evnt, arg, type)
	Xv_Window       win;
	Event          *evnt;
	Notify_arg      arg;
	Notify_event_type type;
{
  NeWStoken	NeWSwin=xv_get(image_canvas,PSCANVAS_NEWSTOKEN);
  ps_setcanvas(NeWSwin);
  if (event_action(evnt) == ACTION_DRAG_LOAD ||
      event_action(evnt) == ACTION_DRAG_MOVE ||
      event_action(evnt) == ACTION_DRAG_COPY)
    {
      int n;
      char buffer[128];

      n = xv_decode_drop(evnt, buffer, sizeof(buffer));
      if (n == -1) {
	fprintf(stderr, "xv_decode_drop failed\n");
      }
      else {
	if (n >= sizeof(buffer))
	  {
	    fprintf(stderr, "Data was truncated; %d bytes lost.\n",
		    n - sizeof(buffer) + 1);
	  }
	code_setfile(buffer);
	image_setfile(buffer);
	image_paint(NULL);
      }
    }
  return (notify_next_event_func(win, evnt, arg, type));
}

/*
 * Set the current code file.
 */
void
code_setfile(s)
	char           *s;
{
	strcpy(code_file, s);
	xv_set(code_textsw, TEXTSW_FILE, code_file, 0);
}

/*
 * Handle image canvas events.
 */
Notify_value
image_eventproc(win, evnt, arg, type)
	Xv_Window       win;
	Event          *evnt;
	Notify_arg      arg;
	Notify_event_type type;
{
  /*
   * Setcanvas to the image canvas.  This isn't automatically done
   * for arbitrary events -- only repaint and resize events
   *
   */
  NeWStoken	NeWSwin=xv_get(image_canvas,PSCANVAS_NEWSTOKEN);
  ps_setcanvas(NeWSwin);
  if (event_action(evnt) == ACTION_DRAG_LOAD ||
      event_action(evnt) == ACTION_DRAG_MOVE ||
      event_action(evnt) == ACTION_DRAG_COPY)
    {
      int n;
      char buffer[128];

      n = xv_decode_drop(evnt, buffer, sizeof(buffer));
      if (n == -1) {
	fprintf(stderr, "xv_decode_drop failed\n");
      }
      else {
	if (n >= sizeof(buffer))
	  {
	    fprintf(stderr, "Data was truncated; %d bytes lost.\n",
		    n - sizeof(buffer) + 1);
	  }
	code_setfile(buffer);
	image_setfile(buffer);
	image_paint(NULL);
      }
    }
  return (notify_next_event_func(win, evnt, arg, type));
}

/*
 * Clear the current image.
 */
void
image_clear()
{
  NeWStoken	NeWSwin=xv_get(image_canvas,PSCANVAS_NEWSTOKEN);

  ps_gsave();
  ps_setcanvas(NeWSwin);
  xps_erasepage();
  ps_flush_PostScript();
  ps_grestore();
}

/*
 * Paint the current image file onto the current canvas.
 */
void
image_paint(xrects)
	Xv_xrectlist   *xrects;
{
  static char error[300];
  if (xrects == NULL) {
    ps_clear_cliprects();
  }
    ps_gsave();
    xps_blankcanvas();
    ps_grestore();
  if (*image_file)
    {
      xps_runfile(image_objectid, image_file);
    }
  else
    xps_blankcanvas();
  ps_flush_PostScript();
}

/*
 * Handle image repaint events.
 */
void
image_repaintproc(canvas, paint_window, display, xid, xrects)
	PScanvas          canvas;
	NeWStoken      paint_window;
	Display        *display;
	Window          xid;
	Xv_xrectlist   *xrects;
{
  /* Pass any damage on to be used as a clip, NULL damage
   * clears the clipping
   *
   */
  xv_set(image_canvas, PSCANVAS_CLIPRECTS, xrects, 0);
  image_paint(xrects);
}

/*
 * Set the current image file.
 */
void
image_setfile(s)
	char           *s;
{
	strcpy(image_file, s);
}

/*
 * Handle events from the run button.
 */
void
run_eventproc(panel_item, event)
	Panel_item      panel_item;
	Event          *event;
{

	static Seln_holder holder;
	static Seln_request *req;

	holder = seln_inquire(SELN_PRIMARY);
	unlink(tmp_file);

	/*
	 * If there is no selection, write out the entire file.
	 */
	if (holder.state == SELN_NONE ||
	    !seln_holder_same_client(&holder, code_textsw))
		textsw_store_file(code_textsw, tmp_file, 0, 0);

	/*
	 * Otherwise, write out only the selected text.
	 */
	else {
		char            write_buffer[1024];
		unsigned        first_byte;
		unsigned        num_bytes;
		unsigned        req_size;
		unsigned       *srcp;
		FILE           *output_file;

		req = seln_ask(&holder,
			       SELN_REQ_FIRST_UNIT, 0,
			       SELN_REQ_BYTESIZE, 0,
			       0);

		srcp = (unsigned *) req->data; srcp++;
		first_byte = *srcp++; srcp++;
		num_bytes = *srcp++;

		if (num_bytes == 0)
			textsw_store_file(code_textsw, tmp_file, 0, 0);
		else {
			output_file = fopen(tmp_file, "w");
			while (num_bytes) {
				req_size = 1024 > num_bytes ? num_bytes : 1024;
				xv_get(code_textsw, TEXTSW_CONTENTS, first_byte,
				       write_buffer, req_size);
				fwrite(write_buffer, 1, req_size, output_file);
				first_byte += req_size;
				num_bytes -= req_size;
			}
			fclose(output_file);
		}
	}
	image_setfile(tmp_file);
	image_paint(NULL);
}
