#ifndef lint
static char *sccsid = "@(#)chalk.c 23.3 90/06/19";
#endif

#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>

#define MAXHOSTS 20
#define BUFSIZE 4096
#define NEWSPORT 2000
#define CHALKPORT 2001
#define CHALKPSFILE "/demo/chalk.ps"
#define CHALKTIMEOUT 10L
#define TRUE 1
#define FALSE 0

static int debug;
extern int errno;

/*
* Dispatcher for ChalkTalk
*
*	Josh Axelrod
*	2/9/89
*
*/

/*
*
* Connect to a group of machines
*
*/
int connecttoservers(names, sockets)
char **names;
int *sockets;
{
  struct sockaddr_in name;
  struct hostent *host;
  int count = 0;

  while (*names && (count < MAXHOSTS)) {
    if ((host = gethostbyname(*names)) == (struct hostent *) NULL) {
      (void) fprintf(stderr,
	"connecttoservers:gethostbyname:%s\n", *names);
      names++;
      continue;
    }
    bzero((char *) &name, sizeof(name));
    bcopy((char *) host->h_addr, (char *) &name.sin_addr.s_addr,
      host->h_length);
    name.sin_family = AF_INET;
    name.sin_port = htons(NEWSPORT);

    if ((sockets[count] = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
      perror("connecttoservers:socket");
      names++;
      continue;
    }
    if (connect(sockets[count],
	(struct sockaddr *) & name, sizeof(name)) == -1) {
      perror("connecttoservers:connect");
      names++;
      continue;
    }

    names++;
    count++;
  }
  return (count);
}

/*
* Download initializing PostScript
*
*/

int initializeservers(sockets, n)
int *sockets;
int n;
{
  extern char *getenv(), *sprintf();
  int fd, nread, i;
  char buf[BUFSIZE];
  char *dirname = debug ? "./" : (char *) getenv("OPENWINHOME");
  char *psfile = debug ? "chalk.ps" : CHALKPSFILE;
  char *fname;

  if (dirname == NULL) {
    perror("initializeservers:getenv");
    return (1);
  }

  fname = (char *) malloc(strlen(dirname) + strlen(psfile) + 1);
  fname[0] = 0;

  strcat(fname, dirname);
  strcat(fname, psfile);

  if ((fd = open(fname, O_RDONLY)) == -1) {
    fprintf(stderr, "couldn't open %s\n", fname);
    perror("initializeservers:open");
    return (1);
  }
  free(fname);
  do {	/* blow init ps code down all sockets */
    if ((nread = read(fd, buf, BUFSIZE)) == -1) {
      perror("initializeservers:read");
      return (1);
    }
    for (i = 0; i < n; i++)
      if (pswrite(sockets[i], buf, nread) == -1) {
	perror("initializeservers:pswrite");
	return (1);
      }
  } while (nread > 0);

  (void) close(fd);
  return (0);
}

buildlistener(socket, socketindex, id)
{
  int pid;
  char buf[BUFSIZE];
  pid = getpid();

 /* Build a lwp to act as a listener for draw events from other
    connections. Name is pid+socketindex+id */

  (void) sprintf(buf, "/chalk%d%d%d BuildListener\n", pid,
    socketindex, id);
  if (pswrite(socket, buf, strlen(buf)) == -1) {
    perror("buildlistener:pswrite");
    return (1);
  }

  return (0);
}

/*
* Handle i/o
*
*/

void loop(sockets, n, lsocket)
int *sockets;
int n;
int lsocket;
{
  char buf[BUFSIZE];
  int i, nbusy, nread, pid;
  static int disconnects[MAXHOSTS];
  fd_set imask;
  struct timeval timeout;
  extern char *sprintf();

  timeout.tv_sec = CHALKTIMEOUT;
  timeout.tv_usec = 0;

  pid = getpid();

  while (1) {

    do {

      int nowork = TRUE;

      FD_ZERO(&imask);

      FD_SET(lsocket, &imask);	/* add listen socket */

      for (i = 0; i < n; i++)	/* add client sockets */
	if (!disconnects[i]) {
	  FD_SET(sockets[i], &imask);
	  nowork = FALSE;
	}

      if (nowork)
	return;

      if ((nbusy = select(MAXHOSTS, &imask,
	    (fd_set *) NULL, (fd_set *) NULL, &timeout)) == -1) {
	perror("loop:select");
	return;
      }
    } while (!nbusy);

    if ((n < MAXHOSTS) && FD_ISSET(lsocket, &imask)) {
      if ((sockets[n] = addclient(lsocket)) == -1) {
	(void) fprintf(stderr, "loop:addclient");
      }
      else {
	n++;
      /* add new listener to existing connections */
	for (i = 0; i < n - 1; i++)
	  if (!disconnects[i])
	    if (buildlistener(sockets[i], i, n - 2)) {
	      (void) fprintf(stderr, "loop:buildlistener\n");
	      continue;
	    }
      /* build listeners for new connection */
	for (i = 0; i < n - 1; i++)
	  if (buildlistener(sockets[n - 1], n - 1, i)) {
	    (void) fprintf(stderr, "loop:buildlistener\n");
	    continue;
	  }
      }
      nbusy--;
    }
    for (i = 0; i < n; i++) {

      if (!nbusy)
	break;

      if (FD_ISSET(sockets[i], &imask)) {
	nbusy--;
	bzero(buf, BUFSIZE);

	if ((nread = read(sockets[i], buf, sizeof(buf))) == -1) {
	  perror("loop:read");
	  return;
	}
	else {
	  int j;
	  char buf2[BUFSIZE + 20];

	  if (!strncmp(buf, "_quit", 5)) {
	    if (debug)
	      (void) printf("[%d] exited\n", i);
	    disconnects[i] = TRUE;
	  }
	  else {
	    for (j = 0; j < n; j++)
	      if ((j != i) && !disconnects[j]) {

		buf[nread] = '\0';
		(void) sprintf(buf2, "{%s} /chalk%d%d%d E ", buf,
		  pid, j, (j < i ? j : j - 1));

		if (pswrite(sockets[j], buf2, strlen(buf2)) == -1) {
		  perror("loop:pswrite");
		  continue;
		}
	      }
	  }
	}
      }
    }
  }
}

int addclient(lsocket)
{
  int fd;
  int namelen = sizeof(struct sockaddr_in);
  struct sockaddr_in name;

  if ((fd = accept(lsocket,
	(struct sockaddr *) & name, &namelen)) == -1) {
    perror("addclient:accept");
    return (-1);
  }
  if (write(fd, "starting chalkboard", 19) == -1) {
    perror("addclient:write");
    return (-1);
  }
  if (debug) {
    (void) fprintf(stderr, "new client request from %x\n",
      name.sin_addr.s_addr);
  }
  (void) close(fd);

 /* got addr from accept, just switch ports */

  name.sin_family = AF_INET;
  name.sin_port = htons(NEWSPORT);

  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("addclient:socket");
    return (-1);
  }

  if (connect(fd, (struct sockaddr *) & name, namelen) == -1) {
    perror("addclient:connect");
    return (-1);
  }

  if (initializeservers(&fd, 1)) {
    (void) fprintf(stderr, "addclient:initializeservers\n");
    return (-1);
  }
  
  return (fd);
}

int buildlistensocket(portnumber)
{
  struct sockaddr_in name;
  int s;

  name.sin_family = AF_INET;
  name.sin_addr.s_addr = INADDR_ANY;
  name.sin_port = htons(portnumber);

  if ((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("buildlistensocket:socket");
    return (-1);
  }
  if (bind(s, (struct sockaddr *) & name, sizeof(name)) == -1) {
    while (errno == EADDRINUSE) {
      name.sin_port++;
      errno = 0;
      if (bind(s, (struct sockaddr *) & name, sizeof(name)) == -1)
	continue;
      else
	break;
    }
    if (errno) {
      perror("buildlistensocket:bind");
      return (-1);
    }
  }

  if (listen(s, 5) == -1) {
    perror("buildlistensocket:listen");
    return (-1);
  }

  (void) fprintf(stderr, "Chalk listening on port %d\n",
    ntohs(name.sin_port));

  return (s);
}

/*
*   hack until I extend and use cps
*/

int pswrite(fd, buf, buflen)
char *buf;
{
  if (debug)
    (void) write(2, buf, buflen);
  return (write(fd, buf, buflen));
}

void usage() 
{
  (void) fprintf(stderr, "Usage:chalk [-p port] host [host ...]\n");
   exit(1);  
}
main(argc, argv)
int argc;
char **argv;
{
  int s[MAXHOSTS];
  int nhosts = 0;
  int lsocket, i, j;
  int portnumber = CHALKPORT;

  if (argc < 2)
    usage();

  argv++;
  while (*argv) {
    if (**argv == '-') {
      switch (*(*argv + 1)) {
       case 'd':
	debug++;
	break;
       case 'p':
	if (argv[1]) {
	  argv++;
	  portnumber = atoi(*argv);
	}
	else
	  usage();
	break;
       default:
	usage();
	break;
      }
      argv++;
    }
    else
      break;
  }

  if ((lsocket = buildlistensocket(portnumber)) == -1) {
    (void) fprintf(stderr, "main:buildlistensocket\n");
    exit(1);
  }

  if ((nhosts = connecttoservers(argv, s)) < 1) {
    (void) fprintf(stderr, "main:connecttoservers\n");
    exit(1);
  }

  if (initializeservers(s, nhosts)) {
    (void) fprintf(stderr,
      "main:initializeservers:no servers connected\n");
    exit(1);
  }

  for (i = 0; i < nhosts; i++)
    for (j = 0; j < nhosts - 1; j++)
      if (buildlistener(s[i], i, j)) {
	(void) fprintf(stderr, "main:buildlistener\n");
	continue;
      }

  loop(s, nhosts, lsocket);

  while (nhosts)
    (void) close(s[nhosts--]);

  (void) close(lsocket);

  exit(0);
}
