/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static char sccsid[] = "@(#)M% 23.2 90/06/19";
#endif

/*
 * Copyright (c) 1989 by Sun Microsystems, Inc.
 */

/*
 * objectdiff: Takes to files in the format of the file produced by
 *             the objectdump NeWS primitive.  A stylized diff of the
 *             files is produced with a TOTAL difference tabulated.
 */

#include <stdio.h>

#define OUTPUT_FORMAT_STR "%9d bytes for %4d \"%s\n"
#define INPUT_FORMAT_STRING "  %d bytes for %d \"%[ -z]\n"
#define TOTAL_TYPE_STRING "TOTAL\" objects."
#define BUF_SIZE 200

main(argc, argv)
int	argc;
char	*argv[];
{
	FILE *f1, *f2;
	char *fn1, *fn2;
	int bytes1, objects1;
	int bytes2, objects2;
	int bytes, objects;
        char type1[BUF_SIZE], type2[BUF_SIZE];
        char buf1[BUF_SIZE], buf2[BUF_SIZE];
	int bs1, bs2;
	int total_bytes = 0, total_objects = 0;

	total_bytes = total_objects = 0;
	/* Get input arguments (file 1 and file 2) */
	if (argc < 3) {
		fprintf(stderr, "Too few args: f1 f2\n");
		exit(1);
	}
	fn1 = argv[1];
	fn2 = argv[2];
	/* Open files */
	f1 = fopen(fn1, "r");
	f2 = fopen(fn2, "r");
	if (f1 == 0 || f2 == 0) {
		fprintf(stderr, "Both files can't be opened\n");
		exit(1);
	}
	/* Diff files */
	while (!feof(f1) && !feof(f2)) {
                type1[0] = type2[0] = buf1[0] = buf2[0] = NULL;
		(void) fgets(buf1, BUF_SIZE, f1);
		(void) fgets(buf2, BUF_SIZE, f2);
                if (buf1[0] == NULL || buf2[0] == NULL)
                        break;
		bs1 = sscanf(buf1, INPUT_FORMAT_STRING,
			     &bytes1, &objects1, type1);
		bs2 = sscanf(buf2, INPUT_FORMAT_STRING,
			     &bytes2, &objects2, type2);
                if (bs1 != 3 || bs2 != 3) {
			continue;
                }
                if (strcmp(type1, type2)) {
                        fprintf(stderr, "type sync error: %s, %s\n",
                                type1, type2);
                        exit(1);
                }
                bytes = bytes2 - bytes1;
                objects = objects2 - objects1;
		if (bytes || objects) {
		    fprintf(stdout, OUTPUT_FORMAT_STR, bytes, objects, type1);
		    total_bytes += bytes;
		    total_objects += objects;
		}
	}
        /* Print total */
        if (total_bytes)
                fprintf(stdout, OUTPUT_FORMAT_STR,
                        total_bytes, total_objects, TOTAL_TYPE_STRING);
	fclose(f1);
	fclose(f2);
        exit(0);
}
