/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static char sccsid[] = "@(#)psh.c 23.2 90/06/19";
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc.
 */

/*-
	PostScript shell
	Used for building applications written entirely
	in PostScript.  Merely takes its stdin and writes
	it to the NeWS server.

 */

#include <stdio.h>
#include <NeWS/psio.h>
char       *programname;

PSFILE       *PostScript,
             *PostScriptInput;

#ifndef SYSV
processfile(fd, more_files)
    register    fd;
{
    int         mask;
    int         mask0;
    int         max;
    int         first = 1;
    register PSFILE *outf;
    if (PostScript == 0 && ps_open_PostScript() == 0) {
	fprintf(stderr, "%s: Cannot connect to window server\n",
		programname);
	exit(1);
    }
    outf = PostScript;
    mask0 = 1 << psio_fileno(PostScriptInput);
    max = psio_fileno(PostScriptInput);
    if (fd > max)
	max = fd;
    max++;

    while (1) {
	char        buf[8 * 1024];
	register    n;

	mask = mask0;
	if (fd >= 0) {
	    mask |= 1 << fd;
	}
	psio_flush(outf);
	if (select(max, &mask, 0, 0, 0) < 0)
	{
	    perror(programname);
	    exit(1);
	}
	if (mask & mask0)
	{
	    n = read(psio_fileno(PostScriptInput), buf, sizeof buf);
	    if (n == 0) {
		if (fd >= 0)
		    fprintf(stderr, "%s: NeWS server disconnected\n",
			    programname);
		exit(0);
	    }
	    if (n < 0) {
		perror(programname);
		exit(1);
	    }
	    write(1, buf, n);
	}
	else if (fd >= 0 && (mask & (1 << fd)) != 0)
	{
	    register char *in;
	    n = read(fd, buf, sizeof buf);
	    if (n == 0) {
		if (more_files)
		    return;		/* only quit after last file */

		psio_fprintf(outf, " quit\n");
		close(fd);
		fd = -1;
		continue;
	    }
	    if (n < 0) {
		perror(programname);
		exit(1);
	    }
	    if (buf[0] == '#' && first)
		buf[0] = '%';
	    for (in = buf; --n >= 0; in++) {
		psio_putc(*in, outf);
	    }
	    first = 0;
	}
	else {
	    fprintf(stderr, "%s: bogus read mask %x\n", programname, mask);
	    exit(1);
	}
    }
}
#else
#include <signal.h>
#include <errno.h>

extern int errno;

int showmsg = 1;
int child = -1;

terminate()
{
    showmsg = 0;
}

processfile(fd, more_files)
    register    fd;
{
    char buf[8 * 1024];
    register    n;
    register PSFILE *outf;

    if (PostScript == 0 && ps_open_PostScript() == 0) {
	fprintf(stderr, "%s: Cannot connect to window server\n",
		programname);
	exit(1);
    }
    outf = PostScript;

    if (child == -1 && (child = fork()) > 0) {	/* child */
	signal(SIGCLD, terminate);
	while (n = read(psio_fileno(PostScriptInput), buf, sizeof buf)) {
	    if (n < 0) {
		if (errno == EINTR)
		    continue;
		else {
	            perror(programname);
		    break;
		}
	    }
	    write(1, buf, n);
	}
	if (showmsg) {
	    fprintf(stderr, "%s: NeWS server disconnected\n", programname);
	    signal(SIGCLD, SIG_DFL);
	    kill(child, SIGKILL);
	}
	exit(0);
    } else if (child == 0) {	/* parent */
	int first = 1;
	register char *in;

	while (n = read(fd, buf, sizeof buf)) {
	    if (n < 0) {
	        perror(programname);
		break;
	    }
	    if (buf[0] == '#' && first)
	        buf[0] = '%';
	    for (in = buf; --n >= 0; in++) {
	        psio_putc(*in, outf);
	    }
	    first = 0;
	    psio_flush(outf);
	}
	if (more_files)
	    return;
	psio_fprintf(outf, " quit\n");
	psio_flush(outf);
    } else {
	fprintf(stderr, "%s: fork failure\n", programname);
	exit(1);
    }
}
#endif

main(argc, argv)
    char      **argv;
{
    int		input_file;

    programname = argv[0];

    if (argc < 2)
	processfile(0, 0);
    else while (--argc > 0) {
	if ((++argv)[0][0] == '-')
	    switch (argv[0][1]) {
	    case '\0':
		processfile(0, argc > 1);
		break;
	    default:
		fprintf(stderr, "%s: invalid switch '%s'\n",
			programname, argv[0]);
		exit(1);
	    }
	else {
	    input_file = open(argv[0], 0);
	    if (input_file <= 0) {
		fprintf(stderr, "%s: cannot open %s\n", programname, argv[0]);
		exit(1);
	    }
	    processfile(input_file, argc > 1);
	}
    }
    exit(0);
}
