/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static char sccsid[] = "@(#)display.c 23.3 90/06/19 Copyright 1985 Sun Micro";
#endif

/*
 * Copyright (c) 1985 by Sun Microsystems, Inc. 
 */

/*-
	display.c

	display.c, Wed Mar 26 15:56:31 1986

		David Rosenthal,
		Sun Microsystems
 */

#ifdef	notdef
#define Diagnose
#endif
/*
 * Screen display module 
 *
 * The external interface of this module is the routine tc_refresh(), and the
 * initialization routine tc_init_screen(). 
 */

#include	<sys/types.h>
#include	<ref/config.h>
#include	"screen.h"
#include	"tcap.h"

#define CHARS_PER_LINE		80
#define LINES_PER_SCREEN	24
#define CHARS_PER_SCREEN	1920

Unsgn16     CharsPerLine = CHARS_PER_LINE;
Unsgn16     LinesPerScreen = LINES_PER_SCREEN;
int outstanding_displays = 0;
int Damage;
struct pair Dot = {0, 0};

static Repairing;
static struct pair ScreenDot = {0, 0};
static char *CursorChar = " ";
static Unsgn8 CursorProp;
extern int PageFull;
struct line *lastOutputLine; 

static
RenderCursor(goingup)
{
    register invert = (CursorProp & ReverseVideoLook);
    register x = ScreenDot.x,
	     y = ScreenDot.y;
    register char *c = CursorChar;

    if (goingup)
	invert = !invert;
    if (invert)
	CursorUp(x, y, c, 1);
    else
	CursorDown(x, y, c, 1);
    if (CursorProp & UnderlineLook) {
	if (invert)
	    WhiteLine(x, y, 1);
	else
	    UnderLine(x, y, 1);
    }
}

static
RemoveCursor()
{
    if (ScreenDot.x >= 0) {
	register struct line *l = screen[ScreenDot.y].lineptr;
#ifdef CURSORDEBUG
	psio_fprintf(psio_stderr,
		     "(%2d,%2d/%2d) cp:%2d ce:%2d le:%2d vi:%2d  '%c'[%2x]",
		     ScreenDot.x, ScreenDot.y, l->usedtobe,
		     l->changeposition, l->changeend,
		     l->length, screen[ScreenDot.y].visible,
		     *CursorChar, CursorProp);
#endif

	if (l->usedtobe != ScreenDot.y
	    || ScreenDot.x <= l->changeposition
	    || ScreenDot.x >= l->changeend
	    || ScreenDot.x >= screen[ScreenDot.y].visible
	    || ScreenDot.x >= l->length) {
	    RenderCursor(0);
#ifdef CURSORDEBUG
	    psio_fprintf(psio_stderr, " y\n");
	} else {
	    psio_fprintf(psio_stderr, " n\n");
#endif
	}
	ScreenDot.x = -1;
    }
}

PlaceCursor(x, y)
{
    register struct line *l;

    if (x >= CharsPerLine)
	ScreenDot.x = CharsPerLine - 1;
    else if (x < 0)
	ScreenDot.x = 0;
    else
	ScreenDot.x = x;
    if (y >= LinesPerScreen)
	ScreenDot.y = LinesPerScreen - 1;
    else if (y < 0)
	ScreenDot.y = 0;
    else
	ScreenDot.y = y;
    l = screen[ScreenDot.y].lineptr;
    CursorChar = (ScreenDot.x < l->length ? l->body + ScreenDot.x : " ");
    CursorProp = (ScreenDot.x < l->length ? l->prop [ScreenDot.x] : 0);
    if (!PageFull)			/* hide cursor if output blocked */
	RenderCursor(1);
}

/* --------------- External Routines Below ------------------- */

do_display_resize() {
    register i;
    ReInitialize();
    for (i = 0; i<LinesPerScreen; i++) {
	screen[i].lineptr->changeposition = 0;
	screen[i].lineptr->changeend = screen[i].lineptr->length;
	screen[i].visible = 0;
    }
    ++Damage;
    tc_refresh(0);
}

do_damage()
{
    int row, col, endrow, endcol;
    register temp;
    register struct line *linep;

    /* must only be called from top level (output not blocked) */
    tc_refresh(1);
    BeginRepair(&row, &col, &endrow, &endcol);
    Repairing++;
    if (row > endrow) {
	temp = row;
	row = endrow;
	endrow = temp;
    }
    if (col > endcol) {
	temp = col;
	col = endcol;
	endcol = temp;
    }
    if (row < 0)
	row = 0;
    if (endrow >= LinesPerScreen)
	endrow = LinesPerScreen - 1;
    if (col < 0)
	col = 0;
    if (endcol >= CharsPerLine)
	endcol = CharsPerLine - 1;

    while (row <= endrow) {
	linep = screen[row++].lineptr;
	DamageLine (linep, col);
	DamageLine (linep, endcol);
    }
    ++Damage;
    tc_refresh(0);
}

PSFILE       *
tc_init_screen(arg0, term, fixedsize, xorg, yorg, Wwidth, Wheight,
framelabel, iconlabel, fontsize)
    char *term, *arg0;
    char *framelabel, *iconlabel;
{
    register int i;
    PSFILE       *f;
    char        frametitle[100];
    char        icontitle[100];
    char	host[100];
    extern int	console;
    extern char *malloc();
    extern PSFILE *ps_open_PostScript();

    if ((f = ps_open_PostScript()) == NULL) {
	return (NULL);
    }

    tc_init_selection();
    lines = (struct line *) malloc(LinesPerScreen * sizeof(struct line));
    screen = (struct screen *) malloc(LinesPerScreen * sizeof(struct screen));
    for (i = 0; i < LinesPerScreen; i++) {
	lines[i].body = malloc(CharsPerLine+1);
	lines[i].prop = (Unsgn8 *) malloc(CharsPerLine+1);
	lines[i].length = 0;
	lines[i].changeposition = 1;
	lines[i].changeend = 0;
	lines[i].usedtobe = i;
	clear_body(lines + i, 0, CharsPerLine);
	screen[i].lineptr = lines + i;
	screen[i].visible = 0;
    }
    lastOutputLine = screen[0].lineptr;
    if (framelabel == NULL) {
	sprintf(frametitle, "%s %s emulator",
			    term, (console ? "console" : "terminal"));
	framelabel = frametitle;
    }
    if (iconlabel == NULL) {
        gethostname(host, sizeof (host));
	iconlabel = host;
    }
    Initialize(arg0, xorg, yorg, Wwidth, Wheight, CharsPerLine, LinesPerScreen, fixedsize,
	framelabel, iconlabel, fontsize);
    StartInput();
    /* XXX - set up PostScript process for i/p etc. */
    psio_flush(PostScript);
    return (f);
}

/*
 * Flush a run of lines to be copied.
 */
#define	FlushCopies(start, current, delta) { \
    CopyLines(start - delta, delta, CharsPerLine, current-start); \
    if (delta > 0) { \
	register int j; \
	for (j = current - 1; j >= start; j--) { \
	    hit[j] = 1; \
	    screen[j].visible = screen[j-delta].visible; \
	} \
    } else { \
	for (; start < current; start++) { \
	    hit[start] = 1; \
	    screen[start].visible = screen[start-delta].visible; \
	} \
    } \
    invalidate_selections(); \
}

/*
 * Flush a run of lines to be erased.
 */
#define FlushErases(start, current, len) \
	    EraseRectangle(0, start, len + 1, current-start); \

tc_refresh(force)
    int force;
{
    register struct line *l;
    register int i, c, pos, end;
    register Unsgn8 *cp;
    register struct screen *sp;
    int delta, curdelta, len;
    Unsgn8 *tp, *ep;
    static int OldLinesPerScreen = 0;
    static Unsgn8 *hit = 0;
    extern char *malloc(), *realloc();
    static int maxod = 0;

    if (!Damage)
	return;
    if (outstanding_displays) {
	if (force) {			/* this output must go through */
	    while (outstanding_displays)
		if (transferio(1) < 0)
		    return -1;
	} else
	    return;
    }

    if (LinesPerScreen != OldLinesPerScreen) {
	OldLinesPerScreen = LinesPerScreen;
	if (hit)
	    hit = (Unsgn8 *) realloc(hit, LinesPerScreen);
	else
	    hit = (Unsgn8 *) malloc(LinesPerScreen);
    }
    if (!hit)
	return;
    bzero(hit, LinesPerScreen);
    RemoveCursor();

    /*
     * Figure out which lines have moved and by
     * how much.  Accumulate delta line movements
     * and copy lines to perform scrolling.
     */
    curdelta = 0;
    sp = screen;
    for (i = 0; i < LinesPerScreen; i++) {
	l = (sp++)->lineptr;
	delta = 0;
	if ((pos = l->usedtobe) != i) {
	    /*
	     * Line has moved.  Either the contents are still
	     * the same, in which case they can be copied to
	     * their new location, or they have changed, in
	     * which case we must repaint and/or erase.
	     */
	    if (l->changeposition > 0
		|| l->changeend < l->length) {
		/*
		 * Some of line contents are untouched, if they're
		 * still on the display, calculate the distance
		 * they've moved (the delta).  Otherwise,
		 * set the ``change position'' to force the
		 * entire line to be repainted below.
		 */
		if (pos < LinesPerScreen && pos >= 0 && !hit[pos]) {
		    delta = i - pos;
		} else {
		    l->changeposition = 0;
		    l->changeend = l->length;
		}
	    }
	    l->usedtobe = i;
	}
	/*
	 * If the delta changes, we need to start a new
	 * run of lines to copy.  If there was a previous
	 * run, then copy those first.
	 */
	if (delta != curdelta) {
	    if (curdelta != 0) {
		FlushCopies(c, i, curdelta);
		if (delta && hit[i - delta]) {
		    /* The current "from" line was hit during that
		     * copy maneuver.  It shouldn't be copied now.
		     */
		    delta = 0;
		    l->changeposition = 0;
		    l->changeend = l->length;
		}
	    }
	    curdelta = delta;		/* delta for current run */
	    c = i;			/* starting line of run */
	}
    }
    /*
     * Catch any trailing run needing to be copied.
     */
    if (curdelta != 0)
	FlushCopies(c, i, curdelta);

    /*
     * Figure out which lines are cleared.
     * Accumulate ranges of cleared lines
     * and clear them in one big rectangle.
     */
    len = -1;
    sp = screen;
    for (i = 0; i < LinesPerScreen; i++, sp++) {
	l = (sp)->lineptr;
	if (sp->visible != 0 &&
	    ((l->length == 0 && (sp->visible > 1 ||
				 !(l->prop[0] & SelModes))) ||
	     (l->changeposition == 0 && (l->changeend >= l->length ||
					 l->changeend >= sp->visible)))) {
	    /*
	     * Entire line has disappeared or changed, start or continue a run.
	     */
	    if (len < 0)
		c = i;
	    if ((int) sp->visible > len)
		len = sp->visible;
	    if ((int) l->changeend > len)
		len = l->changeend;
	    sp->visible = 0;
	    if (l->length == 0) {  /* No need to paint this line */
		l->changeposition = 1;
		l->changeend = 0;
	    }
	} else if (len >= 0) {
	    /*
	     * This is the end of a run.  Erase the run.
	     */
	    FlushErases(c, i, len);
	    len = -1;
	}
    }
    /*
     * Catch any trailing run needing to be erased.
     */
    if (len >= 0)
	FlushErases(c, i, len);

    /*
     * Finally, perform any partial line
     * erasures and/or text painting required.
     */
    for (sp = screen, i = 0; i < LinesPerScreen; sp++, i++) {
	l = sp->lineptr;
	/*
	 * If the line shrank, erase visible text past eol.
	 */
	pos = l->changeposition;
	end = l->changeend;
	if (l->length < sp->visible &&
	    (l->length + 1 < sp->visible || !(l->prop[l->length] & SelModes)))
	    EraseRectangle(l->length, i, sp->visible - l->length, 1);
	if (end > l->length)
	    end = l->length;
	if (pos <= end) {
	    /*
	     * If replacing text, erase replaced text.
	     */
	    if (pos < sp->visible)
		EraseRectangle(pos, i, end+1 - pos, 1);
	    l->body[l->length] = ' ';
	    ep = &l->prop[end + 1];
	    for (cp = &l->prop[pos]; cp < ep; pos += len) {
		/*
		 * Calculate longest sub-string of
		 * changed text with identical properties.
		 */
		tp = cp;
		for (c = *cp++; c == *cp && cp < ep; cp++)
		    ;
		len = cp - tp;
		/*
		 * Paint sub-string according to properties.
		 * Similar to but not same as RenderCursor
		 * since the cleararea of these characters
		 * was already batched with all of the other
		 * characters that changed on this line.
		 */
		if (c & ReverseVideoLook)
		    CursorUp(pos, i, &l->body[pos], len);
		else
		    PaintChars(pos, i, &l->body[pos], len);
		if (c & UnderlineLook) {
		    if (c & ReverseVideoLook)
			WhiteLine(pos, i, len);
		    else
			UnderLine(pos, i, len);
		}
	    }
	    l->changeposition = l->length + 1;
	    l->changeend = 0;
	}
	sp->visible = l->length;
	if (sp->visible < CharsPerLine
	    && (l->prop[sp->visible] & (ReverseVideoLook | UnderlineLook)))
	    ++sp->visible;
    }

    if (Repairing) {
	EndRepair();
	Repairing = 0;
    }
    DoneDisplaying();
    ++outstanding_displays;
    Damage = 0;
    FlushPostScript();
    lastOutputLine = screen[Dot.y].lineptr;
}
