/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static char sccsid[] = "@(#)io.c 23.2 90/06/19 Copyright 1985 Sun Micro";
#endif

/*
 * Copyright (c) 1985 by Sun Microsystems, Inc. 
 */

#ifdef SYSV
#include <stropts.h>
#include <poll.h>
#endif
#include <sys/types.h>
#include <ref/config.h>
#include <stdio.h>
#include <NeWS/psio.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include "tcap.h"
#include "screen.h"

#define Unsgn8		unsigned char

extern	int errno;
extern	int PageMode;
extern	int PageFull;
extern	PSFILE *Client, *ClientInput;

extern int outstanding_displays;	/* displays sent to server */
static	display_called = 0;		/* no screen output waiting */
extern struct pair Dot;

#define PTY 0
#define KEYBOARD 1
#define POSTSCRIPT 2
#define NFDS 3

#define	MAX(a,b)	((a)>(b)?(a):(b))

#ifndef SYSV

int	rf, wf, ef, max, fdmask[NFDS];

#define INIT_SELECT()						\
    fdmask[PTY] = 1 << psio_fileno(Client);			\
    fdmask[KEYBOARD] = 1 << psio_fileno(PostScriptInput);	\
    fdmask[POSTSCRIPT] = 1 << psio_fileno(PostScript);		\
    max = MAX(psio_fileno(Client), psio_fileno(PostScriptInput)) + 1

#define CLEAR_SELECT()		(wf = 0, rf = 0)
#define WRITE_SELECT(type)	(wf |= fdmask[type])
#define READ_SELECT(type)	(rf |= fdmask[type])

#define DO_SELECT()		((ef = wf | rf), select(max, &rf, &wf, &ef, 0))

#define CHECK_WRITE(type)	(wf & fdmask[type])
#define CHECK_READ(type)	(rf & fdmask[type])
#define CHECK_EXCEPTION()	(ef)

#define PERROR_STR		"select"
#define NDELAY_FLAG		FNDELAY
#define NDELAY_ERRNO		EWOULDBLOCK

#else /* SYSV */

struct pollfd	rwf[NFDS];

#define INIT_SELECT()						\
    poll[PTY].fd = psio_fileno(Client);				\
    poll[KEYBOARD].fd = psio_fileno(PostScriptInput);		\
    poll[POSTSCRIPT].fd = psio_fileno(PostScript);

#define CLEAR_SELECT()		(poll[PTY].events = 0,		\
				 poll[KEYBOARD].events = 0,	\
				 poll[POSTSCRIPT].events = 0)
#define WRITE_SELECT(type)	(rwf[type].events |= POLLOUT)
#define READ_SELECT(type)	(rwf[type].events |= POLLIN)

#define DO_SELECT()		(poll(&poll, NFDS, -1))

#define CHECK_WRITE(type)	(rwf[type].revents & POLLOUT)
#define CHECK_READ(type)	(rwf[type].revents & POLLIN)
#define POLLEXCEPT		(POLLHUP | POLLERR | POLLNVAL)
#define CHECK_EXCEPTION()	(poll[PTY].revents & POLLEXCEPT ||
				 poll[KEYBOARD].revents & POLLEXCEPT ||
				 poll[POSTSCRIPT].revents & POLLEXCEPT)

#define PERROR_STR		"poll"
#define NDELAY_FLAG		O_NDELAY
#define NDELAY_ERRNO		EIO

#endif /* SYSV */

/*
 * Input dispatcher: take data from postscript
 * program and pty, dispatching each to the
 * appropriate handler.
 */
terminal()
{
    PageFull = 0;			/* not inhibiting scrolling */

    (void) fcntl(psio_fileno(Client), F_SETFL,
		 fcntl(psio_fileno(Client), F_GETFL, 0)|NDELAY_FLAG);
    (void) fcntl(psio_fileno(PostScriptInput), F_SETFL,
		 fcntl(psio_fileno(PostScriptInput), F_GETFL, 0)|NDELAY_FLAG);

    INIT_SELECT();

    /* Force an initial synchronization with the server. */
    DoneDisplaying();
    ++outstanding_displays;

    while (transferio(outstanding_displays) == 0)
	tc_refresh(0);
}

transferio(blockoutput)
{
    CLEAR_SELECT();
    if (psio_availinputbytes(PostScriptInput)) {
	/* select won't fire off to tell us we have chars in the buffer */
	if (kbd_input())
	    return -1;
	else if (blockoutput && outstanding_displays == 0)
	    /* blockoutput means wait for outstanding_displays to go to 0 */
	    return 0;
    }
    READ_SELECT(KEYBOARD);
    if (psio_outblocked(PostScript))
	WRITE_SELECT(POSTSCRIPT);
    else if (!blockoutput)
	READ_SELECT(PTY);
    if (psio_needsflush(Client))
	WRITE_SELECT(PTY);
    while (DO_SELECT() < 0) {
	if (errno != EINTR) {
	    perror(PERROR_STR);
	    return -1;
	}
    }
    if (CHECK_EXCEPTION())
	return -1;
    /*
     * Flush PostScript descriptor.
     */
    if (CHECK_WRITE(POSTSCRIPT))
	psio_flush_nb(PostScript);
    /*
     * Try to flush pty, if clogged, before reading from keyboard.
     */
    if (CHECK_WRITE(PTY))
	psio_flush_nb(Client);
    /*
     * Take keyboard input.
     */
    if (CHECK_READ(KEYBOARD))
	if (kbd_input())
	    return -1;
    /*
     * Finally, take pty stuff and send it to the display
     * except when we're already flush with output.
     */
    if (!psio_outblocked(PostScript) && CHECK_READ(PTY))
    {
	tc_display();
    }

    return 0;
}

/*
 * Flush PostScript destined for window.
 */
FlushPostScript()
{
    psio_flush_nb(PostScript);
}

/*
 * Handle keyboard input + postscript events.
 */
static
kbd_input()
{
    int tag, ret;
    int	col, row, rank, level, synch_id, preview;
    int c;
    char str[1];

    while (ps_check_PostScript_event() > 0) {
	if ((ret = ps_peek_PostScript_event(&tag)) < 0) {
	    if (psio_error(PostScriptInput))
		perror("ps_peek_PostScript_event");
	    break;
	}
	if (ret == 0) {
	    c = psio_getc(PostScriptInput);
	typed_char:
	    if (PageMode) {
		if (PageFull) {
		    scrollreset(c == '\r');
		    if (c == '\r' || c == ' ')
			continue;
		} else
		    scrollreset(0);
	    }
	    psio_putc(c, Client);
	} else {
	    switch (tag) {
	    case EVENT_TOGGLEAM:
		toggleautomargins();
	    eat_ps_event:
		ps_read_PostScript_event(&tag);
		break;

	    case EVENT_TOGGLEPM:
		togglepagemode();
		goto eat_ps_event;
		break;

	    case EVENT_REPAIR:
		/* eat tag before calling do_damage because of recursion */
		ps_read_PostScript_event(&tag);
		do_damage();
		break;

	    case EVENT_RESIZE:
		ReInitialize();		/* should get EVENT_REPAIR next */
		goto eat_ps_event;
		break;

	    case EVENT_SETSEL:		/* col row rank level */
		ps_event_setsel(&row, &col, &rank, &level, &preview);
		tc_select_at(col, row, rank, level, preview);
		break;

	    case EVENT_EXTENDSEL:		/* col row rank level */
		ps_event_extsel(&row, &col, &rank, &level, &preview);
		tc_extend_to(col, row, rank, level, preview);
		break;

	    case EVENT_DESELECT:		/* rank */
		ps_event_desel(&rank);
		tc_deselect(rank);
		break;

	    case EVENT_SELREQUEST:		/* rank */
		ps_event_selrequest(&rank, &synch_id);
		tc_send_contents(rank, synch_id);
		break;

	    case EVENT_INSELECTION:		/* col row rank synch-id */
		ps_event_insel(&row, &col, &rank, &synch_id);
		tc_in_selectionp(col, row, rank, synch_id);
		break;

	    case EVENT_DONEDISPLAY:
		if (--outstanding_displays <= 0)
		    if (!Damage)
			PlaceCursor(Dot.x, Dot.y);
		goto eat_ps_event;
		break;

	    case EVENT_TYPEDCHAR:
		ps_event_char(str, 1);
		c = str[0];
		goto typed_char;

	    default:
		psio_fprintf(psio_stderr, "%d: what tag is this?\n", tag);
		goto eat_ps_event;
		break;
	    }
	}
    }
    if (psio_needsflush(Client))
	psio_flush_nb(Client);
    return(psio_eof(PostScriptInput) || psio_error(PostScriptInput));
}
