/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static char sccsid[] = "@(#)selection.c 23.2 90/06/19 Copyright 1987 Sun Micro";
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc. 
 */

/*-
	selection.c

	selection.c, Fri Feb 27 23:51:49 1987

		Robin Schaufler,
		Sun Microsystems
 */

#ifdef	notdef
#define Diagnose
#endif
/*
 * Selection module 
 *
 * The external interfaces of this module are
 * tc_select_at(col, row, rankm, level)
 * tc_extend_to(col, row, rank, level)
 * tc_deselect(rank)
 */

#include	<assert.h>
#include	<ctype.h>
#include	<sys/types.h>
#include	<ref/config.h>
#include	"screen.h"
#include	"tcap.h"

struct selection {
    struct range	range;
    char		*contents;
    unsigned		active:1;
    unsigned		server_valid:1;
    unsigned		sendpath:1;
    unsigned		preview:1;
} selections[4];

#define CHARACTER_UNIT	1
#define WORD_UNIT	2
#define LINE_UNIT	3

#define YES		1
#define NO		0

extern Unsgn16	    CharsPerLine;
extern Unsgn16	    LinesPerScreen;
extern struct pair  Dot;
struct pair	    dot_pivot;
int		    dot_pivot_valid;

static Unsgn8	    rankModes[4] = { 0, PrimSelMode, SecnSelMode, 0 };

tc_init_selection()
{
    struct selection	*sp;
    int			i;

    for (i = 1; i <= 2; i++) {
	sp = &selections[i];

	sp->contents = (char *) malloc((CharsPerLine * LinesPerScreen) + 1);
	sp->active = sp->server_valid = sp->sendpath = sp->preview = 0;
    }
}

invalidate_selections()
{
    dot_pivot_valid = NO;
}

/*
 * For simplicity sake, selections are maintained as character properties.
 * This way, when the screen contents change, no ranges need to be updated.
 * However this is slow, when doing extends, exactly the operation you
 * want to be fast; eventually a set of ranges should be maintained.
 */

static void
SelectAt(col, row, rank, level, preview)
    register int	col, row, rank, level, preview;
{
    register struct line    *linep;
    struct selection	    *sp;

    if (col >= CharsPerLine)
	col = CharsPerLine - 1;
    else if (col < 0)
	col = 0;
    if (row >= LinesPerScreen)
	row = LinesPerScreen - 1;
    else if (row < 0)
	row = 0;
    linep = screen[row].lineptr;
    if (col > linep->length) {
	col = linep->length;
	linep->body[col] = ' ';
    }
    linep->prop[col] |= rankModes[rank];
    DamageLine(linep, col);
    ++Damage;
    dot_pivot.x = col;
    dot_pivot.y = row;
    sp = &selections[rank];
    sp->contents[0] = linep->body[col];
    sp->contents[1] = '\0';
    sp->active = 1;
    sp->preview = preview;
    sp->range.first = dot_pivot;
    if (col + 1 > linep->length && row < LinesPerScreen - 1) {
	sp->range.last_plus_one.y = row + 1;
	sp->range.last_plus_one.x = 0;
    } else {
	sp->range.last_plus_one.y = row;
	sp->range.last_plus_one.x = col + 1;
    }
    dot_pivot_valid = YES;
    if (sp->server_valid) {
	sp->server_valid = 0;
	ps_invalidate_selection(rank);
    }
}

static
Deselect(rank)
    register int rank;
{
    register int    i;
    struct selection	*sp;

    sp = &selections[rank];
    sp->active = 0;
    sp->preview = 0;
    *(sp->contents) = '\0';
    for (i = 0; i < LinesPerScreen; i++) {
	register struct line *l = screen[i].lineptr;
	register int j;

	for (j = 0; j <= l->length; j++) {
	    if (l->prop[j] & rankModes[rank]) {
		DamageLine(l, j);
		++Damage;
		l->prop[j] &= ~rankModes[rank];
	    }
	}
    }
    if (sp->server_valid) {
	sp->server_valid = 0;
	ps_invalidate_selection(rank);
    }
    dot_pivot_valid = NO;
}

static struct range
FindRange(rank)
    register int rank;
{
    register int i;
    struct range range;

    range.first.x = range.first.y = -1;
    range.last_plus_one.x = range.last_plus_one.y = -1;

    for (i = 0; i < LinesPerScreen; i++) {
	register struct line *l = screen[i].lineptr;
	register int j;

	for (j = 0; j <= l->length; j++) {
	    if (range.first.x == -1) {
		if (l->prop[j] & rankModes[rank]) {
		    range.first.x = j;
		    range.first.y = i;
		}
	    } else {
		if (! (l->prop[j] & rankModes[rank])) {
		    range.last_plus_one.x = j;
		    range.last_plus_one.y = i;
		    break /* for j */;
		}
	    }
	}
	if (range.last_plus_one.x > -1)
	    break;
    }
    if (range.first.x > -1 && range.last_plus_one.x == -1) {
	range.last_plus_one.y = LinesPerScreen - 1;
	range.last_plus_one.x
	    = screen[LinesPerScreen-1].lineptr->length;
    }
    return range;
}

static int
GetRangeContents(rangep, rank)
    register struct range *rangep;
{
    register int i, len = -1;
    register struct line *l;
    register char *cp = selections[rank].contents;

    *cp = '\0';
    if (!rangep || rangep->first.x == -1)
	return 0;
    i = rangep->first.y;
    l = screen[i].lineptr;
    if (rangep->last_plus_one.y == i) {
	len = rangep->last_plus_one.x;
	if (len > l->length)
	    len = l->length;
    } else {
	len = l->length;
    }
    len -= rangep->first.x;
    strncpy(cp, l->body + rangep->first.x, len);
    cp += len;

    for (i = rangep->first.y + 1; i < rangep->last_plus_one.y; i++) {
	l = screen[i].lineptr;
	if (screen[i-1].lineptr->length < CharsPerLine)
	    *cp++ = '\n';
	strncpy(cp, l->body, l->length);
	cp += l->length;
    }

    if (i == rangep->last_plus_one.y) {
	l = screen[i].lineptr;
	if (i > rangep->first.y) {
	    if (screen[i-1].lineptr->length < CharsPerLine)
		*cp++ = '\n';
	    if ((len = rangep->last_plus_one.x) > 0) {
		if (len > l->length)
		    len = l->length;
		strncpy(cp, l->body, len);
		cp += len;
	    }
	}
    } else {
	i = rangep->last_plus_one.y;
	l = screen[i].lineptr;
    }
    if (rangep->last_plus_one.x > l->length)
	*cp++ = '\n';
    len = cp - selections[rank].contents;
    selections[rank].contents[len] = '\0';

    return len;
}

/* Highlight the selection.  oldrange is the range of the selection
   BEFORE the current redisplay and newrange is the range of the
   selection AFTER the redisplay.  This routine adjusts the selection
   without blinking by toggling the property of the lines where the two
   regions DO NOT overlap.  This also deals with the case that there is
   no selection (oldrange is null).

   This depends on the two selections overlapping, but it wouldn't be
   that hard to make it smart enough to deal with that situation, too. */

static int
pair_before(p1, p2)
register struct pair	*p1, *p2;
{
    return (p1->y < p2->y || (p1->y == p2->y && p1->x < p2->x));
}

static void
toggle_line_prop(linep, start, length, rank)
    register struct line *linep;
    register int start, length, rank;
{
    register int    i;
    Unsgn8	    rank_prop;

    if (length <= 0)
	return;
    rank_prop = rankModes[rank];
    DamageLine(linep, start);
    DamageLine(linep, start+length-1);
    ++Damage;
    for (i = start; i < start + length; i++)
	linep->prop[i] ^= rank_prop;
}

#define null_range(r)	((r)->first.y == (r)->last_plus_one.y && \
			 (r)->first.x == (r)->last_plus_one.x)

static void
toggle_range(range, rank)
struct range	*range;
int		rank;
{
    register struct line    *l;
    register int	    i, len = -1;

    if (null_range(range))
	return;
    i = range->first.y;
    l = screen[i].lineptr;
    if (range->last_plus_one.y == i) {
	len = range->last_plus_one.x;
    } else {
	len = l->length;
    }

    len -= range->first.x;
    toggle_line_prop(l, range->first.x, len, rank);

    for (i = range->first.y + 1; i < range->last_plus_one.y; i++) {
	toggle_line_prop(l, l->length, 1, rank);
	l = screen[i].lineptr;
	toggle_line_prop(l, 0, l->length, rank);
    }

    if (i == range->last_plus_one.y && i > range->first.y) {
	toggle_line_prop(l, l->length, 1, rank);
	l = screen[i].lineptr;
	toggle_line_prop(l, 0, range->last_plus_one.x, rank);
    }
}

static void
SetRange(newrange, oldrange, rank)
    register struct range   *newrange, *oldrange;
    int			    rank;
{
    struct range	    diffrange;

    /* first take care of anything on the left side of the selection */
    if (pair_before(&oldrange->first, &newrange->first)) {
	diffrange.first = oldrange->first;
	diffrange.last_plus_one = newrange->first;
    } else {
	diffrange.first = newrange->first;
	diffrange.last_plus_one = oldrange->first;
    }
    toggle_range(&diffrange, rank);

    /* and now take care of changes on the right hand side of the selection */
    if (pair_before(&oldrange->last_plus_one, &newrange->last_plus_one)) {
	diffrange.first = oldrange->last_plus_one;
	diffrange.last_plus_one = newrange->last_plus_one;
    } else {
	diffrange.first = newrange->last_plus_one;
	diffrange.last_plus_one = oldrange->last_plus_one;
    }
    toggle_range(&diffrange, rank);
}

#define wordc(c)    (isalnum(c) || (c == '_'))

static void
expand_range(range, level)
struct range	*range;
int		level;
{
    struct line	*linep;
    int	col;

    if (level == CHARACTER_UNIT)
	return;
    if (level == WORD_UNIT) {
	int inword;

	/* first the front part */
	linep = screen[range->first.y].lineptr;
  	col = range->first.x;
	if (wordc(linep->body[col]))
	    while (col > 0 && wordc(linep->body[col - 1]))
		col -= 1;
	range->first.x = col;

	/* now the last part */
	if (range->last_plus_one.x > 0) {
	    col = range->last_plus_one.x - 1;
	    linep = screen[range->last_plus_one.y].lineptr;
	    while (col < linep->length && wordc(linep->body[col]))
		col += 1;
	    range->last_plus_one.x = col;
	}
    } else if (level == LINE_UNIT) {
	range->first.x = 0;
	if (range->last_plus_one.x != 0) {
	    if (range->last_plus_one.y == LinesPerScreen - 1)
		range->last_plus_one.x = screen[range->last_plus_one.y].lineptr->length;
	    else {
		range->last_plus_one.y += 1;
		range->last_plus_one.x = 0;
	    }
	}
    }
}

static void
ExtendTo(col, row, rank, level, preview)
    register int    col, row;
    int		    rank, level, preview;
{
    register struct line    *linep;
    struct selection	    *sp;
    struct range	    oldrange, diffrange, newrange;
    unsigned int	    relation;
    struct pair		    newpair;

    if (col >= CharsPerLine)
	col = CharsPerLine - 1;
    else if (col < 0)
	col = 0;
    if (row >= LinesPerScreen)
	row = LinesPerScreen - 1;
    else if (row < 0)
	row = 0;
    sp = &selections[rank];
    assert(sp->active);
    sp->preview = preview;

    /* Only find the range in case we might not already know it.
       If the screen scrolls in the middle of dragging out a selection
       this might get confused, but, well, you get what you deserve. */
    if (dot_pivot_valid == NO)
	sp->range = FindRange(rank);
    oldrange = sp->range;
    if (oldrange.first.x == -1) {
	SelectAt(col, row, rank, level, preview);
	return;
    }
    linep = screen[row].lineptr;
    if (col > linep->length)
	col = linep->length;

    newpair.x = col;
    newpair.y = row;

    /* Although this is an adjust, it's a click down, so we
       should calculate the new pivot point. */
    if (dot_pivot_valid == NO) {
	if (pair_before(&newpair, &oldrange.first))
	    dot_pivot = oldrange.last_plus_one;
	else
	    dot_pivot = oldrange.first;
	dot_pivot_valid = YES;
    }

    if (pair_before(&newpair, &dot_pivot)) {
	newrange.first = newpair;
	newrange.last_plus_one = dot_pivot;
    } else {
	newpair.x += 1;
	if (newpair.x > screen[newpair.y].lineptr->length) {
	    if (newpair.y == LinesPerScreen - 1)
		newpair.x = screen[newpair.y].lineptr->length;
	    else {
		newpair.x = 0;
		newpair.y += 1;
	    }
	}
	newrange.first = dot_pivot;
	newrange.last_plus_one = newpair;
    }

    if (level > CHARACTER_UNIT)
	expand_range(&newrange, level);

    linep = screen[newrange.first.y].lineptr;
    if (newrange.first.x > linep->length)
	newrange.first.x = linep->length;
    sp->range = newrange;
    dot_pivot_valid = preview;
    SetRange(&newrange, &oldrange, rank);
    if (sp->server_valid) {
	sp->server_valid = 0;
	ps_invalidate_selection(rank);
    }
    if (!preview)
	(void) GetRangeContents(&newrange, rank);
}

/* --------------- External Routines Below ------------------- */

tc_select_at(col, row, rank, level, preview)
    int	col, row, rank, preview;
{
    SelectAt(col, row, rank, level, preview);
}

tc_extend_to(col, row, rank, level, preview)
    int	col, row, rank, level, preview;
{
    ExtendTo(col, row, rank, level, preview);
}

tc_deselect(rank)
    int rank;
{
    if (rank == 1 || rank == 2)
	Deselect(rank);
    
}

tc_send_contents(rank, synch_id)
    int	rank, synch_id;
{
    struct selection	*sp;
    struct range	range;
    int			len;

    range = FindRange(rank);
    sp = &selections[rank];
    if (sp->preview)
	len = GetRangeContents(&range, rank);
    else
	len = strlen(sp->contents);
    SendSelection(sp->contents, len);
    ContinueProcess(synch_id);
    FlushPostScript();
    sp->server_valid = 1;
}

tc_in_selectionp(col, row, rank, synch_id)
int row, col, rank, synch_id;
{
    struct range    range;
    struct pair	    point;
    int		    inside;

    range = FindRange(rank);
    point.x = col;
    point.y = row;
    inside = !(pair_before(&point, &range.first) ||
	       pair_before(&range.last_plus_one, &point));
    SetInSelection(inside);
    ContinueProcess(synch_id);
    FlushPostScript();
}
