/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static	char sccsid[] = "@(#)tcap_parse.c 23.2 90/06/19 Copyright 1985 Sun Micro";
#endif


/*
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

/*-
	tcap_parse.c:  Parse termcap output based on termcap entry.

	tcap_parse.c, Mon Mar 24 11:25:44 1986

		David Rosenthal,
		Sun Microsystems
 */

#include	<sys/errno.h>
#include	<NeWS/psio.h>
#include	<ctype.h>
#include	<sys/types.h>
#include	<ref/config.h>
#include	"termcap.h"

extern int errno;
extern int Damage;
extern PSFILE *ClientInput;
/*  Import these from tcap_ops.c */
extern struct tcap T[];
extern int Ts;
extern int PageFull;
extern int PageMode;
/*
 * Initialize the display system from the TERMCAP entry.
 * We parse the entry and build the tcap structures
 * describing the operations supported by this type of
 * terminal.  These descriptions are then used by tc_display()
 * in interpreting the data stream generated by the
 * application
 */
int
tc_initialize(term)
    char *term;
{
#ifdef SUNTGETENT
    static void tc_fix_tcap_ent();
#endif
    static char tcapbuf[1024], tcaparea[1024], *unpad();
    char       *areap = tcaparea;
    extern int tgetent(), tgetnum(), tgetflag();
    extern char *tgetstr();
    register struct tcap *tp;
    char	*ret, *tc_insert();

    if (tgetent(tcapbuf, term) != 1)		/* unknown terminal type? */
	return (1);
#ifdef SUNTGETENT
    tc_fix_tcap_ent(tcapbuf);
#endif
    for (tp = T; tp < T+Ts; tp++) {
	switch (tp->t_type) {
	case string:
	    tp->t_text = unpad(tgetstr(tp->t_key, &areap));
	    if (tp->t_text == NULL)
		tp->t_text = tp->t_deftext;
	    break;
	case num:
	    tp->t_x = tgetnum(tp->t_key);
	    break;
	case bool:
	    tp->t_x = tgetflag(tp->t_key);
	    break;
	}
	/* invoke any initialize routine */
	if (tp->t_in && (*tp->t_in)(tp))
	    return (1);
	if (tp->t_text)
	    if (ret = tc_insert(tp)) {
		printf ("Termcap parsing failed:  %s\n", ret);
		return (1);
	    }
    }
    tc_init_ops();
    return (0);
}

#ifndef SYSV
static char *
unpad(s)
    register char *s;
{

    if (s) {
	register pad = 0;

	while (isdigit(*s))
	    pad++, s++;
	if (pad && *s == '*')
	    s++;
    }
    return (s);
}

#else	/* SYSV */

/*
 * Remove the substring of the form "$<x^>" where x = number, and ^ = characters
 * in the set [* /]. This is the terminfo way of specifying delays or padding.
 */
static char *
unpad(s)
    register char *s;
{

    if (s) {
	register char *spt = s;
	register char *spt1, *spt2;
	char *strchr();

	while (spt1 = strchr(spt, '$')) {
		if (*(spt1 + 1) == '<') {	/* found the '$<' pair */
			if (spt2 = strchr(spt1, '>')) {
				strcpy(spt1, ++spt2);	/* found end '>' */
				spt = spt1;		/* copy tail of */
				continue;		/* string over  */
							/* '$<..', look */
							/* for more */
			} else
				break;		/* no match for '$<' so quit */
		} else {
			spt = spt1 + 1;		/* found '$' but no '<', */
			continue;		/* look for more */
		}
			
	}
    }
    return (s);
}
#endif	/* SYSV */


#ifdef SUNTGETENT
#define TCAPBUFSIZE 1024
#define SPECIALSIZE 2

static struct {
    char	*last;
    char	 str[3];
}   tcapSpecials[SPECIALSIZE] = {
    NULL,	"co",
    NULL,	"li"
};

/*
 * Stomp on the first "co" and "li" entries in the termcap entry
 * to avoid braindamage in the Sun version of the termcap library.
 * Apparently the Sun version of tgetent() looks at the terminal
 * state and uses this to prepend extra line+column spec's that
 * reflect the terminal's current state.  This is not what we want,
 * so our only recourse is to undo the this braindamage here.
 */
static void
tc_fix_tcap_ent(buf)
    char *buf;
{
    char *bp = buf;
#ifndef SYSV
    char *index();
#else
#define index(s, c)	(char *)strchr(s, c)
#endif
    int   i;

    /* for each item in buf ... */
    for (bp = index(bp, ':'); bp && *(bp+1); bp = index(bp, ':')) {
	++bp;
	/* for each special tcap code ... */
	for (i = 0; i < SPECIALSIZE; i++) {
	    if (strncmp(tcapSpecials[i].str, bp, 2) == 0) {
		if (tcapSpecials[i].last)
		    strncpy(tcapSpecials[i].last, "xx", 2);
		tcapSpecials[i].last = bp;
		break;
	    }
	}
    }
}
#endif /* SUNTGETENT */

typedef void tc_func();

enum tc_type {
    TC_UNKNOWN,
    TC_CHAR,
    TC_OPERAND,
    TC_LOOKUP,
    TC_NLOOKUP,
    TC_OPERATE,
};

enum tc_optype {
    TCO_DECIMAL = 'd',
    TCO_0DECIMAL = '0',
    TCO_1DECIMAL = '1',
    TCO_2DECIMAL = '2',
    TCO_3DECIMAL = '3',
    TCO_CHARACTER = '.',
};

typedef struct tc_cell {
    enum tc_type	type;
    struct tc_cell	*next;
    union {
	struct tc_cell	**lookup;
	struct tcap	*operator;
	unsigned char	character;
	struct {
	    enum tc_optype	type;
	    int			seed;
	} operand;
    } value;
} TC_cell;

TC_cell	*tc_global[256] = {0};
#define NULL_CELL	((TC_cell *)NULL)
#define NUM_CELLS	256
static char Out_of_memory[] = "Out of memory";

TC_cell *
alloc_cell()
{
    static TC_cell	*cache_ptr = NULL_CELL,
			*cache = NULL_CELL;
    if (cache_ptr == cache) {
	cache = (TC_cell *) malloc(NUM_CELLS * sizeof(TC_cell));
	if (cache == NULL_CELL)
	    return NULL_CELL;
	cache_ptr = &cache[NUM_CELLS - 1];
    }
    return (cache_ptr--);
}

char *
expand_oper_to_lookup(tc_ptr)
register TC_cell	**tc_ptr;
{
    register unsigned char	type;
    register unsigned 		seed, digit;
    register TC_cell		*tc_temp;

    type = (*tc_ptr)->value.operand.type;
    if (type == TCO_CHARACTER)
	return "Constant and character operand parse to same location";
    if (type == TCO_0DECIMAL)
	return "Digit string conflicts with fixed size operand";
    seed = (*tc_ptr)->value.operand.seed * 10;
    (*tc_ptr)->type = TC_NLOOKUP;
    (*tc_ptr)->value.lookup =
	(TC_cell **) malloc (256 * sizeof(TC_cell *));
    if ((*tc_ptr)->value.lookup == (TC_cell **)NULL)
	return Out_of_memory;
    bzero((*tc_ptr)->value.lookup, 256 * sizeof(TC_cell *));
    if ((tc_temp = alloc_cell()) == NULL_CELL)
	return Out_of_memory;
    tc_temp->type = TC_OPERAND;			/* Escape for operand search */
    tc_temp->value.operand.type = type;
    tc_temp->value.operand.seed = seed / 10;
    tc_temp->next = (*tc_ptr)->next;
    (*tc_ptr)->value.lookup['\0'] = tc_temp;
    if (type != TCO_DECIMAL)
	type--;
    for (digit = '0'; digit <= '9'; digit++) {
	if ((tc_temp = alloc_cell()) == NULL_CELL)
	    return Out_of_memory;
	tc_temp->type = TC_OPERAND;
	tc_temp->next = (*tc_ptr)->next;
	tc_temp->value.operand.type = type;
	tc_temp->value.operand.seed = seed + (digit - '0');
	(*tc_ptr)->value.lookup[digit] = tc_temp;
    }
    (*tc_ptr)->next = NULL_CELL;
    return 0;
}

char *
expand_char_to_lookup(tc_ptr)
register TC_cell	**tc_ptr;
{
    register TC_cell	*tc_temp;

    if ((tc_temp = alloc_cell()) == NULL_CELL)
	return Out_of_memory;
    tc_temp->type = TC_LOOKUP;
    tc_temp->next = NULL_CELL;
    tc_temp->value.lookup =
	(TC_cell **) malloc (256 * sizeof(TC_cell *));
    if (tc_temp->value.lookup == (TC_cell **)NULL)
	return Out_of_memory;
    bzero(tc_temp->value.lookup, 256 * sizeof(TC_cell *));
    tc_temp->value.lookup[(*tc_ptr)->value.character] =
	(*tc_ptr)->next;
    *tc_ptr = tc_temp;
    return 0;
}

char *
insert_oper_in_lookup(tc_ptr_arg, tc_common, tc_link, type, seed)
TC_cell			**tc_ptr_arg;
TC_cell			***tc_common;
TC_cell			**tc_link;
register enum tc_optype	type;
register unsigned	seed;
{
    register TC_cell	**tc_ptr, *tc_temp;
    register unsigned	digit;
    char		*ret;

    if (type == TCO_CHARACTER)
	return "Constant and character operand parse to same location";
    if (type == TCO_0DECIMAL)
	return "Digit constant too long for operand match";
    if ((*tc_ptr_arg)->type != TC_NLOOKUP) {
	(*tc_ptr_arg)->type = TC_NLOOKUP;
	if ((tc_temp = alloc_cell()) == NULL_CELL)
	    return Out_of_memory;
	tc_temp->type = TC_OPERAND;		/* Escape for operand search */
	tc_temp->value.operand.type = type;
	tc_temp->value.operand.seed = seed / 10;
	tc_temp->next = *tc_link;
	*tc_link = tc_temp;
    }
    if (type != TCO_DECIMAL)
	type--;
    for (digit = '0'; digit <= '9'; digit++, seed++) {
	tc_ptr = &(*tc_ptr_arg)->value.lookup[digit];
	while (*tc_ptr) {
	    switch ((*tc_ptr)->type) {
	    case TC_CHAR:
		if (ret = expand_char_to_lookup(tc_ptr))
		    return ret;
		/* fall through */
	    case TC_NLOOKUP:
	    case TC_LOOKUP:
		if (ret = insert_oper_in_lookup(tc_ptr, tc_common, tc_link,
						type, seed * 10))
		    return ret;
		goto tc_inserted;
	    case TC_OPERAND:
		if ((*tc_ptr)->value.operand.type != type ||
		    (*tc_common && **tc_common != (*tc_ptr)->next))
		    return "Different operand types parse to same location";
		*tc_common = &(*tc_ptr)->next;
		goto tc_inserted;
	    case TC_OPERATE:
		tc_ptr = &(*tc_ptr)->next;
		break;
	    }
	}
	if ((tc_temp = alloc_cell()) == NULL_CELL)
	    return Out_of_memory;
	tc_temp->type = TC_OPERAND;
	tc_temp->value.operand.type = type;
	tc_temp->value.operand.seed = seed;
	*tc_ptr = tc_temp;
	tc_temp->next = *tc_link;
	*tc_link = tc_temp;
    tc_inserted:
	;
    }
    return 0;
}

char *
tc_insert (tp)
    struct tcap		*tp;
{
    TC_cell		**tc_ptr;
    TC_cell		*tc_temp;
    unsigned char	c, *p = (unsigned char *)tp->t_text;
    char		*ret;

    tc_ptr = &tc_global[*p++];

    while (1) {
	if (*tc_ptr == NULL_CELL) {
	    if ((*tc_ptr = alloc_cell()) == NULL_CELL)
		return Out_of_memory;
	    (*tc_ptr)->type = TC_UNKNOWN;
	}
	switch (c = *p++) {
	case '\0':
	    switch ((*tc_ptr)->type) {
	    case TC_UNKNOWN:
		tc_temp = *tc_ptr;
		tc_temp->next = NULL_CELL;
		break;
	    default:
		if ((tc_temp = alloc_cell()) == NULL_CELL)
		    return Out_of_memory;
		tc_temp->next = *tc_ptr;
		*tc_ptr = tc_temp;
		break;
	    }
	    tc_temp->type = TC_OPERATE;
	    tc_temp->value.operator = tp;
	    return 0;
	default:
	insert_character:
	    switch ((*tc_ptr)->type) {
	    case TC_UNKNOWN:
		(*tc_ptr)->type = TC_CHAR;
		(*tc_ptr)->next = NULL_CELL;
		(*tc_ptr)->value.character = c;
		goto next_tc;
	    case TC_CHAR:
		if (c == (*tc_ptr)->value.character)
		    goto next_tc;
		if (ret = expand_char_to_lookup(tc_ptr))
		    return ret;
		/* fall through */
	    case TC_NLOOKUP:
	    case TC_LOOKUP:
	    lookup_next:
		tc_ptr = &(*tc_ptr)->value.lookup[c];
		break;
	    case TC_OPERAND:
		if (ret = expand_oper_to_lookup(tc_ptr))
		    return ret;
		goto lookup_next;
	    case TC_OPERATE:
	    next_tc_operator:
		p--;
		tc_ptr = &(*tc_ptr)->next;
		break;
	    }
	    break;
	case '%':
	    if ((c = *p++) == '\0')
		return "Unexpected end of termcap string";
	    if (c == '%')
		goto insert_character;
	insert_operand:
	    switch ((*tc_ptr)->type) {
	    case TC_UNKNOWN:
		(*tc_ptr)->type = TC_OPERAND;
		(*tc_ptr)->next = NULL_CELL;
		(*tc_ptr)->value.operand.type = c;
		(*tc_ptr)->value.operand.seed = 0;
		goto next_tc;
	    case TC_OPERAND:
		if (c != (*tc_ptr)->value.operand.type)
		    return "Incompatible operand types in same parse location";
	    next_tc:
		tc_ptr = &(*tc_ptr)->next;
		break;
	    case TC_CHAR:
		/* Convert to a lookup with TC_CHAR inserted */
		if (ret = expand_char_to_lookup(tc_ptr))
		    return ret;
	    case TC_NLOOKUP:
	    case TC_LOOKUP: {
		    TC_cell	**tc_common = (TC_cell **)NULL;
		    TC_cell	*tc_link = NULL_CELL;

		    if (ret = insert_oper_in_lookup(tc_ptr, &tc_common,
						    &tc_link, c, 0))
			return ret;
		    if (tc_link == NULL_CELL)
			tc_ptr = tc_common;
		    else {
			if (tc_common == (TC_cell **)NULL) {
			    if ((tc_temp = alloc_cell()) == NULL_CELL)
				return Out_of_memory;
			    tc_temp->type = TC_UNKNOWN;
			} else
			    tc_temp = *tc_common;
			while (tc_link) {
			    tc_ptr = &tc_link->next;
			    tc_link = tc_link->next;
			    *tc_ptr = tc_temp;
			}
		    }
		}
		break;
	    case TC_OPERATE:
		p--;
		goto next_tc_operator;
	    }
	    break;
	}
    }
}

#define CHBUFSIZ	1024	/* number of chars to handle in one shot */
#define PBUFSIZ		10	/* number of params to buffer for one op */

#define pgetc(e)   {						\
    psio_pgetc(client, c,					\
	       if (psio_availinputbytes(ClientInput) <= 0) {	\
		   savecp = cp;					\
		   savetc_ptr = tc_ptr;				\
		   restart_state = e;				\
		   return;					\
	       read_/**/e :					\
		   cp = savecp;					\
		   tc_ptr = savetc_ptr;				\
		   client = ClientInput;			\
	       }						\
	       );						\
    *cp++ = c;							\
}

#define showchars() {			\
    tp = T+Ts;				\
    tp->t_text = (char *)chars;		\
    tp->t_size = cp-chars;		\
    (*tp->t_op)(tp);			\
    ++Damage;				\
}

/*
 * Interpret data from the application.  We match data against
 * the ``escape sequences'' expected for this termcap description
 * and, if successful, invoke the routines used to emulate the
 * capabilities on the window.
 */
tc_display()
{
    register TC_cell	 	*tc_ptr;
    register PSFILE		*client;
    register struct tcap	*tp;
    register unsigned char	c, *cp;
    register			char_pending = 0;
    register TC_cell		*tc_temp;

    static int			restart_state = 0, lim;
    static int			params[PBUFSIZ], *param;
    static TC_cell		*savetc_ptr;
    static unsigned char	chars[CHBUFSIZ], *savecp;
    static unsigned		param_pc_r, param_pc_B, param_pc_n, param_pc_D;

    switch (restart_state) {
    case 0:
	break;
    case 1:
	goto read_1;
    case 2:
	goto read_2;
    case 3:
	goto read_3;
    case 4:
	goto read_4;
    }

    cp = chars;
    client = ClientInput;

    while (1) {
	psio_pgetc(client, c,
		    if (psio_availinputbytes(ClientInput) <= 0) {
			if (cp > chars) {
			    showchars();
			}
			restart_state = 0;
			return;
		    }
		  );
    retry_match:
	if ((tc_ptr = tc_global[c]) == NULL) {
	    *cp++ = c;
	    if (cp == &chars[sizeof(chars)]) {
		showchars();
		cp = chars;
	    }
	} else {
	    if (cp > chars) {
		showchars();
		cp = chars;
	    }
	    *cp++ = c;
	    param = params;
	    param_pc_r = param_pc_B = param_pc_n = param_pc_D = 0;
	    char_pending = 0;		/* We've already looked this one up */
	    while (tc_ptr) {
		switch (tc_ptr->type) {
		case TC_CHAR:
		    if (!char_pending)
			pgetc(1);
		    if (tc_ptr->value.character != c) {
			--cp;
			goto retry_match;
		    }
		    tc_ptr = tc_ptr->next;
		    char_pending = 0;
		    break;
		case TC_OPERAND:
		    *param = tc_ptr->value.operand.seed;
		    switch (tc_ptr->value.operand.type) {
		    case TCO_DECIMAL:
			lim = 127;
			break;
		    case TCO_1DECIMAL:
			lim = 1;
			break;
		    case TCO_2DECIMAL:
			lim = 2;
			break;
		    case TCO_3DECIMAL:
			lim = 3;
			break;
		    case TCO_CHARACTER:
			if (!char_pending)
			    pgetc(2);
			*param = c;
			char_pending = 0;
			/* Drop through */
		    case TCO_0DECIMAL:		/* param is already done */
			goto got_value;
		    }
		    while (--lim >= 0) {
			if (!char_pending)
			    pgetc(3);
			if (!isdigit(c)) {
			    char_pending = 1;
			    break;
			}
			*param = *param * 10 + (c-'0');
			char_pending = 0;
		    }
		got_value:
		    param++;
		    tc_ptr = tc_ptr->next;
		    break;
		case TC_NLOOKUP:
		case TC_LOOKUP:
		    if (!char_pending)
			pgetc(4);
		    tc_temp = tc_ptr->value.lookup[c];
		    if (tc_temp == NULL_CELL) {
			if (tc_ptr->type == TC_NLOOKUP) {
			    tc_temp = tc_ptr->value.lookup['\0'];
			    char_pending = 1;
			} else {
			    --cp;
			    goto retry_match;
			}
		    } else
			char_pending = 0;
		    tc_ptr = tc_temp;
		    break;
		case TC_OPERATE:
		    tp = tc_ptr->value.operator;
		    if (param > params) {
			register int	*p;
			if (tp->t_pc_r != param_pc_r) {
			    if (param > &params[1]) {
				int		temp;

				/* only reverse two params for now */
				temp     = param[0];
				param[0] = param[1];
				param[1] = temp;
			    }
			}
			for (p = params; p < param; p++) {
			    /* BCD encoded */
			    if (tp->t_pc_B != param_pc_B)
				*p  = (10*(*p/16))+(*p%16);
			    /* xor'ed with 0140 */
			    if (tp->t_pc_n != param_pc_n)
				*p ^= 0140;
			    /* reverse encoded */
			    if (tp->t_pc_D != param_pc_D)
				*p  += 2 * (16 - (*p % 16));
			}
			param_pc_r = tp->t_pc_r;
			param_pc_B = tp->t_pc_B;
			param_pc_n = tp->t_pc_n;
			param_pc_D = tp->t_pc_D;
			if ((tp->t_y = params[0]) >= tp->t_yilim)
			    tp->t_y -= tp->t_yi;
			if (param > &params[1]
				    && (tp->t_x = params[1]) >= tp->t_xilim)
			    tp->t_x -= tp->t_xi;
		    }
		    (*tp->t_op)(tp);
		    tc_ptr = tc_ptr->next;
		    ++Damage;
		    /* These characters were used! */
		    cp = chars;
		    if (char_pending)
			*cp++ = c;
		    break;
		}
	    }
	    cp = chars;
	    if (char_pending)
		*cp++ = c;
	}
    }
}
