/*
 * @(#)obj_types.h 23.3 90/06/19 SMI
 */

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

/*
 * Definitions for server object types.  
 *
 * ***** Read *all* comments before making any changes to this file. *****
 *
 * If this list is changed, it is the responsibility of the programmer to:
 *
 *  1)  read *all* comments in this file before adding/deleting entries;
 *
 *  2)  change the constants defined at the end of this file, if necessary;
 *
 *  3)  update *all* server accounting processing to deal with the change: 
 *		reffinder
 *		objectdump (for allocated types)
 *		decode_object()
 *		REF_TRACE code in Core
 *		'type' operator (in PostScript.c initialization)
 *		etc.;
 *
 *  4)  send mail to the doc alias with information about the change: type
 *	name; added/deleted; if added, its use, etc.; other server/processing
 *	changes;
 *
 *  5)  update the comments in this file as appropriate;
 *
 *  6)  understand that changes to this list may necessitate a rebuild of all
 *	fonts, in addition to a server recompile, and schedule check-ins as
 *	appropriate.
 */

#ifndef _obj_types_h
#define _obj_types_h

/*
 * Object type list.  
 *	The order of this list is important.  New types must be added
 *	taking into account the following:
 *
 *	null_type must be the first entry in the list.
 *
 *	the numeric types must immediately follow "null_type" because
 *	    of assumptions in the numeric coercion macros and functions.
 *
 *	type >= REFCOUNTED_TYPE && type < BOUNDING_TYPE: 
 *	    Type is a valid refcounted type (and therefore has an
 *	    entity structure at the root of its data structure).
 *
 *	keyword_type < type > process_type: 
 *	    Type is represented by a corpus data structure.
 *
 *	maximum_type must be the last entry in the list.
 */
enum type {

    null_type,			/* object not refcounted */
    integer_type,		/* WARNING:  The numeric coercion functions */
    fixed_type,			/*	     assume that the numeric types  */
    real_type,			/*	     immediately follow null_type.  */
    boolean_type,
    called_operator_type,
    color_type,
    save_type,
    magic_variable_type,
    marker_type,
    fontid_type,
    operator_type,
    keyword_type,

    array_type,			/* nucleus level - CORPUS */
    packedarray_type,
    cmapentry_type,
    environ_type,
    event_type,
    file_type,
    monitor_type,
    opaque_type,
    shape_type,
    string_type,
    dictionary_type,
    vm_type,

    process_type,		/* nucleus level - non-CORPUS */
    x_interest_type,

    orphan_canvas_type,		/* multi-level */
    xn_canvas_type,
    cs_gc_type,
    x_gc_type,

    colormap_type,		/* cscript level; used/ref'ed elsewhere */
    cursor_type,
    psfont_type,
    visual_type,

    cs_font_type,		/* cscript only; not used/ref'ed elsewhere */
    cs_fontfamily_type,
    cs_transfers_type,
    cs_halftone_type,

    maximum_type,

};

/*
 * Constants used in comparisons to determine:
 *	whether an object has an entity at its root (ENTITY_TYPE)
 *	whether an object is a refcounted type (REFCOUNTED_TYPE)
 *	whether an object is a legal type (less than maximum) (BOUNDING_TYPE)
 *
 * **************************************************************************
 * NOTE: Since the definitions are entries from the enum list above, they
 * 	 may require changing when a new type is added to the list.
 * **************************************************************************
 */

#define ENTITY_TYPE		array_type
#define REFCOUNTED_TYPE		array_type
#define BOUNDING_TYPE		maximum_type
#define NUMERIC_TYPE_MASK	3		/* == int | fixed | real */

#endif 	/* _obj_types_h */
