/*
 * @(#)PostScript.h 23.5 90/06/19 SMI
 */

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

/*
 * PostScript.h - Definitions for the Postscript interpreter.
 */

#ifndef	_PostScript_h
#define _PostScript_h

#include <Core/core.h>
#include <cscript/cscript.h>
#include <nucleus/inputdist.h>
#include <nucleus/dictionary.h>
#include <nucleus/nucleus.h>


#define ValidObjectBits (~3)
#define object_word0(p) *((int *)(p))
#define object_word1(p) *(((int *)(p))+1)
#define object_has_entity(p) ((unsigned) (p)->type >= (unsigned) ENTITY_TYPE)

/*
 * corpus_copy - Especially for use with event corpuses when they are of
 *	         interest to a given canvas.  The corpus is copied; if an
 *		 instance dict exists, it's copied, too.  Unlike bcopy, the 
 *		 from/to parameters are ordered like strcpy().  NOTE that the
 *		 refcount on the "to" object stays the same.
 */
enum error_type dict_corpus_copy();
#define corpus_copy(type, to, from) 					  \
{ 									  \
    DICT *oldd, *newd;							  \
    int	refcnt = to->refcount;						  \
    int softcnt = to->softcount;					  \
    bcopy((char *)(from), (char *)(to), corpus_size(type)); 	 	  \
    if(oldd = instancedict_of(from)) {					  \
	newd = (DICT *) new_composite(NO_POOL,				  \
			(struct object *)NULL, dictionary_type,		  \
	    		((int)(oldd)->size - 1), (char *)NULL,		  \
			oldd->dicttype);				  \
	dict_corpus_copy(newd, oldd);					  \
	set_instancedict(to, newd); 					  \
    } 									  \
    to->refcount = refcnt;						  \
    to->softcount = softcnt;						  \
}

#define UNPACKED_OBJECT_TBLSIZE		256
extern struct object unpacked_objects[];

struct object
    get_packedarray_interval(/* (struct object *)array, int i, int cnt */),
    get_packedarray_object(/*   (struct object *)array, int i */),
    make_executable_array(/* (char *)where, int num_objects, int length */),
    *unpack(/* struct body *b, enum bool keep_old_array */);

#define unset_packed_array_flag(b)\
	((struct corpus *)(b))->corpus.array.packed_size = 0
#define set_packed_array_flag()   syntax error
#define set_packed_array_size(b, size)\
	((struct corpus *)(b))->corpus.array.packed_size = size

/* Test for packed array attached to this entity */
#define ispacked_array_corpus(b)\
    (((struct corpus *)(b))->corpus.array.packed_size != 0)

enum bool {false, true};


/*
* UNALIGNED_OK can be defined true on architectures that
* allow reads and writes of a machine word from any address. 
*/

#if defined(mc68020) | defined(vax) | defined(i386)
#define UNALIGNED_OK 1
typedef struct object * array_element_type;
typedef unsigned int *packed_address;
#elif defined(sparc) | defined(mc68010)
typedef unsigned char * array_element_type;
typedef unsigned char *packed_address;
#define UNALIGNED_OK 0
#else
	/* Break compilation */
	!!! MACHINE WITH UNKNOWN WORD ALIGNMENT ADDRESSING !!! 
#endif

#if UNALIGNED_OK == 1
#define object_copy(a, b)		*a = *b
#define address_copy(a, b)        	*a = *b
#endif

#if UNALIGNED_OK == 0
/*
 * object_copy(a, b)
 * struct object *a;
 * unsigned char *b;
 * Copy a <- b.  'a' must have proper alignment. 'b' can be anywhere.
*/

#define object_copy(a, b) \
{ \
    register unsigned int *uip = (unsigned int *)a; \
    register unsigned char *cp = (unsigned char *)b; \
    register unsigned int r0, r1; \
    cp = (unsigned char *)((unsigned int) cp & ~03); /* Word boundary */ \
    r0 = ((unsigned int *)cp)[0];       /* Fetch 2 words */ \
    r1 = ((unsigned int *)cp)[1]; \
    switch((unsigned int)b & 03) {      /* Switch on the from alignment */ \
    case 01: \
        r0 = (r0 << 8) | ( r1 >> 24); \
        r1 = (r1 << 8) | (((unsigned int *)cp)[2] >> 24); \
        break; \
    case 02: \
        r0 = (r0 << 16) | ( r1 >> 16); \
        r1 = (r1 << 16) | (((unsigned int *)cp)[2] >> 16); \
        break; \
    case 03: \
        r0 = (r0 << 24) | ( r1 >> 8); \
        r1 = (r1 << 24) | (((unsigned int *)cp)[2] >> 8); \
        break; \
    } \
    uip[0] = r0; \
    uip[1] = r1; \
}

/*
 * This operation may shift in garbage. It is used in conjunction
 * with set_*_from_packed_address which forces a proper address.
 */
#define address_copy(a, b) \
{ \
    register unsigned int *uip = (unsigned int *)a; \
    register unsigned char *cp = (unsigned char *)b; \
    register unsigned int r0; \
    cp = (unsigned char *)((unsigned int) cp & ~03); /* Word boundary */ \
    r0 = ((unsigned int *)cp)[0];       /* Fetch a word */ \
    switch((unsigned int)b & 03) {      /* Switch on the from alignment */ \
    case 01: \
        r0 = (r0 << 8) | (((unsigned int *)cp)[1] >> 24); \
        break; \
    case 02: \
        r0 = (r0 << 16) | (((unsigned int *)cp)[1] >> 16); \
        break; \
    case 03: \
        r0 = (r0 << 24) | (((unsigned int *)cp)[1] >> 8); \
        break; \
    } \
    uip[0] = r0; \
}

#endif  /* Unaligned object copy */


#ifndef	lint
/*
 * 	Set fields in an object.  Mostly applies to non-refcounted types.
 */
#define set_fixed_object(p, n) 						   \
  	((object_word0(p) = ((unsigned) fixed_type)<<TYPE_SHIFT),	   \
  	(p)->value.fixed = (n))
#define set_integer_object(p, n)					   \
  	((object_word0(p) = ((unsigned) integer_type)<<TYPE_SHIFT),	   \
  	(p)->value.integer = (n))
#ifndef LITTLEENDIAN
#define set_executable_keyword_from_address(p, addr) (                	   \
	(object_word0(p) =						   \
	    ((((unsigned)(keyword_type))<<TYPE_SHIFT) | 2)		   \
	         | (ADDR_MASK & (addr << 2))),				   \
	(object_word1(p) = 0))
#define set_keyword_from_address(p, addr) (                           	   \
	(object_word0(p) =						   \
	    (((unsigned)(keyword_type))<<TYPE_SHIFT)			   \
		 | (ADDR_MASK & (addr << 2))),				   \
	(object_word1(p) = 0))
#else
#define set_executable_keyword_from_address(p, addr) (                	   \
	(object_word0(p) =						   \
	    ((((unsigned)(keyword_type))<<TYPE_SHIFT) | 2)		   \
	         | (ADDR_MASK & (addr >> 6))),				   \
	(object_word1(p) = 0))
#define set_keyword_from_address(p, addr) (                           	   \
	(object_word0(p) =						   \
	    (((unsigned)(keyword_type))<<TYPE_SHIFT)			   \
		 | (ADDR_MASK & (addr >> 6))),				   \
	(object_word1(p) = 0))
#endif	/* ifndef LITTLEENDIAN */
#define set_keyword_object(p, kwd)					   \
	set_typed_entity_object(p, keyword_type, (ENTITY *)kwd)
#define set_boolean_object(p, n) 					   \
  	((object_word0(p) = ((unsigned) boolean_type)<<TYPE_SHIFT),	   \
  	(p)->value.fixed = (n))
#define set_typed_object(p, type) 					   \
  	(object_word0(p) = ((unsigned) (type))<<TYPE_SHIFT)
#define set_entire_typed_object(p, type) 				   \
  	(object_word0(p) = ((unsigned)(type))<<TYPE_SHIFT, object_word1(p) = 0)
#define clear_object(p) 						   \
	(object_word0(p) = 0, object_word1(p) = 0)
/*
 * 	Set fields in an object, including a pointer to the ENTITY
 * 	that represents the associated type-specific information.
 */
#define set_typed_entity_object(p, type, e) ( 				   \
	(object_word0(p) = (((unsigned)(type))<<TYPE_SHIFT) | (int)(e)),   \
	(object_word1(p) = 0))
#define set_executable_typed_entity_object(p, type, e) ( 		   \
	(object_word0(p) = ((((unsigned)(type))<<TYPE_SHIFT) | 2) | (int)(e)),\
	(object_word1(p) = 0))
/*
 * 	Set fields in an object struct by copying the specified ref.  Used 
 *	when an object struct is already defined, an entity is created with 
 *	the object information, and now the object must reference the entity.  
 *	Copy the information from the ref to word0 of the object.
 */
#define set_object_from_ref(p, ref) (					   \
	(object_word0(p) = refword(&(ref))), (object_word1(p) = 0))
/*
 * 	Test whether two objects are bit-for-bit equal or lesser.
 * 	For equal test, why do we ignore the flag bits (executable and soft)?
 */
#define equal_object(p1, p2) 						   \
  	(((object_word0(p1)^object_word0(p2))&ValidObjectBits) == 0 &&	   \
   	object_word1(p1)==object_word1(p2))
#define lesser_object(p1, p2) 						   \
  	(object_word1(p1)==object_word1(p2) ? object_word0(p1)<object_word0(p2)\
				 : object_word1(p1)<object_word1(p2))

#else	/* lint */

extern unsigned set_fixed_object(/* struct object *, unsigned */);
extern unsigned set_boolean_object(/* struct object *, unsigned */);
extern unsigned set_typed_object(/* struct object *, type */);
extern unsigned set_entire_typed_object(/* struct object *, type */);
extern void clear_object(/* struct object * */);
extern unsigned set_typed_entity_object(/* struct object *, type, ENTITY * */);
extern unsigned set_executable_typed_bodied_object(/* struct object *, type, ENTITY * */);
extern unsigned set_object_from_ref(/* struct object *, REF */);
extern unsigned equal_object(/* struct object *,struct object * */);
extern unsigned lesser_object(/* struct object *, struct object * */);

#endif	/* lint */



#define object_incref(objp) incref(getref(objp))
#define object_decref(objp) decref(getref(objp))

/* 
 * struct corpus - used for objects that exist only at the nucleus level;
 *
 * 	NOTE that union section names must match the type name (minus '_type').
 *	This is necessary for macros (like corpus_size()) that use the type 
 *	name to work.
 */
struct corpus {
    ENTITY_FIELDS				/* header and ref info */

    union {
        struct {                                /* == array_type: growable */
            unsigned short size;                /* Number of objects in
						 * array */
            unsigned short packed_size;         /* Number of bytes allocated
                                                 * for packed representation
						 * (0 == unpacked array) */
	    IntBackPtr *interest_backptr;	/* the interests that contain
						 * this array */
            struct object *objects;		/* the objects */
        } array;

        struct {                                /* == string_type: growable */
            unsigned short size;                /* Number of characters in
                                                 * string */
            unsigned char chars[1];             /* the string */
        } string;

    	struct {                                /* == file_type */
            PSFILE      *inbuf;                 /* Input side of this IO
                                                 * channel */
            PSFILE      *outbuf;                /* Output side of this IO
                                                 * channel */
            struct object fname;                /* Name of the file */
	    struct object xstr;			/* The string being exec'd */
            struct object *tok;                 /* Compressed tokens declared
                                                 * for file*/
            short       ntok;                   /* Number of such tokens */
        } file;

        struct {                                /* == event_type */
            struct object name;
            struct object action;
            struct object client_data;
            struct object runnable_match;
            struct object canvas;
            REF process; 			/*  process interested in
                                                 * this event, or intended
                                                 * recipient */
            REF inter;                		/* interest matched by this
                                                 * event */
            REF kstate;                         /* array of down keys */
            REF next;                           /* next event in queue or
						 * next interest on the
						 * processes interest list */
	    MatchMask nameMask;			/* name match bits */
	    MatchMask actionMask;		/* action match bits */
	    short loc_x;                  	/* locator position */
	    short loc_y;
	    short delta_x;
	    short delta_y;
            unsigned  serial;                   /* event #, or interest's last
                                                 * match */
            fract time;                         /* minutes since startup */
            fract priority;                     /* applies only to interests */
            unsigned is_queued:1;               /* event is in the eventq */
            unsigned is_interest:1;             /* event is in an interest list                                                 */
            unsigned is_exclusive:1;            /* interest gobbles matching
                                                 * events */
            unsigned pre_child:1;               /* interest matches ahead of
                                                 * interests on child canvases
                                                 */
            unsigned name_match:2;              /* was /AnyValue specified in
                                                 * an interest name or action */
            unsigned action_match:2;            /* and how */
	    unsigned cancelmotionhint:1;	/* PointerMotionHint - X11 */
	    unsigned damage_interest:1;		/* interested in damage? */
	    unsigned motion_event:1;
	    unsigned locationnotset:1;
	    unsigned keyboard_event:1;
	    unsigned mouse_event:1;
	    unsigned sync:1;
	    unsigned validMatchMask:1;
	    unsigned synthetic:1;		/* Synthetic event */
	    unsigned systeminterest:1;		/* E.g. X11 focus interest */
	    unsigned pad:14;
        } event;

	struct {                                /* == monitor_type */
            struct execution_environment *first,
                                         *last,
                                         *holder;
            short reentry_depth;                /* # times current holder has
                                                 * entered the same monitor */
        } monitor;

        struct environment *environ;        	/* == environ_type */
        char *opaque;				/* == opaque_type */
	SHAPE shape;				/* == shape_type */
	struct  {				/* == cmapentry_type */
            REF        cmap;			/* associated COLORMAP */
            unsigned long   index;	
            short   	    ncolors;		
            unsigned long   *colorlist;
        } cmapentry;
	struct {
	    Pool		*vm_pool;
	    struct object	vm_name;
	} vm;
    } corpus;
};

extern REF empty_canvas;

extern struct object type_table[]; 	/* type to name mapping table */

extern char delim[];			/* A table indicating which characters 
					 * are delimiters */
extern char translate_digit[];		/* A table which maps characters to
				   	 * their value as a digit */

#define corpus_size(t) (((int)&((struct corpus *)0)->corpus) \
			       + sizeof ((struct corpus *)0)->corpus.t)
#define array_end(a)	((struct object *)(((char *)a) + corpus_size(array)))
/*
 * get instance dict from an entity pointer
 */
#define instancedict_of(entityp) \
	((DICT *)(refword(entityp) & ADDR_MASK))
#define set_instancedict(entityp, dictp) \
	(refword(entityp) = refword(entityp) & ~ADDR_MASK | (int)(dictp))

#define corpus_of(ref) ((struct corpus *)(entity_addr(ref)))


struct corpus *new_composite(/* *pool, *object, type, elements,
				initbytes, dicttype */);
#define make_growable_object(t,w,n)	syntax
#define make_array(n) 			syntax
#define make_string(n, str) 		syntax
#define new_growable_corpus(t,n)	syntax
#define new_array_corpus(n) 		syntax
#define new_string_corpus(n) 		syntax
#define new_dict(n) 			syntax

struct object new_kwd_name (/* hash, name, length */);
struct object lookup_kwd_name(/* name, length */);
void convert_string_to_name();
void convert_string_to_existing_name();
struct object parse_file (/* f */);
struct object parse_string (/* str */);
struct object open_file (/* fn, len, dir */);
struct execution_environment *create_process();
struct object *result_matrix(/*matrix object*/);
struct object *arg_matrix(/*matrix object*/);

enum type	coerce_to_numeric(/* *objstart, *objend */);
enum type	coerce_to_nonint(/* *objstart, *objend */);
enum error_type	coerce_to_fract(/* *objstart, *objend */);
enum type	binary_coerce_to_numeric(/* *object1 */);
enum type	binary_coerce_to_nonint(/* *object1 */);
enum error_type	binary_coerce_to_fract(/* *object1 */);

#ifndef	lint

#define object_type_pair(o1,o2) ((((int) (o1).type)<<6)+(int)(o2).type)
#define tp(t1,t2) ((((int) EXPAND(t1)_type)<<6)+(int)EXPAND(t2)_type)

#define numeric_type_pair(o1,o2) (typebyte(o1)|(typebyte(o2)>>2))
#define is_numeric_pair(o1,o2)		\
	((typebyte(o1)|typebyte(o2)) > (NUMERIC_TYPE_MASK << BYTE_TYPE_SHIFT))
#define ntp(t1,t2) ((((int) EXPAND(t1)_type)<<2)|(int)EXPAND(t2)_type)

#else	/* lint */

extern unsigned object_type_pair(/* struct object *, struct object * */);
#define tp(t1,t2) (EXPAND(t1)_type+EXPAND(t1)_type+EXPAND(t2)_type)

extern unsigned numeric_type_pair(/* struct object *, struct object * */);
extern unsigned is_numeric_pair(/* struct object *, struct object * */);
#define ntp(t1,t2) (EXPAND(t1)_type+EXPAND(t1)_type+EXPAND(t2)_type)

#endif	/* lint */

/* REMIND: Temporary hack to use X-style initialization */
/* #define pr_open(x)	private_pr_open(x) */
#define TemporaryInitHack




/* REMIND BJB: almost everything from here down should be moved to nucleus.h */

/*
 * Now for information about the execution of the interpreter and processes.
 */

enum execution_type {
    packedarray_body_execution,
    packedarray_nobind_body_execution,
    array_body_execution,
    array_nobind_body_execution,
    file_execution,
    integer_for_execution,
    fixed_for_execution,
    real_for_execution,
    stopped_execution,
    forall_execution,
    pathforall_execution,
    undetermined_execution,
    monitor_execution,
    buildimage_execution,
    token_execution,
    send_execution,
    nonPS_file_execution,
    setscreen_execution,
    noop_execution,
};

enum error_type {
    no_error_code,
    accept_error_code,
    dictfull_error_code,
    dictstackoverflow_error_code,
    dictstackunderflow_error_code,
    execstackoverflow_error_code,
    interrupt_error_code,
    invalidaccess_error_code,
    invalidexit_error_code,
    invalidfileaccess_error_code,
    invalidfont_error_code,
    invalidrestore_error_code,
    ioerror_error_code,
    limitcheck_error_code,
    nocurrentpoint_error_code,
    rangecheck_error_code,
    stackoverflow_error_code,
    stackunderflow_error_code,
    syntaxerror_error_code,
    timeout_error_code,
    typecheck_error_code,
    undefined_error_code,
    undefinedfilename_error_code,
    undefinedresult_error_code,
    unimplemented_error_code,
    unmatchedmark_error_code,
    unregistered_error_code,
    VMerror_error_code,
    killprocess_error_code,
};

extern char *error_names[];	/* Printable versions of the error codes */

/* 
 * struct execution_stack - The execution stack contains all information needed
 *			    to run the interpreter at each level of the call 
 *			    stack: it has return information & indications of 
 * 			    the type of operation in progress (eg. `for' and 
 *			    `repeat' statements).
 */
struct execution_stack {
    int      (*interpreter)();  /* the protocol interpreter in effect */
    enum execution_type type:32;
    struct object executed;	/* The object being executed */
    union {
	struct {		/* An array object being executed */
	    int         left;
            unsigned char *ip;
	}           array;
	struct {		/* A file or string being executed */
	    PSFILE     *file;
	}           file;
	struct {		/* fixed for */
	    fract       initial,
	                increment,
	                limit;
	    short       stackpos;
	}           ifor;
	struct {		/* real for */
	    float       initial,
	                increment,
	                limit;
	}           rfor;
	struct {
	    PATHREADER  *pathreader;
	} pathforall;
	struct {
	    struct object object;   /* The optop[-2] object */
	    unsigned char *ip;	    /* Packedarray current object pointer */
	} forall;
	struct object monitor;
	struct {
	    REF image;
	    char first;			/* Boolean */
	    char psdepth;		/* 1, 2, 4, 8, or 32 */
	    short remrow;
	    int remaining;
	    int pos;
	} image;
	struct {
	    short bottomdict,
		  topdict;
	} send;
	struct {		/* A non-PS file or string being executed */
	    PSFILE     *file;
	    char       *private;
	}           nonPS_file;
	struct {
	    unsigned short ix, iy;
	    double denom;
	} setscreen;
    }           env;
};


/* 
 * struct dictstack_ent - An entry in a dictionary stack 
 */
struct dictstack_ent {
    REF dict;		/* The actual dictionary or a magic object */
};

/*
 * default process stack sizes and limits
 */
#define NEW_ESTACK_SIZE		5
#define NEW_DSTACK_SIZE		5
#define NEW_OSTACK_SIZE		10
#define MAX_OSTACK_SIZE		1500
#define MAX_ESTACK_SIZE		250
/* REMIND - no MAX_DSTACK_SIZE!!! */

/*
 * the redbook says that you can't pop systemdict off of your dictionary
 * stack, so we set the default to 1
 */
#define DEFAULT_DICT_LOCK	1

/*
 * struct execution_environment -
 * 	The environment in which PostScript executes.  It's the context for a
 * 	single thread of control.  Supposedly PostScript() should be able to
 *	switch between execution environments to effect timeslicing [eg. for 
 * 	when errors occur].
 */
struct execution_environment {

    ENTITY_FIELDS		/* entity header */

    struct object		  process_name; /* string object for name */
    struct pool_desc		 *private_pool;	/* private VM for process */
    REF				  privatevm;	/* VM process lives in */
    REF				  currentvm;	/* current VM for allocation */
    struct execution_environment *next,	/* The next process in the set of
					 * runable processes */
	*pgnext,		/* A doubly linked list of processes in a
				   process group */
	*pgprev,
	*all_next,		/* LL of all processes that have ever
				 * been created */
	*priority_next,		/* LL of processes in the priority_waitq */
	*eventdist_next;	/* LL of processes waiting on an event dist */
    enum error_type error_code;	/* the error that is currently being
				 * processed */
    char *error_detail;		/* More detailed description of what exactly
				   happened */
    struct saveset *savelog;	/* A log of all the changes to variables
				   that have been made under the influence
				   of a "save" command.  It is undo-able by
				   a later "restore" */
    struct object stdprnt;	/* The standard print file for this
				   execution_environment -- this is where
				   output usually goes */
    struct object stddiag;	/* Where diagnostics usually go */
    REF		dollar_error;	/* This process's private $error dict */
    REF		errordict;	/* This process's private errordict */
    short       restart_state;	/* the state in which the interpreter
				 * should be restarted when its time to
				 * start interpreting this environment */
    char	autobind;	/* true=>keywords that correspond to
				   systemdict names will be automatically
				   bound to their definition by the reader */
    unsigned int interruptible_iowait:1; /* This process can be successfully
				 * restarted when in an io (file) wait */
    unsigned int donttimeout:1;	/* don't timeout this process EVER */
    unsigned int compat_input_dist:1; /* input distribution compatibility */
    unsigned int pad:5;
    short       event;		/* The event being waited for, >0 => that
				 * it is a file id */
    char	seeking_token;	/* True iff the token primitive is being
				   executed */
    char	detail_desired;	/* The amount of error reporting detail
				 * desired in the $error directory */
    short	susp_restart;	/* The place to restart the process after
				 * a continueprocess */
    short	susp_event;	/* The event to be waited for after
				 * a continueprocess */
    short	write_restart;	/* The place to restart the process after
				 * the writing is fully flushed */
    short	write_event;	/* The event to be waited for when
				 * the writing is fully flushed */
    char	packedarrays;	/* If true, executable arrays are packed */
    char	bindoverride;	/* If true, act like binding is off */
    struct object last_executed;/* The last thing executed in this context
				 * before the error */
    char       *static_buffer;	/* A scratch buffer for the parser */
    int         bufsize;	/* The size of the parsers buffer */
    int         fill;		/* The parse buffer fill pointer */
    int         ofill;		/* The old fill pointer: the start of the
				 * current name or string */
    int         arrstart;	/* The start of the current {} construct */
    struct object execee;	/* The thing currently being executed */
    REF gontext;		/* pointer to cscript graphics context */
    struct corpus *operand_stack;
    struct dictstack_ent *dictionary_stack,
    *dict_top,			/* The top valid entry in dictstack */
    *dict_limit;		/* The first non-allocated entry in dictstack */
    /* REMIND: this could be a short */
    int dict_lock;		/* the dict entry that is locked */
    struct object *underflow,   /* The bottom of the operand stack */
		  *overflow,	/* The limit of the operand stack */
		  *optop;	/* The top of the operand stack -- optop[-1]
				   is the top element, optop[-2] is the one
				   below it... */
    struct execution_stack *execution_stack,
               		   *limit,
               		   *pos;
    REF interests;		/* The events that are interesting to this
				   process */
    REF eq_head;		/* the queue of events waiting to be */
    struct corpus *eq_tail;	/*  processed by this process	     */
    int		priority;	/* the process priority */
    struct object	terminal_value;	/* process termination value */
    unsigned short xcid;	/* An X client's unique ID */
};

/* 
 * 	Ensure that the dictionary stack of process "ee" has room for "n"
 * 	new entries; fails only if we're running out of memory 
 */
#define have_dict_space(ee, n) \
  ((ee)->dict_top+(n) >= (ee)->dict_limit ? enlarge_dict_space(ee, n) : 1)

/* 
 *	The kinds of match an interest-event's name or action may require 
 */ 
#define DirectMatch     0
#define SimpleAnyMatch  1
#define DictAnyMatch    2 

/*
 * 	These are the various common process priorities in the server
 */
#define         user_priority           0
#define         system_priority         100


/* 
 *	The various events that a process can be waiting for and states that
 * 	it can be in.
 */
#define fileio_event_wait(fd) ((fd)+1)
#define runnable_process 0
#define input_event_wait -1	/* some sort of button change or locator
				   motion */
#define dead_process	 -2	/* dead, and its carcass is reusable */
#define zombie_process	 -3	/* dead, but some process id handle is
				   outstanding, so the carcass is being kept
				   around until the PostScript program uses
				   it */
#define process_activity_wait -4
#define suspended_process -5	/* A process which has stopped, normally for
				   debugging */
#define monitor_wait	-6	/* Waiting for a monitor to become available
				   */

#define event_write_flag 0x4000 /* Indicates waiting on a write */

/*
 * 	All processes that have ever been created.
 */
extern struct execution_environment *all_processes;
/*
 * 	The currently executing process 
 */
extern struct execution_environment *current_process;
/*
 *	The current pool (VM space) to allocate objects from.
 */
extern Pool			    *current_pool;
/*
 *	The VM object for the current pool.
 */
extern REF			     current_vm;
/*
 *	The VM space to allocate objects not relating to a process.
 */
extern Pool			    *server_pool;
/*
 *	The VM object for the server pool.
 */
extern REF			     server_vm;
/*
 * 	The set of processes waiting for activity in another process
 */
extern struct execution_environment *waiting_procs;
/*
 * 	The set of processes waiting on a priority change
 */
extern struct execution_environment *priority_waitq; 
/*
 * 	The PostScript run queue
 */
extern struct execution_environment *process_runq;
extern struct execution_environment *process_set;
/*
 * 	The number of processes that currently exist
 */
extern int process_population;	
/*
 * 	Set of processes waiting for a read to complete; indexed by file ID. 
 */
extern struct execution_environment **process_io_waiters;
/*
 *    Set of processes waiting for an event to be distributed
 */
extern struct execution_environment *process_eventdist_waiters;


/*
 *	Macros to manipulate process_io_waiters
 */
#define enqueue_io_waiter(ee, fd) {			\
    ee->next = process_io_waiters[fd];			\
    process_io_waiters[fd] = ee;			\
}
#define dequeue_io_waiter(ee, fd) {			\
    register struct execution_environment **eep;	\
    eep = &process_io_waiters[fd];			\
    while (*eep)					\
	if (*eep == ee) {				\
	    *eep = ee->next;				\
	    ee->next = 0;				\
	    break;					\
	} else						\
	    eep = &((*eep)->next);			\
}

/*
 * 	The set of globally defined compressed input tokens.
 */
extern struct object syscommon_vec[];

#endif /* _PostScript_h */
