/*
 * @(#)gcstruct.h 23.3 90/06/19
 */
/***********************************************************
Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

/*
 * gcstruct.h - X11 GC structure
 */


#ifndef GCSTRUCT_H
#define GCSTRUCT_H

#include <cscript/cscript.h>
#include "gc.h"
#include "miscstruct.h"
#include "pixmap.h"

#ifdef USE_EXTENSIONS

typedef struct _GCInterest {
    struct _GCInterest	*pNextGCInterest;
    struct _GCInterest	*pLastGCInterest;
    int			length;		
    ATOM		owner;		/* extension id of owning extension */
    unsigned long	ValInterestMask;
    void		(* ValidateGC) ();
    unsigned long	ChangeInterestMask;
    int			(* ChangeGC) ();
    void		(* CopyGCSource) ();
    void		(* CopyGCDest) ();
    void		(* DestroyGC) ();
    pointer		extPriv;	/* pointer extension private data */
} GCInterestRec;


typedef struct	_GCExtension	{
    GCInterestPtr	pNextGCInterest;
    GCInterestPtr	pLastGCInterest;

    pointer		devPriv;	/* private to the device */

    void (* FillSpans)();
    void (* SetSpans)();

    void (* PutImage)();
    void (* CopyArea)();
    void (* CopyPlane)();

    void (* PolyPoint)();
    void (* Polylines)();
    void (* PolySegment)();
    void (* PolyRectangle)();
    void (* PolyArc)();
    void (* FillPolygon)();
    void (* PolyFillRect)();
    void (* PolyFillArc)();
    int (* PolyText8)();
    int (* PolyText16)();
    void (* ImageText8)();

    void (* ImageText16)();
    void (* ImageGlyphBlt)();
    void (* PolyGlyphBlt)();
    void (* PushPixels)();
    void (* LineHelper)();
    void (* ChangeClip) ();
    void (* DestroyClip) ();
    void (* CopyClip)();
} GCExtension;

#endif

typedef struct _GC {

    CS_GC_FIELDS			/* header, ref and cscript GC fields */

    SCREENINFO		pScreen;		
    PixUnion		tile;
    REF			stipple;
    DDXPointRec		patOrg;
    DDXPointRec		clipOrg;
    unsigned short	lastBorderWidth;
    unsigned char 	depth;    
    unsigned 		clientClipType:2;	/* pixmap, region, or none */
    unsigned		arcMode:1;
    unsigned		subWindowMode:1;
    unsigned		graphicsExposures:1;
    unsigned		tileIsPixel:1; 		/* tile is solid pixel */
    unsigned 		Xgcpad:2;
#ifdef USE_EXTENSIONS
    GCExtension		*gcext;
    unsigned long	stateChanges;	/* masked with GC_* */
    unsigned long       serialNumber;
#endif
} GC;

#endif /* GCSTRUCT_H */
