/*
 * @(#)windowstr.h 23.2 90/06/19
 */
/***********************************************************
Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

#ifndef WINDOWSTRUCT_H
#define WINDOWSTRUCT_H

#include <cscript/cscript.h>
#include "window.h"
#include "pixmapstr.h"
#include "property.h"
#include "resource.h"	/* for ROOT_WINDOW_ID_BASE */
#include "dix.h"
#include "miscstruct.h"
#include "grabs.h"
#include <X11/Xprotostr.h>
#include "input.h"

#define	SameBackground(as, a, bs, b)				\
	((as) == (bs) && ((as) == None ||			\
			  (as) == ParentRelative ||		\
			  SamePixUnion(a,b,as == BackgroundPixel)))

#define	SameBorder(as, a, bs, b)	\
	EqualPixUnion(as, a, bs, b)

/*
 * this struct holds additional data for X interest compression.
 * each of the interest holds a ptr to the interest for each set.
 */
#define	NUM_SETS	3		/* number of interest sets */

/*
 * Interesting event info
 */
typedef struct _xev_handle_stuff	{
	ClientPtr		client;			/* the client */
	XID			resource;		/* resource DB id */
	Mask			eventMask;		/* interesting events */
	XInterest		*asyncinterest;
	XInterest		*syncinterest;
	struct _xev_handle_stuff *next;
} xev_handle_stuff;

typedef struct _WindowOpt	{
    unsigned long backingBitPlanes;
    unsigned long backingPixel;
    PropertyPtr	userProps;
    GrabPtr passiveGrabs;
} WindowOpt;

/* 
 * X specific window information.  Every X window has one of these structs.
 */
typedef struct _XWindowStuff {
	ClientPtr client;		/* client object for creator */
	Mask eventMask;			/* interesting events for this canvas */
	Mask suppressMask;		/* suppressed events for this canvas */

	unsigned cvInResourceDB:1;	/* this canvas is still in the resource
					 * DB.  If false, it has been destroyed
					 * as far as the X protocol is
					 * concerned, but may still have 
					 * references inside the server.
					 */
	unsigned backingStore:2;
	unsigned pad:13;
	unsigned short borderWidth;


	xev_handle_stuff *xevh;		/* event info & interests */
	XInterest *suppressinterest;	/* event suppression */

	/*
	 * grody hack to deal with multiple exposure interests on
	 * the same canvas
	 */
	unsigned int	last_damage_serial;
	SHAPE		last_damage_area;

	WindowOpt	*optional;

} XWindowStuff;


#define xcs_x(cv) \
	((XWindowStuff *) ((XNCANVAS) entity_addr(cv))->xwindowstuff)
#define	xcs_client(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->client)
#define	xcs_wid(cv) \
	(((XNCANVAS) entity_addr(cv))->wid)
#define xcs_xevh(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->xevh)
#define	xcs_suppmask(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->suppressMask)
#define	xcs_allmask(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->eventMask)
#define	xcs_borderwidth(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->borderWidth)
#define	xcs_visual(cv)	cs_get_visual((cv))


#define	xcs_backmap(cv) \
	(((XNCANVAS) entity_addr(cv))->background.pixmap)
#define	xcs_backpixel(cv) \
	(((XNCANVAS) entity_addr(cv))->background.pixel)
#define	xcs_backgroundstate(cv)	\
	(((XNCANVAS) entity_addr(cv))->bgState)

#define	xcs_bordermap(cv) \
	(((XNCANVAS) entity_addr(cv))->border.pixmap)
#define	xcs_borderpixel(cv) \
	(((XNCANVAS) entity_addr(cv))->border.pixel)
#define	xcs_borderispixel(cv)	\
	(((XNCANVAS) entity_addr(cv))->borderIsPixel)



#define	xcs_backstore(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->backingStore)

#define	xcs_colormap(cv) \
	cs_get_colormap((cv))
#define	xcs_override(cv) \
	(((XNCANVAS) entity_addr(cv))->overrideRedirect)
#define	xcs_cvinresourcedb(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->cvInResourceDB)
#define xcs_saveback(cv) \
	(((XNCANVAS) entity_addr(cv))->saveback)

#define	xcs_lastdamserial(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->last_damage_serial)
#define	xcs_lastdamarea(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->last_damage_area)

/* WindowOpt macros */

#define	xcs_optional(cv) \
	(((XNCANVAS) entity_addr(cv))->xwindowstuff->optional)

#define	wUseDefault(cv, field, def)	\
	(xcs_optional(cv) ? xcs_optional(cv)->field : def)

#define	xcs_backingbitplanes(cv)	wUseDefault(cv, backingBitPlanes, ~0L)
#define	xcs_backingpixel(cv)	wUseDefault(cv, backingPixel, 0)
#define	xcs_properties(cv)	wUseDefault(cv, userProps, NULL)
#define xcs_passivegrabs(cv)	wUseDefault(cv, passiveGrabs, NULL)

#define	xcs_setbackingbitplanes(cv, value)	\
	(xcs_optional(cv)->backingBitPlanes = (value))
#define	xcs_setbackingpixel(cv, value)	\
	(xcs_optional(cv)->backingPixel = (value))
#define	xcs_setproperties(cv, value)	\
	(xcs_optional(cv)->userProps = (value))
#define xcs_setpassivegrabs(cv, value)	\
	(xcs_optional(cv)->passiveGrabs = (value))



/* return the real ID of a window, taking care not to find one that's 
 * been pulled from the resource DB */

#define	xcs_realwid(cv) (xcs_x(cv) ?	\
			    (xcs_cvinresourcedb(cv) ? xcs_wid(cv) : None) \
			    : xcs_wid(cv))

extern int DeleteWindow();
extern int ChangeWindowAttributes();
extern int WalkTree();
extern CreateRootWindow();
extern REF CreateWindow();
extern int DeleteWindow();
extern int DestroySubwindows();
extern int GetWindowAttributes();
extern int ConfigureWindow();
extern int ReparentWindow();
extern int MapWindow();
extern int MapSubwindow();
extern void UnmapWindow();
extern int UnmapSubwindow();

extern Bool MakeWindowOptional();

extern void xcs_destroy_window();
#endif /* WINDOWSTRUCT_H */
