/*
 * @(#)xcsmacros.h 23.4 90/06/19
 */

/* various graphics macros used mostly in xcsProcs */

#ifndef cs_ensurepathspace
#define cs_ensurepathspace(GC, N) \
    Set_Path(Get_Context(gc_get_context((GRAPHICS_CONTEXT) entity_addr(GC)), \
			 CTX_PATH), PATH_SIZE, ((N) + 1))
#endif

#ifdef DEBUG
#define rcanvas_of(ref) 					     \
	(assert((ref).type == xn_canvas_type || 		     \
			(ref).type == null_type), 		     \
	((RCANVAS)(entity_addr((ref)))))
#else
#define rcanvas_of(ref) 					     \
	((RCANVAS)(entity_addr((ref))))
#endif
#define ocanvas_of(ref) ((OCANVAS)(entity_addr((ref))))

/*
 * note that this macro will expect 'bw' to be declared by the routine
 * calling it, since it will be used again
 *
 * we can get even more by exposing some of the viscious casting
 */
#define	xcs_setcanvas(gcref, winref)					\
    do {								\
	GCPtr	gctmp;							\
	GRAPHICS_CONTEXT	gcont;					\
	gctmp = (GCPtr) entity_addr(gcref);				\
	bw = 0;								\
        if (xcs_type(winref) == DRAWABLE_WINDOW &&			\
	  (bw = xcs_borderwidth(winref)) != gctmp->lastBorderWidth) {	\
            cs_translateclip((gcref),					\
		    (int)(bw - gctmp->lastBorderWidth),			\
		    (int)(bw - gctmp->lastBorderWidth));		\
            gctmp->lastBorderWidth = bw;				\
        }								\
	cs_setfillorigin(gcref, gctmp->patOrg.x + bw, 			\
	    gctmp->patOrg.y +bw);					\
	gcont = (GRAPHICS_CONTEXT)(entity_addr(gcref));			\
	gc_set_canvas(gcont, winref);					\
	cv_newpath(gcont);						\
    } while (0)

#define	xcs_setdrawable(gcref, gcont, ctx, bw, winref)			\
    do {								\
	GCPtr	gctmp;							\
	gctmp = (GCPtr) entity_addr(gcref);				\
	bw = 0;								\
        if (xcs_type(winref) == DRAWABLE_WINDOW &&			\
	  (bw = xcs_borderwidth(winref)) != gctmp->lastBorderWidth) {	\
            cs_translateclip((gcref),					\
		    (int)(bw - gctmp->lastBorderWidth),			\
		    (int)(bw - gctmp->lastBorderWidth));		\
            gctmp->lastBorderWidth = bw;				\
        }								\
	cs_setfillorigin(gcref, gctmp->patOrg.x + bw, 			\
	    gctmp->patOrg.y +bw);					\
	gcont = (GRAPHICS_CONTEXT)(entity_addr(gcref));			\
	gc_set_canvas(gcont, winref);					\
	ctx = gc_get_context(gcont);					\
	Clear_Path(Get_Context(ctx, CTX_PATH), PATH_RESET);		\
    } while (0)


/*
 * just change the canvas & gc
 */

#define	xcs_setcanvasonly(gcref, winref)				\
    do	{								\
	GRAPHICS_CONTEXT	gcont;					\
	gcont = (GRAPHICS_CONTEXT)(entity_addr(gcref));			\
	gc_set_canvas(gcont, winref);					\
    } while (0)

#define	xcs_setdrawableonly(gcref, gcont, ctx, winref)			\
    do	{								\
	gcont = (GRAPHICS_CONTEXT)(entity_addr(gcref));			\
	ctx = gc_get_context(gcont);					\
	gc_set_canvas(gcont, winref);					\
    } while (0)


/*
 * drop the cursor & overlay
 */
#define	xcs_validate_and_drop(drawable, gc)				\
    do {								\
	if (drawable.type == xn_canvas_type) {				\
	    register RCANVAS CV = rcanvas_of(drawable);			\
	    cv_validate_clip(CV, (gc)->drawthrough,(gc)->paintouter,0); \
	    cv_overlaysdown();						\
	}								\
    } while(0)

/*
 * X 'renderpath' -- draws to screen & retained areas
 */

#define	xcs_renderpath(drawable, ctx, shade)				\
    do {								\
	register RASTER screenRaster, retainedRaster;			\
	Set_Context(ctx, CTX_SHADE, shade);				\
	if (drawable.type == xn_canvas_type && 				\
	    (screenRaster = canvas_get_screenpart(rcanvas_of(drawable)))) {\
	    Set_Context(ctx, CTX_RASTER, screenRaster);			\
	    Display_Context(ctx, CTX_RENDER_FLUSH);			\
	}								\
	retainedRaster = canvas_get_retainedpart(ocanvas_of(drawable));	\
	if (retainedRaster != 0 &&					\
		canvas_get_retainedclipnotempty(ocanvas_of(drawable))) {\
	    Set_Context(ctx, CTX_RASTER, retainedRaster);		\
	    Display_Context(ctx, CTX_RENDER_FLUSH);			\
	}								\
    } while (0)

/*
 * X 'rendershape' -- draws to screen & retained areas
 */

#define	xcs_rendershape(drawable, ctx, shape)				\
    do {								\
	register RASTER screenRaster, retainedRaster;			\
	if (drawable.type == xn_canvas_type && 				\
	    (screenRaster = canvas_get_screenpart(rcanvas_of(drawable)))) {\
	    Set_Context(ctx, CTX_RASTER, screenRaster);			\
	    Display_Shape(shape, ctx, 0);				\
	}								\
	retainedRaster = canvas_get_retainedpart(ocanvas_of(drawable));	\
	if (retainedRaster != 0 &&					\
		canvas_get_retainedclipnotempty(ocanvas_of(drawable))) {\
	    Set_Context(ctx, CTX_RASTER, retainedRaster);		\
	    Display_Shape(shape, ctx, 0);				\
	}								\
    } while (0)

#define xcs_ensurepathspace(ctx, N) \
    Set_Path(Get_Context(ctx, CTX_PATH), PATH_SIZE, ((N) + 1))

/* graphics operations -- assume that the path is big enough */
/* NOTE:  beware of over-eager text substitution -- use CAPS for args */
#define	xcs_moveto(CTX, X, Y)						\
    do {								\
	register PATH_POINT	*pptr = (CTX)->ctx_buf_ptr;		\
	pptr->b.t = PT_MOVE | PT_BIN2D; pptr->b.x = fracti(X);		\
	(pptr++)->b.y = fracti(Y);					\
	(CTX)->ctx_buf_ptr = pptr;					\
	assert (pptr <= (CTX)->ctx_buf_end);				\
    } while(0)

#define	xcs_lineto(CTX, X, Y)						\
    do {								\
	register PATH_POINT	*pptr = (CTX)->ctx_buf_ptr;		\
	pptr->b.t = PT_LINE | PT_BIN2D; pptr->b.x = fracti(X);		\
	(pptr++)->b.y = fracti(Y);					\
	(CTX)->ctx_buf_ptr = pptr;					\
	assert (pptr <= (CTX)->ctx_buf_end);				\
    } while(0)

#define	xcs_rlineto(CTX, X, Y)						\
    do {								\
	register PATH_POINT	*pptr = (CTX)->ctx_buf_ptr;		\
	pptr->b.t = PT_LINEREL | PT_BIN2D; pptr->b.x = fracti(X);	\
	(pptr++)->b.y = fracti(Y);					\
	(CTX)->ctx_buf_ptr = pptr;					\
	assert (pptr <= (CTX)->ctx_buf_end);				\
    } while(0)

