/*
 * @(#)cs_canvas.h 23.3 90/06/19
 */

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

/*
 * cs_canvas.h - canvas structures and information
 */

#ifndef _cs_canvas_h
#define _cs_canvas_h

#include <Core/core.h>
#include <graphics/shapes.h>
#include "cs_common.h"
#include "cs_cursor.h"
#include "cs_transform.h"

/* Bit Gravity */

#define ForgetGravity		0
#define NorthWestGravity	1
#define NorthGravity		2
#define NorthEastGravity	3
#define WestGravity		4
#define CenterGravity		5
#define EastGravity		6
#define SouthWestGravity	7
#define SouthGravity		8
#define SouthEastGravity	9
#define StaticGravity		10

/* Window gravity + bit gravity above */

#define UnmapGravity		0
#define OtherGravity		11

/* Visibility */

#define Unobscured		0
#define Partially_Obscured	1
#define Totally_Obscured	2

#define DLPATH 0
#define DLTEXT 1

/* Display list attached to an overlay canvas. */
struct overlayDL {
	struct overlayDL *next;
	int	type;		/* DLPATH || DLTEXT */
	union {
	    PATH		path;
	    RENDERING_INFO	*text;
	} pr;
};

struct cs_savebacklist {
    struct cs_savebacklist* next;
    REF cv;
};

/*
 * Cscript orphan canvas structure - the basis of all canvases.
 */
#define ORPHAN_FIELDS							      \
    ENTITY_FIELDS		/* header information */		      \
		      							      \
    short width, height;	/* The dimensions of the canvas. */	      \
    char depth;			/* The depth of this canvas. */		      \
									      \
    FLAG validclips:1;		/* True if this canvas has valid raster       \
				   clips. */				      \
    FLAG shared:1;		/* True of this canvas is mmaping file */     \
    FLAG retainedclipnotempty:1;/* True if the canvas has a retained part */  \
    				/* with a non-empty clip. */		      \
    FLAG pad1:5;							      \
                                                                              \
    RASTER retainedpart;	/* The hidden part of the canvas. */  \
									      \
    SHAPE outerclip;		/* The shape of this canvas. */		      \
									      \
    char   *graphicsDamage;	/* Description of graphics exposes	      \
				 * on this canvas */			      \
    struct screeninfo *screeninfo;	/* the screeninfo that holds this canvas  */ \
									      \
    FMATRIX trans;							      \
									      \
    REF colormap;		/* The colormap associated with it */	      \
									      \
    /*									      \
     * REMIND: this is temporary and should really be in the xn_canvas	      \
     * structure at the nucleus level.  Robin and lemke are trying to 	      \
     * decide what to do with it.					      \
     */									      \
    unsigned long wid;		/* ID for this canvas */		      \
									      \
    REF visual;			/* visual for the canvas */


struct orphan_canvas {
    ORPHAN_FIELDS
};

typedef struct orphan_canvas *OCANVAS;

/*
 * Cscript rooted canvas structure - it exists in the canvas hierarchy.
 */
#define ROOTED_FIELDS							       \
									       \
    ORPHAN_FIELDS	        /* base of information about a canvas */       \
									       \
    REF parent;			/* The canvas that contains this canvas. */    \
    REF topchild;		/* The topmost visible child of this canvas. */\
    struct rooted_canvas *bottomchild; /* The bottom most child */             \
    REF next;			/* Doubly linked list of canvases.  */         \
    REF prev;			/* The canvases are arranged in overlapping */ \
				/* layers with the next canvas higher than a */\
				/* previous canvas. */			       \
    REF opaqueparent;		/* The opaque parent of this canvas.  It */    \
				/* points to itself if not transparent. */     \
									       \
    RASTER	screenpart,	/* The on-screen part of a canvas */	       \
		savebackpart;	/* The saveback part of a canvas */	       \
									       \
    struct spoint position;	/* Position relative to parent. */	       \
									       \
    struct spoint screenpos;	/* Absolute screen position. */		       \
									       \
    SHAPE innerclip;		/* "Client Drawable" part of canvas */         \
									       \
    SHAPE globalclip;		/* A clipping region within the shape of the */\
				/* canvas that is in effect for all graphics */\
				/* contexts that access the canvas - it is */  \
				/* not part of a graphics context. */ 	       \
									       \
    struct cs_savebacklist *oldvalidsavebacks;				       \
									       \
    FLAG bitGravity:4;		/* The bits for a window move this  */	       \
    				/* direction when the window is resized */     \
									       \
    FLAG winGravity:4;		/* The windows move this direction */	       \
    				/* when their parent gets resized */	       \
    FLAG visibility:2;		/* The window's current visibility */	       \
									       \
    FLAG drawthrough:1;		/* The drawthrough bit of the last context */  \
				/* used to paint in this canvas. */	       \
									       \
    FLAG visinterest:1;		/* True if keeping track of visibility */      \
									       \
    FLAG transparent:1;		/* True iff this canvas is transparent; i.e.,*/\
				/* its bits are really owned by its parent. */ \
									       \
    FLAG mapped:1;		/* True iff this canvas has been mapped */     \
				/* onto a screen. */			       \
									       \
    FLAG maprequested:1;	/* True iff this canvas wants to be mapped */  \
				/* but can't be (because parent is unmapped, */\
				/* for example.) */			       \
									       \
    FLAG retained:1;		/* True iff this canvas is retained. */	       \
									       \
    FLAG hasopaquechildren:1;	/* True if this canvas has opaque children. */ \
									       \
    FLAG complete:1;		/* True if the retained part of the canvas */  \
				/* is complete. */			       \
									       \
    FLAG saveback:1;		/* True if the bits behind this canvas get */  \
				/* saved in this non-retained canvas. */       \
									       \
    FLAG overlay:1;		/* True if this is an overlay canvas. */       \
									       \
    FLAG whenmapped:1;		/* True if this canvas is only retained */     \
    				/* "when mapped"			*/     \
    FLAG allchildrendisjoint:1;	/* True if this canvas' children are all */    \
				/* disjoint */				       \
				/* siblings covering it up */		       \
    FLAG tryforalldisjoint:1;   /* True if disjointness should be attempted */ \
				/* children. */				       \
    FLAG allchildrensolid:1;    /* True if no children changed between maps */ \
    FLAG visibleValid:1;	/* True if visible is valid */                 \
    FLAG drawableValid:1;	/* True if drawable is valid */                \
    FLAG mark1:1;		/* Temporary flag */                           \
    FLAG mark2:1;               /* Temporary flag */                           \
    FLAG allchildrenunmapped:1;	/* used for map performance */                 \
    FLAG childmaybemapped:1;    /* used for Circulate unmapped performance */  \
    FLAG visiblewasValid:1;	/* used for map performance */		       \
									       \
    REF hasoverlay;  		/* pointer to this canvas' overlay, if any. */ \
				/* Null if Doesn't have overlay.  If overlay */\
				/* exists, hasoverlay must == topchild, so */  \
				/* that clipping comes out right. */	       \
									       \
    struct overlayDL *overlaydata; /* Display list for overlay canvas. */      \
				   /* Otherwise unused */		       \
									       \
    SHAPE damage;		/* Description of damage on this canvas. */    \
    SHAPE visible;		/* a cached shape of what's visible */         \
    SHAPE drawable;		/* a cached shape of what's drawable */        \
    SHAPE tmpDam;							       \
									       \
    REF cursor;			/* My cursor. nil means use my parent's */     \
									       \
    struct _XWindowStuff  *xwindowstuff; /* X protocol specific information */

struct rooted_canvas {
    ROOTED_FIELDS
};

typedef struct rooted_canvas *RCANVAS;

/*
 * Macros for accessing fields in the canvas structures.  The first
 * list is for information contained in the orphan fields; the second
 * for information contained in the rooted fields - split just for 
 * convenience.  All macros take pointer to a canvas structure as an
 * arg because these are all internal to cscript.  The only REFs passed 
 * to any macro here is as a second arg used to set a field that is 
 * defined as a REF.
 */
#define canvas_get_width(xxx) ((xxx)->width)
#define canvas_set_width(xxx,yyy) ((xxx)->width = (yyy))

#define canvas_get_height(xxx) ((xxx)->height)
#define canvas_set_height(xxx,yyy) ((xxx)->height = (yyy))

#define canvas_get_depth(xxx) ((xxx)->depth)
#define canvas_set_depth(xxx,yyy) ((xxx)->depth = (yyy))

#define canvas_get_retainedpart(xxx) ((xxx)->retainedpart)
#define canvas_set_retainedpart(xxx,yyy) ((xxx)->retainedpart = (yyy))

#define canvas_get_outerclip(xxx) ((xxx)->outerclip)
#define canvas_set_outerclip(xxx,yyy) ((xxx)->outerclip = (yyy))

#define canvas_get_validclips(xxx) ((xxx)->validclips)
#define canvas_set_validclips(xxx,yyy) ((xxx)->validclips = (yyy))

#define canvas_get_graphicsDamage(xxx) ((xxx)->graphicsDamage)
#define canvas_set_graphicsDamage(xxx,yyy) ((xxx)->graphicsDamage = (yyy))

#define canvas_get_screeninfo(xxx) ((xxx)->screeninfo)
#define canvas_set_screeninfo(xxx,yyy) ((xxx)->screeninfo = (yyy))
#define canvas_get_transform(xxx) ((xxx)->trans)
#define canvas_set_transform(xxx,yyy) ((xxx)->trans = (yyy))

#define canvas_get_colormap(xxx) ((xxx)->colormap)
#define canvas_set_colormap(xxx,yyy) ((xxx)->colormap = (yyy))

#define canvas_get_visual(xxx) ((xxx)->visual)
#define canvas_set_visual(xxx,yyy) ((xxx)->visual = (yyy))

#define canvas_get_shared(xxx) ((xxx)->shared)
#define canvas_set_shared(xxx,yyy) ((xxx)->shared = (yyy))

/*
 * Second list - rooted fields
 */
#define canvas_get_parent(xxx) ((xxx)->parent)
#define canvas_set_parent(xxx,yyy) ((xxx)->parent = (yyy))

#define canvas_get_topchild(xxx) ((xxx)->topchild)
#define canvas_set_topchild(xxx,yyy) ((xxx)->topchild = (yyy))

#define canvas_get_nextcanvas(xxx) ((xxx)->next)
#define canvas_set_nextcanvas(xxx,yyy) ((xxx)->next = (yyy))

#define canvas_get_prevcanvas(xxx) ((xxx)->prev)
#define canvas_set_prevcanvas(xxx,yyy) ((xxx)->prev = (yyy))

#define canvas_get_opaqueparent(xxx) ((xxx)->opaqueparent)
#define canvas_set_opaqueparent(xxx,yyy) ((xxx)->opaqueparent = (yyy))

#define canvas_get_screenpart(xxx) ((xxx)->screenpart)
#define canvas_set_screenpart(xxx,yyy) ((xxx)->screenpart = (yyy))

#define canvas_get_savebackpart(xxx) ((xxx)->savebackpart)
#define canvas_set_savebackpart(xxx,yyy) ((xxx)->savebackpart = (yyy))

#define canvas_get_retainedclipnotempty(xxx) ((xxx)->retainedclipnotempty)
#define canvas_set_retainedclipnotempty(xxx,yyy) \
	((xxx)->retainedclipnotempty = (yyy))

#define canvas_get_position(xxx) ((xxx)->position)
#define canvas_set_position(xxx,yyy) ((xxx)->position = (yyy))

#define canvas_get_x(xxx) ((xxx)->position.x)
#define canvas_set_x(xxx,yyy) ((xxx)->position.x = (yyy))

#define canvas_get_y(xxx) ((xxx)->position.y)
#define canvas_set_y(xxx,yyy) ((xxx)->position.y = (yyy))

#define canvas_get_screenpos(xxx) ((xxx)->screenpos)
#define canvas_set_screenpos(xxx,yyy) ((xxx)->screenpos = (yyy))

#define canvas_get_screenposx(xxx) ((xxx)->screenpos.x)
#define canvas_set_screenposx(xxx,yyy) ((xxx)->screenpos.x = (yyy))

#define canvas_get_screenposy(xxx) ((xxx)->screenpos.y)
#define canvas_set_screenposy(xxx,yyy) ((xxx)->screenpos.y = (yyy))

#define canvas_get_innerclip(xxx) ((xxx)->innerclip)
#define canvas_set_innerclip(xxx,yyy) ((xxx)->innerclip = (yyy))

#define canvas_get_globalclip(xxx) ((xxx)->globalclip)
#define canvas_set_globalclip(xxx,yyy) ((xxx)->globalclip = (yyy))

#define canvas_get_oldvalidsavebacks(xxx) ((xxx)->oldvalidsavebacks)
#define canvas_set_oldvalidsavebacks(xxx,yyy) ((xxx)->oldvalidsavebacks = (yyy))

#define canvas_get_bitGravity(xxx) ((xxx)->bitGravity)
#define canvas_set_bitGravity(xxx,yyy) ((xxx)->bitGravity = (yyy))

#define canvas_get_winGravity(xxx) ((xxx)->winGravity)
#define canvas_set_winGravity(xxx,yyy) ((xxx)->winGravity = (yyy))

#define canvas_get_visibility(xxx) ((xxx)->visibility)
#define canvas_set_visibility(xxx,yyy) ((xxx)->visibility = (yyy))

#define canvas_get_drawthrough(xxx) ((xxx)->drawthrough)
#define canvas_set_drawthrough(xxx,yyy) ((xxx)->drawthrough = (yyy))

#define canvas_get_visinterest(xxx) ((xxx)->visinterest)
#define canvas_set_visinterest(xxx,yyy) ((xxx)->visinterest = (yyy))

#define canvas_get_transparent(xxx) ((xxx)->transparent)
#define canvas_set_transparent(xxx,yyy) ((xxx)->transparent = (yyy))

#define canvas_get_mapped(xxx) ((xxx)->mapped)
#define canvas_set_mapped(xxx,yyy) ((xxx)->mapped = (yyy))

#define canvas_get_maprequested(xxx) ((xxx)->maprequested)
#define canvas_set_maprequested(xxx,yyy) ((xxx)->maprequested = (yyy))

#define canvas_get_retained(xxx) ((xxx)->retained)
#define canvas_set_retained(xxx,yyy) ((xxx)->retained = (yyy))

#define canvas_get_hasopaquechildren(xxx) ((xxx)->hasopaquechildren)
#define canvas_set_hasopaquechildren(xxx,yyy) ((xxx)->hasopaquechildren = (yyy))

#define canvas_get_complete(xxx) ((xxx)->complete)
#define canvas_set_complete(xxx,yyy) ((xxx)->complete = (yyy))

#define canvas_get_saveback(xxx) ((xxx)->saveback)
#define canvas_set_saveback(xxx,yyy) ((xxx)->saveback = (yyy))

#define canvas_get_overlay(xxx) ((xxx)->overlay)
#define canvas_set_overlay(xxx,yyy) ((xxx)->overlay = (yyy))

#define canvas_get_whenmapped(xxx) ((xxx)->whenmapped)
#define canvas_set_whenmapped(xxx,yyy) ((xxx)->whenmapped = (yyy))

#define canvas_get_hasoverlay(xxx) ((xxx)->hasoverlay)
#define canvas_set_hasoverlay(xxx,yyy) ((xxx)->hasoverlay = (yyy))

#define canvas_get_overlaydata(xxx) ((xxx)->overlaydata)
#define canvas_set_overlaydata(xxx,yyy) ((xxx)->overlaydata = (yyy))

#define canvas_get_damage(xxx) ((xxx)->damage)
#define canvas_set_damage(xxx,yyy) ((xxx)->damage = (yyy))

#define canvas_get_tmpDam(xxx) ((xxx)->tmpDam)
#define canvas_set_tmpDam(xxx,yyy) ((xxx)->tmpDam = (yyy))

#define canvas_get_cursor(xxx) ((xxx)->cursor)
#define canvas_set_cursor(xxx,yyy) ((xxx)->cursor = (yyy))


/* Define access macros for scenes. */

#define scene_get_canvases(xxx) ((xxx)->canvases)
#define scene_set_canvases(xxx,yyy) ((xxx)->canvases = (yyy))

#define scene_get_display(xxx) ((xxx)->display)
#define scene_set_display(xxx,yyy) ((xxx)->display = (yyy))

#define scene_get_scratchgc(xxx) ((xxx)->scratchgc)
#define scene_set_scratchgc(xxx,yyy) ((xxx)->scratchgc = (yyy))

#define scene_get_transform(xxx) ((xxx)->transform)

#define scene_get_maxShape(xxx) ((xxx)->maxShape)
#define scene_set_maxShape(xxx,yyy) ((xxx)->maxShape = (yyy))


#endif /* _cs_canvas_h */
