/*
 * @(#)cs_cmapst.h %23.5 90/06/19
 */

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

/*
 * cs_cmapst.h - colormap structures
 */

#ifndef cmapst_h
#define cmapst_h 1

#include	"cs_cmap.h"
#include	"cs_screene.h"

#ifndef COLORLIST_SIZE
#include <graphics/sh_clrlst.h>
#endif

#ifdef CMAPGRAB
#include <graphics/sh_cmap_grab.h>
#endif CMAPGRAB

/* 
 * Shared color -- the color is used by AllocColorPlanes 
 * NOTE that this isn't used yet
 */

typedef struct {
    unsigned short color;
    short       refcnt;
}           SHAREDCOLOR;

/*
 * LOCO -- a local color for a PseudoColor cell. DirectColor maps always use
 * the first value (called red) in the structure.  What channel they are
 * really talking about depends on which map they are in.
 */
typedef struct {
    unsigned short red,
                green,
                blue;
}           LOCO;

/*
 * SHCO -- a shared color for a PseudoColor cell. Used with AllocColorPlanes.
 * DirectColor maps always use the first value (called red) in the structure.
 * What channel they are really talking about depends on which map they are
 * in.
 */
typedef struct {
    SHAREDCOLOR *red,
               *green,
               *blue;
}           SHCO;


/* flags for entry usage */
#define		USED		1
#define		PRIVATE		2
#define		USE_CLIST	4

/* beyond the possible number of Clients -- safe for end of list marker */
#define		END_OF_CLIST	0xff

/* color map entry */
typedef struct _CMEntry {
    REF 	pixel;		/* points to cmap entry that owns it (corpus) */
    char        used;		/* we may be filled, but not have a struct
				 * body if its a static map */
    union	{
	unsigned char	*cptr;
	unsigned char	carray[4];
    } clientlist;
}           Entry;


/*
 * COLORMAPs can be used for either Direct or Pseudo color.  PseudoColor only
 * needs one cell table, we arbitrarily pick red.  We keep track of that table
 * with freeRed, numPixelsRed, and clientPixelsRed
 */

typedef struct _ColormapRec {
    ENTITY_FIELDS		/* header and ref information */
    REF         pVisual;
    long        mid;		/* client's name for colormap */
    SCREENINFO  pScreen;	/* screen map is associated with */
    short       class;		/* PseudoColor or DirectColor */
    short       flags;		/* 1 = IsDefault 2 = AllAllocated */
    int         size;
    int         free;
    CMAP_ENTRY  slots;		/* Who's allocated what */
    COLORLIST  *colors;		/* Actual RGB values */
#ifdef CMAPGRAB
    Grabbedcmap *grab_info;	/* NULL if colormap is not grabbed */
#endif CMAPGRAB
}           ColormapRec;

#ifdef CMAPGRAB
#define cs_cmap_is_grabbed(cmap)\
	((cmap)->grab_info)
#endif CMAPGRAB

/*
 * note that this is the same as xColorItem in Xproto.h - DON"T CHANGE IT
 * CARELESSLY!!!  StoreColors() uses it, and the data needs to get passed down
 * politely
 */
typedef struct _citem {
    Pixel       pixel;
    unsigned short red,
                green,
                blue;
    unsigned char flags,
                pad;
}           CITEM;

/* same as above for this struct - xrgb */
typedef struct _rgb {
    unsigned short red,
                green,
                blue,
                pad;
}           RGB;


typedef struct {
	unsigned long red_max;
	unsigned long red_mult;
	unsigned long green_max;
	unsigned long green_mult;
	unsigned long blue_max;
	unsigned long blue_mult;
	unsigned long base_pixel;
} Color_cube_desc;



/*
 * REMIND
 * 
 * probably don't really need arrays of pixels - a single one should suffice.  
 * In the original, the array was for each client, and had flags telling what 
 * the client was using from the map.  in our method, that info should be 
 * moved to the resource DB.
 * 
 * but for now, it'll stay in there.
 */
#endif				/* cmapst_h */
