/*
 * @(#)cs_common.h 23.3 90/06/19 CSCRIPT SMI
 */

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

/*
 * cs_common.h - common public definitions used everywhere in CScript.
 */

#ifndef _cs_common_h_
#define _cs_common_h_

#include <Core/core.h>

#ifndef _SHAPES
#include <graphics/shapes.h>		/* includes fract info */
#endif

#define cfloorfr(x) lfloorfr(x)

struct fpoint {			/* A point in subpixel coordinates */
    fract	x, y;
};

struct spoint {			/* A point, rounded to pixel coordinates */
    short	x, y;
};

struct rpoint {			/* A real point */
    float     x, y;
};

typedef unsigned int FLAG;	/* A boolean value used as a bitfield */
				/* in a structure. */

/* some proc decls */
void cs_frcurrentdashpattern();
void cs_frsetdashpattern();
void cs_frsetdashoffset();
void cs_destroy_rooted_canvas();

/* 
 * The following are functions converted to macros. They are listed
 * here by files which the corresponding functions used to part of.
 */

	/* clip.c */

#ifndef NEWS_ONLY
extern void cs_initouterclip();
extern void cs_translateclip();
#endif

extern void cv_clip_with_rule();
#define cs_clip(gc)							\
	(cv_clip_with_rule(gc_ref(gc), CTX_FILL_NONZERO))
#define cs_eoclip(gc)							\
	(cv_clip_with_rule(gc_ref(gc), CTX_FILL_EVENODD))

extern void cv_clipcanvas_with_rule();
#define cs_clipcanvas(gc)						\
	(cv_clipcanvas_with_rule(gc_ref(gc), CTX_FILL_NONZERO))
#define cs_eoclipcanvas(gc)						\
	(cv_clipcanvas_with_rule(gc_ref(gc), CTX_FILL_EVENODD))

	/* color.c */

void cv_setrgb8color();
void cs_uninstall_cmap();
#define cs_setfgpixel(gc, value)					\
	(gc_set_fgpixel(gc_ref(gc), value))
#define cs_setbgpixel(gc, value)					\
	(gc_set_bgpixel(gc_ref(gc), value))
#define cs_currentfgpixel(gc)						\
	((long)gc_get_fgpixel(gc_ref(gc)))
#define cs_currentbgpixel(gc)						\
	((long)gc_get_bgpixel(gc_ref(gc)))
#define cs_setcolor(gc, color)						\
	Set_Context(ctx_ref(gc), CTX_COLOR, &color);

	/* context.c */

#ifndef NEWS_ONLY
extern void cs_paintouter();
#endif

#define cs_flsetlinewidth(gc, width)					\
	Set_Context(ctx_ref(gc), CTX_LINE_SIZE, width)
#define cs_frsetlinewidth(gc, width)					\
	Set_Context(ctx_ref(gc), CTX_LINE_SIZE_FRACT, width)
#define cs_setlinewidth(gc, width)					\
	cs_frsetlinewidth ((gc), fracti(width));
#define cs_flcurrentlinewidth(gc)					\
	((float) Get_Context(ctx_ref(gc), CTX_LINE_SIZE))
#define cs_frcurrentlinewidth(gc)					\
	((fract) Get_Context(ctx_ref(gc), CTX_LINE_SIZE_FRACT))
#define cs_currentlinewidth(gc)						\
	roundfr(cs_frcurrentlinewidth(gc))

#define cs_setlinecap(gc, cap)						\
	(Set_Context(ctx_ref(gc), CTX_LINE_CAP, (CTX_line_cap) cap))
#define cs_currentlinecap(gc)						\
	((enum cs_linecap)Get_Context(ctx_ref(gc), CTX_LINE_CAP))
#define cs_setlinejoin(gc, join)					\
	(Set_Context(ctx_ref(gc), CTX_LINE_JOIN, (CTX_line_join) join))
#define cs_currentlinejoin(gc)						\
	((enum cs_linejoin)Get_Context(ctx_ref(gc), CTX_LINE_JOIN))

#define cs_emptypath(gc)						\
	(cv_emptypath(gc_ref(gc)))
#define cv_emptypath(gc)						\
	(gc_get_pathisshape(gc)						\
	    ? Get_Shape(gc_get_pathshape(gc), SHAPE_EMPTY)		\
	    : Get_Path(path_gc(gc), PATH_EMPTY))

#define cs_setpath(gc, shape)						\
	cv_setpath(gc_ref(gc), shape)

#define cs_translate_shape(shape, x, y)					\
	cv_translate_shape((shape), (x), (y))
#define cv_translate_shape(shape, x, y)					\
	Translate_Shape((shape), (x), (y))

#define cs_currentpaintthrough(gc)					\
	((gc_ref(gc))->drawthrough)

#define cs_setfillrule(gc, rule)					\
	(cv_setfillrule(gc_ref(gc), (rule)))
#define cv_setfillrule(gc, rule)					\
	(Set_Context(gc_get_context(gc), CTX_AREA_STYLE, (rule)))

#define cs_currentfillrule(gc)						\
	((CTX_area_style)Get_Context(ctx_ref(gc), CTX_AREA_STYLE))
#define cs_setrasteropcode(gc, code)					\
	(Set_Context(ctx_ref(gc), CTX_ROP, (CTX_rop) code))
#define cs_currentrasteropcode(gc)					\
	((CTX_rop)Get_Context(ctx_ref(gc), CTX_ROP))

#define cs_hascurrentpoint(gcref)					\
	cv_hascurrentpoint(gc_ref(gcref))
#define cs_nocurrentpoint(gcref)					\
	cv_nocurrentpoint(gc_ref(gcref))
#define cv_hascurrentpoint(gc)						\
	(!cv_nocurrentpoint(gc))
#define cv_nocurrentpoint(gc)						\
	(gc_get_pathisshape(gc)||Get_Context(gc_get_context(gc), CTX_EMPTY))

#define cs_setlinestyle(gc, line_style) \
	(Set_Context(ctx_ref(gc), CTX_LINE_STYLE, (CTX_line_style) line_style))
#define cs_currentlinestyle(gc)						\
	((CTX_line_style)Get_Context(ctx_ref(gc), CTX_LINE_STYLE))

#define cs_setflat(gc, flatness)					\
	cv_setflat(gc_ref(gc), (flatness))

#define cs_currentflat(gc)						\
	cv_currentflat(gc_ref(gc))

#define cs_currentmiterlimit(gc)					\
	(Get_Context(ctx_ref(gc), CTX_MITER_LIMIT_FRACT))
#define cs_setmiterlimit(gc, limit)					\
	Set_Context(ctx_ref(gc), CTX_MITER_LIMIT_FRACT, limit)

#define cs_setliteralpixels(gc, val)					\
	(gc_set_literalpixels(gc_ref(gc), val))

	/* copyarea.c */

extern int cv_shiftareawithrule();
#define cs_frshiftarea(gc, dx, dy)					\
	(cv_shiftareawithrule(gc_ref(gc), dx, dy, CTX_FILL_NONZERO))
#define cs_eofrshiftarea(gc, dx, dy)					\
	(cv_shiftareawithrule(gc_ref(gc), dx, dy, CTX_FILL_EVENODD))

	/* cs_overlay.c */

#define cs_isoverlay(cv)						\
	(canvas_get_overlay(rcv_ref(cv)))

	/* gsave.c */

#define cs_grestoreable(gc)						\
	(gc_get_gsaved(gc_ref(gc)))

	/* scene.c */

extern void cv_map();
#define cs_map(cv, savebits) {						\
	canvas_set_maprequested(rcv_ref(cv), 1);			\
	cv_map(rcv_ref(cv), savebits);					\
}
extern void cv_unmap();
#define cs_unmap(cv, savebits) {					\
	cv_unmap(rcv_ref(cv), savebits, 1, 0, 0);			\
	canvas_set_maprequested(rcv_ref(cv), 0);			\
}

#define cs_hasdamagearea(cv)						\
	((rcv_ref(cv))->damage)

#define cs_hasgraphicsdamagearea(cv)					\
	((ocv_ref(cv))->graphicsDamage)

#define cs_cleardamagearea(cv) {					\
	assert((rcv_ref(cv))->damage);					\
	Destroy_Shape((rcv_ref(cv))->damage);				\
	(rcv_ref(cv))->damage = (SHAPE) 0;				\
}

	/* sceneutils.c */

#ifndef NEWS_ONLY
extern void cs_xsetcanvas();
extern void cs_extendvisibledamage();
#endif
extern void cv_setcanvasshapewithrule();
#define cs_setcanvasshape(gc, cv, shape, delta)				\
	(cv_setcanvasshapewithrule(gc_ref(gc), rcv_ref(cv),		\
		CTX_FILL_NONZERO, shape, delta))
#define cs_eosetcanvasshape(gc, cv, shape, delta)			\
	(cv_setcanvasshapewithrule(gc_ref(gc), rcv_ref(cv),		\
		CTX_FILL_EVENODD, shape, delta))
#define cs_make_saveback(cv)						\
	(canvas_set_saveback(rcv_ref(cv), 1))
#define cs_make_nonsaveback(cv)						\
	(canvas_set_saveback(rcv_ref(cv), 0))

#define cs_destroypath(path)						\
	(Destroy_Path(path))
#define cs_destroyshape(shape)						\
	(Destroy_Shape(shape))

#define cs_currentcanvas(gc)						\
	(gc_get_canvas(gc_ref(gc)))
#define cs_setcurrentcanvas(gc, cv)					\
	(gc_set_canvas(gc_ref(gc), (cv)))

/*
 * REMINDJDN
 * These names have become misleading. These should really be
 * called something like cs_globalroottocanvas() and
 * cs_canvastoglobalroot().
 */
#define cs_screentocanvas(cv, X, Y, dx, dy) {				\
	*(dx) = (X) - canvas_get_screenposx(rcv_ref(cv))		\
	- canvas_get_x(rcv_ref(cs_screeninfo(cv)->canvases));		\
	*(dy) = (Y) - canvas_get_screenposy(rcv_ref(cv))		\
	- canvas_get_y(rcv_ref(cs_screeninfo(cv)->canvases));		\
}
#define cs_canvastoscreen(cv, X, Y, dx, dy) {				\
	*(dx) = (X) + canvas_get_screenposx(rcv_ref(cv))		\
	+ canvas_get_x(rcv_ref(cs_screeninfo(cv)->canvases));		\
	*(dy) = (Y) + canvas_get_screenposy(rcv_ref(cv))		\
	+ canvas_get_y(rcv_ref(cs_screeninfo(cv)->canvases));		\
}

#define cv_isglobalroot(cv)						\
	(cv == GlobalRoot)
#define cv_isroot(cv)							\
	(((cv)->type == xn_canvas_type) && ((cv)->screeninfo)		\
	&& (ocv_ref((cv)->screeninfo->canvases) == (OCANVAS) (cv)))

#define cs_typedmoveto(gcref, FIELD, TYPE, X, Y) do {			\
	GRAPHICS_CONTEXT gc = gc_ref(gcref);				\
	CONTEXT ctx = ctx_gc(gc);					\
	PATH_POINT *pptr = ctx->ctx_buf_ptr;				\
	if (gc_get_pathisshape(gc)) {					\
	    Destroy_Shape(gc_get_pathshape(gc));			\
	    gc_set_pathisshape(gc, 0);					\
	}								\
	gc_set_pathisclosed(gc, 0);					\
	pptr->FIELD.t = PT_MOVE | (TYPE);				\
	pptr->FIELD.x = (X); pptr->FIELD.y = (Y);			\
	ctx->ctx_buf_ptr = ++pptr;					\
	if (pptr >= ctx->ctx_buf_end) Extend_Context(ctx);		\
} while (0)

#define cs_moveto(gc, x, y)						\
	cs_typedmoveto(gc, b, PT_BIN2D, fracti(x), fracti(y))
#define cs_frmoveto(gc, x, y)						\
	cs_typedmoveto(gc, b, PT_BIN2D, x, y)
#define cs_flmoveto(gc, x, y)						\
	cs_typedmoveto(gc, f, PT_FLT2D, x, y)


#define cs_lineto(gc, x, y)						\
	Draw_I2D(ctx_ref(gc), PT_LINE, x, y)
#define cs_fllineto(gc, x, y)						\
	Draw_F2D(ctx_ref(gc), PT_LINE, x, y)
#define cs_frlineto(gc, x, y)						\
	Draw_B2D(ctx_ref(gc), PT_LINE, x, y)

#define cs_rmoveto(gc, x, y)						\
	Draw_I2D(ctx_ref(gc), PT_MOVEREL, (x), (y))
#define cs_flrmoveto(gc, x, y)						\
	Draw_F2D(ctx_ref(gc), PT_MOVEREL, (x), (y))
#define cs_frrmoveto(gc, x, y)						\
	Draw_B2D(ctx_ref(gc), PT_MOVEREL, (x), (y))

#define cs_rlineto(gc, x, y)						\
	Draw_I2D(ctx_ref(gc), PT_LINEREL, (x), (y))
#define cs_flrlineto(gc, x, y)						\
	Draw_F2D(ctx_ref(gc), PT_LINEREL, (x), (y))
#define cs_frrlineto(gc, x, y)						\
	Draw_B2D(ctx_ref(gc), PT_LINEREL, (x), (y))

	/* sharefile.c */

#ifdef SHARECANVAS
extern int cv_sharefile_map();
#define cs_sharefile_map(fn,cv,mapflag)					\
	(cv_sharefile_map(fn, ocv_ref(cv), mapflag))
extern int cv_sharefile_unmap();
#define cs_sharefile_unmap(cv)						\
	(cv_sharefile_unmap(ocv_ref(cv)))
extern void cv_sharefile_restore();
#define cs_sharefile_restore(cv)					\
	(cv_sharefile_restore(ocv_ref(cv)))
extern int cv_rowbytes_info();
#define cs_rowbytes_info(cv)						\
	(cv_rowbytes_info(ocv_ref(cv)))
#endif

	/* texture.c */

extern void cs_setfillcanvas();

#define cs_setfillorigin(gc, x, y) {					\
	POINT p;							\
	Set_Context(ctx_ref(gc), CTX_FILL_ORG, Point_I2D(&p, x, y));	\
}
#define cs_setfillstyle(gc, style)					\
	(Set_Context(ctx_ref(gc), CTX_FILL_STYLE, style))
#define cs_currentfillstyle(gc)						\
	((CTX_fill_style) Get_Context(ctx_ref(gc), CTX_FILL_STYLE))

#endif /* _cs_common_h_ */

