/*
 * @(#)cs_context.h 23.2 90/06/19 CSCRIPT SMI
 */

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

/*
 * cs_context.h - Graphics context data structure and information.
 */

#ifndef _cs_context_h_
#define _cs_context_h_

#include <graphics/shapes.h>
#include <Core/core.h>
#include "cs_canvas.h"
#include "cs_psfont.h"

struct cs_transfers {
    ENTITY_FIELDS		/* header and ref information */
    fract trsfrs[256];
};

struct cs_htcache {		/* Halftone pattern cache entry */
    RASTER      pattern;	/* A halftone pattern */
    int         grey;		/* The grey level it represents */
    int         lastused;	/* When it was last used */
};

#define HTCACHESIZE 4

struct cs_htinfo {		/* Intermediate structure used in building
				   a threshold array */
    fract       value;		/* Value returned from spot function */
    unsigned short cellrank;	/* A discriminating value used to separate
				   pixels with equal values */
    short       index;		/* The index of this pixel before sorting */
};

struct cs_halftone {
    ENTITY_FIELDS		/* header and ref information */
    short       w,		/* The size of this threshold array */
                h;
    short       dx,		/* displacement from one halftone cell to
				   the next within this threshold array */
                dy;
    int         pos,		/* The position of the next element to be
				   filled into the htinfo array: used only
				   when building the screen */
                limit,		/* The position of the end of this
				   scan line */
                nrep;		/* The number of times to repeat the halftone
				   cell in X and Y to get the edges to
				   stitch together */
    fract	angle,		/* the parameters to setscreen */
		frequency;
    struct object threshold;	/* The threshold array as a string */
    struct object SpotFunction;	/* screen spot function */
    struct cs_htinfo *htinfo;	/* Intermediate storage used while building
				   the threshold array */
    struct cs_htcache htcache[HTCACHESIZE];	/* A cache of recently
						   generated bitmaps for
						   various grey values */
};

typedef struct cs_transfers *TRANSFERS;
typedef struct cs_halftone *HALFTONE;

#define CS_GC_FIELDS							      \
    ENTITY_FIELDS		/* head and ref information */		      \
    REF canvas;			/* Our canvas. */			      \
    CONTEXT context;		/* A shapes context. */			      \
    REF font;			/* The current text font (struct psfont). */  \
    struct								      \
    graphics_context *gsaved;	/* a pointer to a saved context.	      \
				 * the chain of pointers form a linked list.  \
				 */					      \
									      \
    long fgpixel;		/* desired foreground pixel value */	      \
    long bgpixel;		/* desired background pixel value */	      \
    long curfgpixel;		/* Current pixel fg value sent to shapes */   \
    long curbgpixel;		/* Current pixel fg value sent to shapes */   \
									      \
    FLAG paintouter : 1;	/* allow painting to area between outerclip   \
				 * and innerclip if it exists		      \
				 */					      \
									      \
    FLAG drawthrough : 1;	/* allow painting through canvas children */  \
									      \
    FLAG literalpixels : 1;     /* If true, paint only with the pixel values  \
				 * in the context--never use a fill pattern   \
				 * to approximate a color.		      \
				 */					      \
									      \
    FLAG printermatch : 1;	/* True if printer matching is requested for  \
				 * the current font.			      \
				 */					      \
									      \
    FLAG pathisshape : 1;	/* True if context->CTX_PATH is invalid and   \
				 * gc->pathshape should be used instead	      \
				 */					      \
									      \
    FLAG pathisclosed : 1;	/* True if path has been closed		      \
				 * since it was created.		      \
				 */					      \
									      \
    FLAG pad:2;								      \
									      \
    SHAPE pathshape;		/* currentpath if gc->pathisshape is set */   \
    REF transfers; 		/* pointer to transfer function results */    \
    REF halftone; 		/* pointer to halftone matrix */

struct graphics_context {
    CS_GC_FIELDS
};

typedef struct graphics_context* GRAPHICS_CONTEXT;

#define gc_get_canvas(xxx) ((xxx)->canvas)
#define gc_set_canvas(xxx,yyy) ((xxx)->canvas = (yyy))

#define gc_get_context(xxx) ((xxx)->context)
#define gc_set_context(xxx,yyy) ((xxx)->context = (yyy))

#define gc_get_gsaved(xxx) ((xxx)->gsaved)
#define gc_set_gsaved(xxx,yyy) ((xxx)->gsaved = (yyy))

#define gc_get_font(xxx) ((xxx)->font)
#define gc_set_font(xxx,yyy) ((xxx)->font = (yyy))

#define gc_get_paintouter(xxx) ((xxx)->paintouter)
#define gc_set_paintouter(xxx,yyy) ((xxx)->paintouter = (yyy))

#define gc_get_drawthrough(xxx) ((xxx)->drawthrough)
#define gc_set_drawthrough(xxx,yyy) ((xxx)->drawthrough = (yyy))

#define gc_get_literalpixels(xxx) ((xxx)->literalpixels)
#define gc_set_literalpixels(xxx,yyy) ((xxx)->literalpixels = (yyy))

#define gc_get_printermatch(xxx) ((xxx)->printermatch)
#define gc_set_printermatch(xxx,yyy) ((xxx)->printermatch = (yyy))

#define gc_get_pathisshape(xxx) ((xxx)->pathisshape)
#define gc_set_pathisshape(xxx,yyy) ((xxx)->pathisshape = (yyy))

#define gc_get_pathisclosed(xxx) ((xxx)->pathisclosed)
#define gc_set_pathisclosed(xxx,yyy) ((xxx)->pathisclosed = (yyy))

#define gc_get_fgpixel(xxx) ((xxx)->fgpixel)
#define gc_set_fgpixel(xxx,yyy) ((xxx)->fgpixel = (yyy))

#define gc_get_bgpixel(xxx) ((xxx)->bgpixel)
#define gc_set_bgpixel(xxx,yyy) ((xxx)->bgpixel = (yyy))

#define gc_get_curfgpixel(xxx) ((xxx)->curfgpixel)
#define gc_set_curfgpixel(xxx,yyy) ((xxx)->curfgpixel = (yyy))

#define gc_get_curbgpixel(xxx) ((xxx)->curbgpixel)
#define gc_set_curbgpixel(xxx,yyy) ((xxx)->curbgpixel = (yyy))

#define gc_get_pathshape(xxx) ((xxx)->pathshape)
#define gc_set_pathshape(xxx,yyy) ((xxx)->pathshape = (yyy))

#define gc_get_transfers(xxx) ((xxx)->transfers)
#define gc_set_transfers(xxx,yyy) ((xxx)->transfers = (yyy))

#define gc_get_halftone(xxx) ((xxx)->halftone)
#define gc_set_halftone(xxx,yyy) ((xxx)->halftone = (yyy))

#define gc_get_transform(xxx)				\
    (Get_Context((xxx)->context, CTX_VIEW_TRANSFORM))

enum cs_linejoin {
    cs_miterjoin,
    cs_roundjoin,
    cs_beveljoin,
};

enum cs_linecap {
    cs_buttcap,
    cs_roundcap,
    cs_squarecap,
    cs_notlast
};

#define cs_currentprintermatch(gc)					\
    gc_get_printermatch(gc_ref(gc))
#define cs_setprintermatch(gc, value)					\
    gc_set_printermatch(gc_ref(gc), (value))

extern void cv_paintinner();
#define cs_paintinner(gc)						\
	cv_paintinner(gc_ref(gc))

extern void cv_dontpaintthrough();
#define cs_dontpaintthrough(gc)						\
	cv_dontpaintthrough(gc_ref(gc))

extern void cv_setplanemask();
#define cs_setplanemask(gc, mask)					\
	cv_setplanemask(gc_ref(gc), mask)

extern void cs_abortpathread();

#endif /* _cs_context_h_ */
