/* @(#)cs_psfont.h 23.2 90/06/19 */

/*
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

/*
 * Definitions for psfont, fontfamily and related structures, 
 * and the font caching mechanism.
 */

#ifndef _cs_psfont_h
#define _cs_psfont_h

#include "cs_font.h"		/* this includes <Core/core.h> */

/*
 * A psfont - used as a handle to cscript font information.
 * The 'translation' table for a remapped font is a 256-
 * element array of shorts.  The array is allocated in
 * encodefont() in nucleus and is shared by multiple fonts.
 */
struct psfont {
    ENTITY_FIELDS		/* Header and ref information */
    REF family;			/* Fontfamily struct associated with font */
    fract       matrix[3][2];	/* Matrix used for scaling and rotation */
    Sgn8        simplescale;	/* True iff matrix == [s 0 0 s 0 0] */
    Sgn8	printermatch;	/* True iff this font is to be printermatched */
    Sgn16       revision;	/* Matched against the last transform that
				 * lasthit was validated against */
    Unsgn16     *remap;		/* Points to a translation table if this is
				 * a remapped font */
    REF lasthit;		/* The last real font this psfont resolved to */
};

typedef struct psfont* PSFONT;

/*
 * A cached font.
 * If name is NULL, the font was *not* read in from a file.
 */
struct cachedfont {	
    Sgn8	*name;		/* Name of file containing this font */
    Sgn32	hash;		/* The matrix hash for this font */
    struct font *font;		/* The font corresponding to this cache entry;
				 * null=>the font hasn't been read in yet */
    struct cachedfont *next;	/* Next font in same hash bucket */
    fract       matrix[3][2];	/* Matrix used for scaling and rotation */
    unsigned int printermatched:1; /* true iff the widths in this font have been
				    * matched to the widths on a printer */
    Sgn16 pad;			/* Pad out structure */
};


#define FONTHASHSIZE 37		/* Prime numbers are good */
#define FONTTABLESIZE 73

#define HASHFONT(m) ((m)[0][0]+(m)[0][1]+(m)[1][0]+(m)[1][1])

enum ff_buildchar_op {
    BC_WIDTH_ONLY, BC_WIDTH_AND_SIZE_ONLY, BC_PAINT
};

struct fontfamily_opsvector {
    void (*initialize_font)();
    void (*buildchar)();
    void (*charpath)();
    void (*bbox)();
};

/*
 * The fonts in a font family are indexed two ways.  
 * First there is a chained hash table where the encoding and transformation 
 * matrix are used as the key.  It is used for general font lookup and
 * contains all fonts for this family.  
 * Second is a "simple scale" table: each entry has a transformation matrix
 * of the form [s 0 0 s 0 0] - uniform scaling with no rotation.  This table 
 * is indexed by roundfr(s).  Entries in this table point to cachedfont
 * structures also referenced by the hash table.
 * The printermatched entry may point to a single unique cachedfont that 
 * has widths that match a printer's.
 */
struct fontfamily {
    ENTITY_FIELDS		/* Header and ref information */
    union {
	Sgn32  magic;		/* FONTFAMILYMAGIC */
	struct fontfamily_opsvector* ops; /* Family specific methods */
    } m;
    Sgn8       *FamilyName;	/* eg. "Times-Roman" */
    Sgn8       *directory;	/* Directory this family came from (only
				 * used when validating cache entries) */
    Sgn8       *FolioName;	/* Corresponding Folio font, if any */
    union {
	Sgn8       *name;	/* The name of the encoding that
				 * describes the layout of the glyphs
				 * in this font. */
	struct object array;	/* When the family gets read it, the name
				 * gets translated to the array object that
				 * represents the named encoding */
    } encoding;
    struct cachedfont *printermatched;	     /* printer matching table */
    struct cachedfont *hash[FONTHASHSIZE];   /* hash table */
    struct cachedfont *table[FONTTABLESIZE]; /* simple scale table */
};

typedef struct fontfamily *FONTFAMILY;

#define fontfamily_initialize(ff, f, dir)				\
	(*(ff)->m.ops->initialize_font)((f), (dir))

#define fontfamily_buildchar(ff, f, charindex, dir, x, y, op)		\
	(*(ff)->m.ops->buildchar)((f), (charindex), (dir), (x), (y), (op))

#define fontfamily_charpath(ff, f, s, dir, x, y, gc)			\
	(*(ff)->m.ops->charpath)((f), (s), (dir), (x), (y), (gc))

#define fontfamily_bbox(ff, f, psf, ll, ur)				\
        (*(ff)->m.ops->bbox)((f), (psf), (ll), (ur)))

#define FONTFAMILYMAGIC  0x137A2B48

#endif /* _cs_psfont_h */
