/*
 * @(#)cs_screene.h 23.4 90/06/19
 */

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

/*
 * cs_screene.h - screen data structure information.
 */


#ifndef _cs_screene_h
#define _cs_screene_h

#include "cs_canvas.h"
#include "cs_cmap.h"

#define UNINITIALIZED (~0)		/* uninitialized resource IDs */
/* REMIND, stolen from X11/misc.h */
#ifndef MAXFORMATS
#define MAXFORMATS	8
#endif

struct pixmapFormat {
    unsigned char	depth;
    unsigned char	bitsPerPixel;
    unsigned char	scanlinePad;
};

typedef struct pixmapFormat *PIXMAPFORMAT;

/* Define access macros for all the pixmap format slots. */

#define pxmapfmt_get_depth(xxx) ((xxx)->depth)
#define pxmapfmt_set_depth(xxx,yyy) ((xxx)->depth = (yyy))

#define pxmapfmt_get_bitsPerPixel(xxx) ((xxx)->bitsPerPixel)
#define pxmapfmt_set_bitsPerPixel(xxx,yyy) ((xxx)->bitsPerPixel = (yyy))

#define pxmapfmt_get_scanlinePad(xxx) ((xxx)->scanlinePad)
#define pxmapfmt_set_scanlinePad(xxx,yyy) ((xxx)->scanlinePad = (yyy))



struct visual {
    ENTITY_FIELDS		/* header and ref information */
    long	vid;
    short	screen;    
    short       class;
    unsigned long	redMask, greenMask, blueMask;
    int		offsetRed, offsetGreen, offsetBlue;
    short       bitsPerRGBValue;
    short	ColormapEntries;
    short	nplanes;	/* = log2 (ColormapEntries). This does not
				 * imply that the screen has this many planes.
				 * it may have more or fewer */
};

typedef struct visual *VISUAL;

/* Define access macros for all the visual slots. */

#define visual_get_vid(xxx) ((xxx)->vid)
#define visual_set_vid(xxx,yyy) ((xxx)->vid = (yyy))

#define visual_get_screen(xxx) ((xxx)->screen)
#define visual_set_screen(xxx,yyy) ((xxx)->screen = (yyy))

#define visual_get_class(xxx) ((xxx)->class)
#define visual_set_class(xxx,yyy) ((xxx)->class = (yyy))

#define visual_get_redMask(xxx) ((xxx)->redMask)
#define visual_set_redMask(xxx,yyy) ((xxx)->redMask = (yyy))

#define visual_get_greenMask(xxx) ((xxx)->greenMask)
#define visual_set_greenMask(xxx,yyy) ((xxx)->greenMask = (yyy))

#define visual_get_blueMask(xxx) ((xxx)->blueMask)
#define visual_set_blueMask(xxx,yyy) ((xxx)->blueMask = (yyy))

#define visual_get_offsetRed(xxx) ((xxx)->offsetRed)
#define visual_set_offsetRed(xxx,yyy) ((xxx)->offsetRed = (yyy))

#define visual_get_offsetGreen(xxx) ((xxx)->offsetGreen)
#define visual_set_offsetGreen(xxx,yyy) ((xxx)->offsetGreen = (yyy))

#define visual_get_offsetBlue(xxx) ((xxx)->offsetBlue)
#define visual_set_offsetBlue(xxx,yyy) ((xxx)->offsetBlue = (yyy))

#define visual_get_bitsPerRGBValue(xxx) ((xxx)->bitsPerRGBValue)
#define visual_set_bitsPerRGBValue(xxx,yyy) ((xxx)->bitsPerRGBValue = (yyy))

#define visual_get_ColormapEntries(xxx) ((xxx)->ColormapEntries)
#define visual_set_ColormapEntries(xxx,yyy) ((xxx)->ColormapEntries = (yyy))

#define visual_get_nplanes(xxx) ((xxx)->nplanes)
#define visual_set_nplanes(xxx,yyy) ((xxx)->nplanes = (yyy))



struct depth {
    int		depth;
    int		numVids;
    REF  	*vids;		/* array of visual REFs for this depth */
};

typedef struct depth *DEPTH;

/* Define access macros for all the depth slots. */

#define depth_get_depth(xxx) ((xxx)->depth)
#define depth_set_depth(xxx,yyy) ((xxx)->depth = (yyy))

#define depth_get_numVids(xxx) ((xxx)->numVids)
#define depth_set_numVids(xxx,yyy) ((xxx)->numVids = (yyy))

#define depth_get_vids(xxx) ((xxx)->vids)
#define depth_set_vids(xxx,yyy) ((xxx)->vids = (yyy))



/* A screene is a framebuffer */
struct screeninfo {

    int                 myNum;          /*  of this instance in */
					/*  screene[] */

    short		width, 		/* dimensions of this screen */
			height;

    short		mmWidth, 
			mmHeight;

    REF canvases;		/* Points to the root canvas of a */
				/* display.  This is the */
				/* "framebuffer." Its siblings will */ 
				/* be root canvases of other displays, */
				/* its children will be the usual */
				/* opaque windows. */

    RASTER display;		/* The device raster. */

    CONTEXT scratchgc;		/* A scratch Shapes context. */

    FMATRIX trans;

    SHAPE maxShape;		/* The initial and maximum shape */
				/* of the framebuffer */

#ifdef WINDOWGRABBER
    int	wgClientNum;		/* number of window-grabbed clients */
				/* on this screen */
#endif


    int			scrLeft,
			scrTop,
			scrWidth,
			scrHeight;

    short		numDepths;
    short       	rootDepth;
    DEPTH       	allowedDepths;

    short       	numVisuals;
    REF			*visuals;	/* the array of visuals that apply */
    REF			pRootVisual;

    short		minInstalledCmaps, 
			maxInstalledCmaps;

    REF			defaultmap;	/* default builtin colormap */
    REF			staticmap;	/* quick pointer to the StaticColor visual 
					   for use by NeWS -- Note: if -staticvis 
					   these two ptrs will be the same */

    REF			installedmap;	/* current user colormap */
    unsigned long	whitePixel,
			blackPixel;

    char		*fbname;
    char		*ospriv;	/* dependent on os */

    unsigned long	rgf;	/* array of flags; she's -- HUNGARIAN */

    REF			GCperDepth[MAXFORMATS+1];
			/* next field is a stipple to use as default in
			   a GC.  we don't build default tiles of all depths
			   because they are likely to be of a color
			   different from the default fg pixel, so
			   we don't win anything by building
			   a standard one.
			*/
    REF			PixmapPerDepth[1];

    char                backingStoreSupport,
			saveUnderSupport;

    /* Random screen procedures */
    void (* QueryBestSize)();		/* class, pwidth, pheight */

    /* Colormap procedures */
    void (* InstallColormap)();		/* pcmap */
    void (* UninstallColormap)();	/* pcmap */
    void (* StoreColors)();		/* pmap, ndef, pdef */
    void (* ResolveColor)();		/* pred, pgreen, pblue */

    /* 
    ** Shadow copy of installed colormap.
    **
    ** REMIND: this is not optimal.  Should be eliminated
    ** as a part of the osi_setcolors/cv_reload_cmap
    ** cleanup.
    */
    unsigned char   red_installed[256];
    unsigned char   green_installed[256];
    unsigned char   blue_installed[256];
};

typedef struct screene *SCREENE;

/* Old names for compatability */

typedef struct screene *SCENE;
typedef struct screene *SCREENDATA;

/* New Structure Name */
typedef struct screeninfo *SCREENINFO;

/* All Macros Below are for compatability */
 

/* Define access macros for screenes. */

#define screene_get_myNum(xxx) ((xxx)->myNum)
#define screene_set_myNum(xxx,yyy) ((xxx)->myNum = (yyy))

#define screene_get_width(xxx) ((xxx)->width)
#define screene_set_width(xxx,yyy) ((xxx)->width = (yyy))

#define screene_get_height(xxx) ((xxx)->height)
#define screene_set_height(xxx,yyy) ((xxx)->height = (yyy))

#define screene_get_mmWidth(xxx) ((xxx)->mmWidth)
#define screene_set_mmWidth(xxx,yyy) ((xxx)->mmWidth = (yyy))

#define screene_get_mmHeight(xxx) ((xxx)->mmHeight)
#define screene_set_mmHeight(xxx,yyy) ((xxx)->mmHeight = (yyy))

#define screene_get_scene(xxx) ((xxx))
#define screene_set_scene(xxx,yyy) ((xxx) = (yyy))

#define screene_get_scrLeft(xxx) ((xxx)->scrLeft)
#define screene_set_scrLeft(xxx,yyy) ((xxx)->scrLeft = (yyy))

#define screene_get_scrTop(xxx) ((xxx)->scrTop)
#define screene_set_scrTop(xxx,yyy) ((xxx)->scrTop = (yyy))

#define screene_get_scrWidth(xxx) ((xxx)->scrWidth)
#define screene_set_scrWidth(xxx,yyy) ((xxx)->scrWidth = (yyy))

#define screene_get_scrHeight(xxx) ((xxx)->scrHeight)
#define screene_set_scrHeight(xxx,yyy) ((xxx)->scrHeight = (yyy))

#define screene_get_numDepths(xxx) ((xxx)->numDepths)
#define screene_set_numDepths(xxx,yyy) ((xxx)->numDepths = (yyy))

#define screene_get_rootDepth(xxx) ((xxx)->rootDepth)
#define screene_set_rootDepth(xxx,yyy) ((xxx)->rootDepth = (yyy))

#define screene_get_allowedDepths(xxx) ((xxx)->allowedDepths)
#define screene_set_allowedDepths(xxx,yyy) ((xxx)->allowedDepths = (yyy))

#define screene_get_numVisuals(xxx) ((xxx)->numVisuals)
#define screene_set_numVisuals(xxx,yyy) ((xxx)->numVisuals = (yyy))

#define screene_get_visuals(xxx) ((xxx)->visuals)
#define screene_set_visuals(xxx,yyy) ((xxx)->visuals = (yyy))

#define screene_get_pRootVisual(xxx) ((xxx)->pRootVisual)
#define screene_set_pRootVisual(xxx,yyy) ((xxx)->pRootVisual = (yyy))

#define screene_get_minInstalledCmaps(xxx) ((xxx)->minInstalledCmaps)
#define screene_set_minInstalledCmaps(xxx,yyy) ((xxx)->minInstalledCmaps = (yyy))

#define screene_get_maxInstalledCmaps(xxx) ((xxx)->maxInstalledCmaps)
#define screene_set_maxInstalledCmaps(xxx,yyy) ((xxx)->maxInstalledCmaps = (yyy))

#define screene_get_defaultmap(xxx) ((xxx)->defaultmap)
#define screene_set_defaultmap(xxx,yyy) ((xxx)->defaultmap = (yyy))

#define screene_get_staticmap(xxx) ((xxx)->staticmap)
#define screene_set_staticmap(xxx,yyy) ((xxx)->staticmap = (yyy))

#define screene_get_installedmap(xxx) ((xxx)->installedmap)
#define screene_set_installedmap(xxx,yyy) ((xxx)->installedmap = (yyy))

#define screene_get_whitePixel(xxx) ((xxx)->whitePixel)
#define screene_set_whitePixel(xxx,yyy) ((xxx)->whitePixel = (yyy))

#define screene_get_blackPixel(xxx) ((xxx)->blackPixel)
#define screene_set_blackPixel(xxx,yyy) ((xxx)->blackPixel = (yyy))

#define screene_get_fbname(xxx) ((xxx)->fbname)
#define screene_set_fbname(xxx,yyy) ((xxx)->fbname = (yyy))

#define screene_get_ospriv(xxx) ((xxx)->ospriv)
#define screene_set_ospriv(xxx,yyy) ((xxx)->ospriv = (yyy))

#define canvas_get_scene(xxx)		((xxx)->screene)
#define canvas_set_scene(xxx, yyy)	((xxx)->screene = (yyy))

#define canvas_get_canvas(xxx)		((xxx)->canvases)
#define canvas_set_canvas(xxx, yyy)	((xxx)->canvases = (yyy))


typedef struct _DisplayInfo {
    int		imageByteOrder;
    int		bitmapScanlineUnit;
    int		bitmapScanlinePad;
    int		bitmapBitOrder;
    int		numPixmapFormats;
    struct pixmapFormat
		formats[MAXFORMATS];
    int		numScreens;
} DisplayInfo;

typedef struct _DisplayInfo *DISPLAYINFO;

/* Define access macros for screeninfo. */

#define displayinfo_get_imageByteOrder(xxx) ((xxx)->imageByteOrder)
#define displayinfo_set_imageByteOrder(xxx,yyy) ((xxx)->imageByteOrder = (yyy))

#define displayinfo_get_bitmapScanlineUnit(xxx) ((xxx)->bitmapScanlineUnit)
#define displayinfo_set_bitmapScanlineUnit(xxx,yyy) ((xxx)->bitmapScanlineUnit = (yyy))

#define displayinfo_get_bitmapScanlinePad(xxx) ((xxx)->bitmapScanlinePad)
#define displayinfo_set_bitmapScanlinePad(xxx,yyy) ((xxx)->bitmapScanlinePad = (yyy))

#define displayinfo_get_bitmapBitOrder(xxx) ((xxx)->bitmapBitOrder)
#define displayinfo_set_bitmapBitOrder(xxx,yyy) ((xxx)->bitmapBitOrder = (yyy))

#define displayinfo_get_numPixmapFormats(xxx) ((xxx)->numPixmapFormats)
#define displayinfo_set_numPixmapFormats(xxx,yyy) ((xxx)->numPixmapFormats = (yyy))

#define displayinfo_get_formats(xxx) ((xxx)->formats)
#define displayinfo_set_formats(xxx,yyy) ((xxx)->formats = (yyy))

#define displayinfo_get_numScreenes(xxx) ((xxx)->numScreens)
#define displayinfo_set_numScreenes(xxx,yyy) ((xxx)->numScreens = (yyy))

extern DisplayInfo displayInfo;
#define cs_displayinfo()                                                   \
	(&displayInfo)



#endif /* cs_screene_h */
