/*
 * @(#)cs_transform.h 23.2 90/06/19
 */

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

/*
 * cs_transform.h - Transform stuff.
 */

#ifndef _cs_transform_h_
#define _cs_transform_h_

#include <Core/core.h>

typedef fract FMATRIX[3][2];

void cv_load_frmatrix_from_transform();
TRANSFORM cv_load_transform_from_frmatrix();
FMATRIX *cv_defaultmatrix();
void cs_flsetdefaultmatrix();
void cs_frsetdefaultmatrix();
void cs_settransfer();
void cs_frconcat();
void cs_flconcatmatrix();

extern TRANSFORM cv_ScratchFractTransform;
extern TRANSFORM cv_ScratchFloatTransform;
extern TRANSFORM cv_ScratchFloatTransform1;
extern TRANSFORM cv_ScratchFloatTransform2;
extern POINT_B2D frscratchpoint;
extern POINT_F2D flscratchpoint;

#define cv_frscratchxform()	cv_ScratchFractTransform
#define cv_flscratchxform()	cv_ScratchFloatTransform
#define cv_flscratchxform1()	cv_ScratchFloatTransform1
#define cv_flscratchxform2()	cv_ScratchFloatTransform2


/* --- Routines internal to CScript --- */

#define RADIANS(d) (((double)(d)) *					\
       .017453292519943295769236907684886127134428718885417254560971914401710091146)

/*
 * cv_translate(t, dx, dy)
 * cv_fltranslate(t, dx, dy)
 * cv_frtranslate(t, dx, dy)
 *
 *   Concatenate a translation of (`dx', `dy') to the transform `t'.
 *   cv_translate() is the integer version.  cv_fltranslate() is
 *   the float version.  cv_frtranslate() is the fract version.
 */

#define cv_translate(t, dx, dy) {  					\
    Translate_Transform((t), Point_I2D(&frscratchpoint, (dx), (dy)));	\
}    
#define cv_fltranslate(t, dx, dy) {  					\
    Translate_Transform((t), Point_F2D(&flscratchpoint, (dx), (dy)));	\
}
#define cv_frtranslate(t, dx, dy) {  					\
    Translate_Transform((t), Point_B2D(&frscratchpoint, (dx), (dy))); 	\
}

/*
 * cv_scale (t, sx, sy)
 * cv_flscale (t, sx, sy)
 * cv_frscale (t, sx, sy) 
 *
 *   Concatenate a scaling transformation of (`sx', `sy').
 *   cv_scale() is the integer version.  cv_flscale() is the
 *   float version.  cv_frscale is the fract version.
 */
#define cv_scale(t, dx, dy) {  						\
    Scale_Transform((t), Point_I2D(&frscratchpoint, (dx), (dy)));	\
}    
#define cv_flscale(t, dx, dy) {  					\
    Scale_Transform((t), Point_F2D(&flscratchpoint, (dx), (dy)));	\
}
#define cv_frscale(t, dx, dy) {  					\
    Scale_Transform((t), Point_B2D(&frscratchpoint, (dx), (dy))); 	\
}

/*
 * cv_frrotate(t, angle)
 * cv_flrotate(t, angle)
 *
 *   Concatenate a rotation transform of `angle' degrees to the
 *   transform `t'.  cv_frrotate() is the fract version; cv_flrotate()
 *   is the float version.
 */

#define cv_rotate(t, angle) {						\
    Rotate_Transform(t,							\
	TRANS_AXIS_Z | TRANS_ANGLE_DEGREES | TRANS_ANGLE_FRACT,		\
	    fracti(angle));						\
}
#define cv_frrotate(t, angle) {						\
    Rotate_Transform(t,							\
	TRANS_AXIS_Z | TRANS_ANGLE_DEGREES | TRANS_ANGLE_FRACT,	angle);	\
}
#define cv_flrotate(t, angle) {  					\
    Rotate_Transform(t,							\
	TRANS_AXIS_Z | TRANS_ANGLE_DEGREES | TRANS_ANGLE_FLOAT, angle); \
}

/* 
 * transform (x,y) -> (xt, yt) 
 */
#define cv_frtransform(transform, X, Y, xt, yt) {  			\
    	Point_B2D (&frscratchpoint, (X), (Y));  			\
    	Point_Transform (transform, &frscratchpoint);  			\
    	*(xt) = frscratchpoint.x;  					\
    	*(yt) = frscratchpoint.y;  					\
}
#define cv_fltransform(transform, X, Y, xt, yt) {  			\
    	Point_F2D (&flscratchpoint, (double)(X), (double)(Y));  	\
    	Point_Transform (transform, &flscratchpoint);  			\
    	*(xt) = flscratchpoint.x;  					\
    	*(yt) = flscratchpoint.y;  					\
}
/* 
 * Inverse transforms. 
 */
#define cv_fritransform(transform, X, Y, xt, yt) {  			\
    	Point_B2D (&frscratchpoint, (X), (Y));  			\
    	InversePoint_Transform (transform, &frscratchpoint);  		\
    	*(xt) = frscratchpoint.x;  					\
    	*(yt) = frscratchpoint.y;  					\
}
#define cv_flitransform(transform, X, Y, xt, yt) {  			\
    	Point_F2D (&flscratchpoint, (double)(X), (double)(Y));  	\
    	InversePoint_Transform (transform, &flscratchpoint);  		\
    	*(xt) = flscratchpoint.x;  					\
    	*(yt) = flscratchpoint.y;  					\
}
/* 
 * Delta transforms.  
 */
#define cv_frdtransform(transform, X, Y, xt, yt) {  			\
	frscratchpoint.t = PT_MOVEREL_B2D,				\
	frscratchpoint.x = (X),						\
	frscratchpoint.y = (Y);						\
    	Point_Transform (transform, &frscratchpoint);  			\
    	*(xt) = frscratchpoint.x;  					\
    	*(yt) = frscratchpoint.y;  					\
}
#define cv_fldtransform(transform, X, Y, xt, yt) {  			\
	flscratchpoint.t = PT_MOVEREL_F2D,				\
	flscratchpoint.x = (double) (X),				\
	flscratchpoint.y = (double) (Y);				\
    	Point_Transform (transform, &flscratchpoint);  			\
    	*(xt) = flscratchpoint.x;  					\
    	*(yt) = flscratchpoint.y;  					\
}
#define cv_dtransform(transform, X, Y, xt, yt) {  			\
	frscratchpoint.t = PT_MOVEREL_B2D,				\
	frscratchpoint.x = fracti(X),					\
	frscratchpoint.y = fracti(Y);					\
    	Point_Transform (transform, &frscratchpoint);  			\
    	*(xt) = roundfr (frscratchpoint.x);  				\
    	*(yt) = roundfr (frscratchpoint.y);  				\
}
/* 
 * Inverse delta transforms. 
 */
#define cv_fridtransform(transform, X, Y, xt, yt) {  			\
	frscratchpoint.t = PT_MOVEREL_B2D,				\
	frscratchpoint.x = (X),						\
	frscratchpoint.y = (Y);						\
    	InversePoint_Transform (transform, &frscratchpoint);  		\
    	*(xt) = frscratchpoint.x;  					\
    	*(yt) = frscratchpoint.y;  					\
}
#define cv_flidtransform(transform, X, Y, xt, yt) {  			\
	flscratchpoint.t = PT_MOVEREL_F2D,				\
	flscratchpoint.x = (double) (X),				\
	flscratchpoint.y = (double) (Y);				\
    	InversePoint_Transform (transform, &flscratchpoint);  		\
    	*(xt) = flscratchpoint.x;  					\
    	*(yt) = flscratchpoint.y;  					\
}
#define cv_idtransform(transform, X, Y, xt, yt) {  			\
	frscratchpoint.t = PT_MOVEREL_B2D,				\
	frscratchpoint.x = fracti(X),					\
	frscratchpoint.y = fracti(Y);					\
    	InversePoint_Transform (transform, &frscratchpoint);  		\
    	*(xt) = roundfr (frscratchpoint.x);  				\
    	*(yt) = roundfr (frscratchpoint.y);  				\
}

/*
 * --- Exported routines --- 
 */
/*
 * cv_initmatrix (gc) -- initialize the CTM
 *
 *   Initialize the CTM of `gc'.  If the context refers to a
 *   canvas, then take the matrix from the default canvas transform.
 *   Otherwise, assume the identity transform.
 *   NOTE that canvas_get_transform() expects a pointer to an
 *   orphan canvas because that's where the transform information is.
 */
#define cv_initmatrix(gc) {  					   	\
    OCANVAS init_cv;					   		\
    init_cv = (OCANVAS) entity_addr(gc_get_canvas(gc));		   	\
    if (init_cv != (OCANVAS)0)				   	   	\
   	Write_b2d_Transform(gc_get_transform(gc),			\
			    canvas_get_transform(init_cv));		\
    else  							   	\
	Identity_Transform (gc_get_transform(gc));  		   	\
}

/*
 * cs_translate(gc, dx, dy)
 * cs_fltranslate(gc, dx, dy)
 * cs_frtranslate(gc, dx, dy)
 *
 *   Concatenate a translation of (`dx', `dy') to the CTM of `gc'.
 *   cs_translate() is the integer version.  cs_fltranslate() is the
 *   float version.  cs_frtranslate() is the fract version.
 */
#define cs_translate(gc, dx, dy) 					\
    	cv_translate(gc_get_transform((GRAPHICS_CONTEXT)		\
			(entity_addr(gc))), (dx), (dy))
#define cs_fltranslate(gc, dx, dy) 					\
    	cv_fltranslate(gc_get_transform((GRAPHICS_CONTEXT)		\
			(entity_addr(gc))), (dx), (dy))
#define cs_frtranslate(gc, dx, dy) 					\
    	cv_frtranslate(gc_get_transform((GRAPHICS_CONTEXT)		\
			(entity_addr(gc))), (dx), (dy))
/*
 * cs_scale(gc, sx, sy)
 * cs_flscale(gc, sx, sy)
 * cs_frscale(gc, sx, sy)
 *
 *   Concatenate a translation of (`dx', `dy') to the CTM of `gc'.
 *   cs_scale() is the integer version.  cs_flscale() is the float
 *   version.  cs_frscale() is the fract version.
 */
#define cs_scale(gc, sx, sy) 						\
    	cv_scale(gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))), \
				(sx), (sy))
#define cs_flscale(gc, sx, sy) 						\
    	cv_flscale(gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))),\
				(sx), (sy))
#define cs_frscale(gc, sx, sy) 						\
    	cv_frscale(gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))),\
				(sx), (sy))
/*
 * cs_flrotate (gc, angle)
 * cs_rotate (gc, angle)
 * cs_frrotate (gc, angle)
 *
 *   Concatenate a rotation transformation of `angle' degrees to the
 *   CTM of `gc'.  cs_flrotate() is the float version.  cs_rotate() is
 *   the integer version.  cs_frrotate() is the fract version.
 */
#define cs_flrotate(gc, angle) 						\
    	cv_flrotate(gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))), \
		angle)

#define cs_rotate(gc, angle) 						\
    	cv_rotate(gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))), \
		angle)

#define cs_frrotate(gc, angle) 						\
    	cv_frrotate(gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))), \
		angle)

/*
 * cs_currentmatrix (gc, matrix)
 *
 *   Copy the CTM of `gc' to `matrix'
 */
#define cs_currentmatrix(gc, matrix) 					\
    	cv_load_frmatrix_from_transform ((matrix), 			\
		gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))))

/*
 * cs_setmatrix (gc, matrix)
 *
 *   Set the CTM of `gc' to `matrix'
 */
#define cs_setmatrix(gc, matrix) 					\
    	(void) cv_load_transform_from_frmatrix ( 			\
	 gc_get_transform((GRAPHICS_CONTEXT)(entity_addr(gc))), (matrix))

/* 
 * --- forward transforms --- 
 */
#define cs_frtransform(gc, x, y, xt, yt) \
    	cv_frtransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
#define cs_fltransform(gc, x, y, xt, yt) 				\
    	cv_fltransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
/* --- Inverse transforms --- */
/*
 * cs_fritransform (gc, x, y, xt, yt)
 *
 *   Apply the inverse of gc's CTM to the point (x, y) and put the
 *   result in (xt, yt).
 */
#define cs_fritransform(gc, x, y, xt, yt) 				\
    	cv_fritransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
/*
 * cs_flitransform (gc, x, y, xt, yt)
 *
 *   Apply the inverse of gc's CTM to the point (x, y) and put the
 *   result in (xt, yt).
 */
#define cs_flitransform(gc, x, y, xt, yt) 				\
    	cv_flitransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
/* 
 * --- Delta transforms --- 
 */
/*
 * cs_fldtransform (gc, x, y, xt, yt)
 *
 * Apply all but the translation components of gc's CTM to the point
 * (x, y) and put the result in (xt, yt).
 */
#define cs_fldtransform(gc, x, y, xt, yt) 				\
    	cv_fldtransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
/*
 * cs_frdtransform (gc, x, y, xt, yt)
 *
 * Apply all but the translation components of gc's CTM to the point
 * (x, y) and put the result in (xt, yt).
 */
#define cs_frdtransform(gc, x, y, xt, yt) 				\
    	cv_frdtransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
#define cs_dtransform(gc, x, y, xt, yt) 				\
    	cv_dtransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))

/* 
 * --- Inverse delta transforms --- 
 */
/*
 * cs_fridtransform (gc, x, y, xt, yt)
 *
 *   Apply all but the translation components of gc's CTM to the point
 *   (x, y) and put the result in (xt, yt).
 */
#define cs_fridtransform(gc, x, y, xt, yt) 				\
    	cv_fridtransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
/*
 * cs_flidtransform (gc, x, y, xt, yt)
 *
 *   Apply all but the translation components of gc's CTM to the point
 *   (x, y) and put the result in (xt, yt).
 */
#define cs_flidtransform(gc, x, y, xt, yt) 				\
    	cv_flidtransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
/*
 * cs_idtransform (gc, x, y, xt, yt)
 */
#define cs_idtransform(gc, x, y, xt, yt) 				\
    	cv_idtransform (gc_get_transform((GRAPHICS_CONTEXT)		\
		(entity_addr(gc))), (x), (y), (xt), (yt))
/*
 * Varieties of transform operators which take matrices as arguments.
 * These all transform fract points to fract points.
 */
#define cs_frmtransform(matrix, x, y, xt, yt)  				\
    	cv_frtransform(cv_load_transform_from_frmatrix(			\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))
#define cs_frmdtransform(matrix, x, y, xt, yt)  			\
    	cv_frdtransform(cv_load_transform_from_frmatrix(		\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))
#define cs_frmitransform(matrix, x, y, xt, yt)  			\
    	cv_fritransform(cv_load_transform_from_frmatrix(		\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))
#define cs_frmidtransform(matrix, x, y, xt, yt)  			\
    	cv_fridtransform(cv_load_transform_from_frmatrix(		\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))
/*
 * These transform float points to float points.
 */
#define cs_flmtransform(matrix, x, y, xt, yt)  				\
    	cv_fltransform (cv_load_transform_from_frmatrix(		\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))
#define cs_flmdtransform(matrix, x, y, xt, yt)  			\
    	cv_fldtransform (cv_load_transform_from_frmatrix(		\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))
#define cs_flmitransform(matrix, x, y, xt, yt)  			\
    	cv_flitransform (cv_load_transform_from_frmatrix(		\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))
#define cs_flmidtransform(matrix, x, y, xt, yt)  			\
    	cv_flidtransform (cv_load_transform_from_frmatrix(		\
		cv_frscratchxform(), (matrix)), (x), (y), (xt), (yt))

#endif /* _cs_transform_h_ */
