/*
 * @(#)cscript.h 23.5 90/06/19 CSCRIPT SMI
 */

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

/*
 * cscript.h - Postscript imaging in C
 */

#ifndef _cscript_h_
#define _cscript_h_

#include <graphics/shapes.h>
#include "cs_common.h"
#include "cs_cursor.h"
#include "cs_canvas.h"
#include "cs_screene.h"
#include "cs_context.h"
#include "cs_transform.h"
#include "cs_cmapst.h"		/* needed for entity initialization 
				 * in PostScript.c */

#define CS_PATSPACE_USER	0
#define CS_PATSPACE_DEVICE	1

/* ----------------------------------------------------------------------
 * public macros
 */

/*
 * these macros are simple shorthand for common type coersions in cscript.
 *
 * the format of the macro name describes deterministically what it does.
 * foo_bar(zippy) == convert the object "zippy" from type "bar" to type "foo".
 *
 * There is some duplication here of gc_get_canvas() and rcanvas_of()
 * functionality, but these seem to make much more sense.
 */

#define gc_ref(ref)	((GRAPHICS_CONTEXT)(entity_addr((ref))))
#define ctx_gc(gc)	((CONTEXT)(gc_get_context((gc))))
#define ctx_ref(ref)	ctx_gc(gc_ref((ref)))
#define path_ctx(ctx)	((PATH)(Get_Context((ctx), CTX_PATH)))
#define path_gc(gc)	path_ctx(ctx_gc((gc)))
#define path_ref(ref)	path_ctx(ctx_ref((ref)))
#define ocv_ref(ref)	((OCANVAS)(entity_addr((ref))))
#define rcv_ref(ref)	((RCANVAS)(entity_addr((ref))))
#define xncv_ref(ref)	((XNCANVAS)(entity_addr((ref))))
#define vis_ref(ref)	((VISUAL)(entity_addr((ref))))
#define psf_ref(ref)	((PSFONT)(entity_addr((ref))))

#define cs_width(cv)		canvas_get_width(ocv_ref(cv))
#define cs_height(cv)		canvas_get_height(ocv_ref(cv))
#define cs_depth(cv)		canvas_get_depth(ocv_ref(cv))

#define cs_xpos(cv)		canvas_get_screenposx(rcv_ref(cv))
#define cs_ypos(cv)		canvas_get_screenposy(rcv_ref(cv))
#define cs_xrelpos(cv)		canvas_get_x(rcv_ref(cv))
#define cs_yrelpos(cv)		canvas_get_y(rcv_ref(cv))

#define cs_parent(cv)		canvas_get_parent(rcv_ref(cv))
#define cs_topchild(cv)		canvas_get_topchild(rcv_ref(cv))
#define cs_canvasabove(cv)	cs_next(cv)
#define cs_next(cv)		canvas_get_nextcanvas(rcv_ref(cv))
#define cs_canvasbelow(cv)	cs_prev(cv)
#define cs_prev(cv)		canvas_get_prevcanvas(rcv_ref(cv))

#define cs_transparent(cv)	canvas_get_transparent(rcv_ref(cv))
#define cs_mapped(cv)		canvas_get_maprequested(rcv_ref(cv))
#define cs_viewable(cv)		cs_ismapped(cv)
#define cs_ismapped(cv)		canvas_get_mapped(rcv_ref(cv))

#define cs_complete(cv)		canvas_get_complete(rcv_ref(cv))
#define cs_retained(cv)		(cv.type == orphan_canvas_type \
			     || canvas_get_retained(rcv_ref(cv)))

#define cs_get_retainedpart(cv)	canvas_get_retainedpart(rcv_ref(cv))
#define cs_get_screenpart(cv)	(cv.type == orphan_canvas_type ? 0 : \
				canvas_get_screenpart(rcv_ref(cv)))

#define cs_winGravity(cv)	canvas_get_winGravity(rcv_ref(cv))
#define cs_bitGravity(cv)	canvas_get_bitGravity(rcv_ref(cv))

#define cs_screeninfo(cv)	canvas_get_screeninfo(rcv_ref(cv)) 

/* Don't use cs_scene() as a LHS. */

#define cs_scene(cv)		(canvas_get_screene(ocv_ref(cv)) ? \
		screene_get_scene(canvas_get_screene(ocv_ref(cv))) : 0)

#define cs_get_scene_scratchgc(cv)	(cs_scene(cv) ? scene_get_scratchgc(cs_scene(cv)) : 0)
#define cs_setsaveback(cv,val)  canvas_set_saveback(rcv_ref(cv),(val))
#define cs_saveback(cv)		canvas_get_saveback(rcv_ref(cv))
#define cs_visinterest(cv)	canvas_get_visinterest(rcv_ref(cv))

#define cs_topcanvas(cv) 	cv_topcanvas(rcv_ref(cv), cv)
#define cv_topcanvas(cv, ref) (						\
	!is_ref_zero((cv)->parent)					\
		? !is_ref_zero(rcv_ref((cv)->parent)->topchild)		\
			? rcv_ref((cv)->parent)->topchild		\
			: (cv)->parent					\
		: !is_ref_zero((cv)->topchild)				\
			? (cv)->topchild				\
			: (ref)						\
)

#define cs_sceneroot(sc)	scene_get_canvases(sc)
#define cs_get_visual(cv)	canvas_get_visual(ocv_ref(cv))
#define cs_get_colormap(cv)	canvas_get_colormap(ocv_ref(cv))


extern void cv_insertcanvasabove();
#define cs_insertcanvasabove(cv1, cv2, x, y)				\
	cv_insertcanvasabove(rcv_ref(cv1), rcv_ref(cv2), x, y)

extern FMATRIX *cv_defaultmatrix();
#define cs_defaultmatrix(cv)						\
	cv_defaultmatrix(ocv_ref(cv))

extern void cv_frusercanvaslocation();
#define cs_frusercanvaslocation(cv, gc, x, y)				\
	cv_frusercanvaslocation(rcv_ref(cv), gc_ref(gc), x, y)

extern void cv_abscanvaslocation();
#define cs_abscanvaslocation(cv, x, y)					\
	cv_abscanvaslocation(rcv_ref(cv), x, y)

extern void cv_init_canvas();
#define cs_init_canvas(cv, parent, w, h, bw, xncan)			\
	cv_init_canvas(rcv_ref(cv), rcv_ref(parent), (w), (h), (bw), (xncan))

#define cs_isglobalroot(cv)						\
	cv_isglobalroot(rcv_ref(cv))
#define cs_isroot(cv)							\
	cv_isroot(ocv_ref(cv))
extern void cv_make_transparent();
#define cs_make_transparent(cv)						\
	cv_make_transparent(rcv_ref(cv))
extern void cv_make_retained();
#define cs_make_retained(cv)						\
	cv_make_retained(rcv_ref(cv))
extern void cv_make_nonretained();
#define cs_make_nonretained(cv)						\
	cv_make_nonretained(rcv_ref(cv))

#define cs_initmatrix(gc)						\
	cv_initmatrix(gc_ref(gc))

extern void cv_clipcanvasshape();
#define cs_clipcanvasshape(gc)						\
	cv_clipcanvasshape(gc_ref(gc))
extern void cv_make_opaque();
#define cs_make_opaque(cv)						\
	cv_make_opaque(rcv_ref(cv))
extern void cs_setparent();
#define cs_remove(cv)							\
	cv_remove(rcv_ref(cv))

extern REF cv_grestore();
#define cs_grestore(gc)							\
	cv_grestore(gc_ref(gc))

extern int cv_pointinshape();
#define cs_pointinshape(shape, x, y)					\
	cv_pointinshape(shape, x, y)

extern void cv_closepath();
#define cs_closepath(gc)						\
	cv_closepath(gc_ref(gc))

#define cs_setwhenmapped(cv)						\
	cv_setwhenmapped(rcv_ref(cv))
#define cv_setwhenmapped(cv) {						\
	if (!(cv)->transparent) canvas_set_whenmapped(cv, 1); }
#define cs_clrwhenmapped(cv)						\
	cv_clrwhenmapped(rcv_ref(cv))
#define cv_clrwhenmapped(cv) {						\
	if (!(cv)->transparent) canvas_set_whenmapped((cv), 0);		\
	if (!(cv)->mapped && (cv)->retained) {				\
	    cv_make_nonretained(cv);					\
	    cv_make_retained(cv);					\
	}								\
}
#define cs_whenmapped(cv)						\
	canvas_get_whenmapped(rcv_ref(cv))

/*
 * Overlay macros - used in cscript, X11/dix, and nucleus...
 */

#define OVERLAYS_ARE_UP		1
#define OVERLAYS_ARE_DOWN	-1
#define OVERLAYS_ARE_NULL	0

extern int cv_overlay_state;

extern void cv_putupoverlays();
#define cs_overlaysup()							\
	cv_overlaysup()
#define cv_overlaysup()							\
	if (cv_overlay_state == OVERLAYS_ARE_DOWN) cv_putupoverlays()

extern void cv_takedownoverlays();
#define cs_overlaysdown()						\
	cv_overlaysdown()
#define cv_overlaysdown() \
	if (cv_overlay_state == OVERLAYS_ARE_UP) cv_takedownoverlays()


extern REF cv_newcontext();
#define cs_newcontext(gcref, cvref)					\
	cv_newcontext((gcref), (cvref))

extern void cv_initcontext();
#define cs_initcontext(gcref, cvref)					\
	cv_initcontext(gc_ref(gcref), (cvref))

extern void cv_initclip();
#define cs_initclip(gc)							\
	cv_initclip(gc_ref(gc))
/*
 * pass canvas ref, not pointer
 */
extern void cv_setcanvas();
#define cs_setcanvas(gcref, canvas, save_path)				\
	cv_setcanvas(gc_ref(gcref), (canvas), (save_path))

extern void cv_currentpoint();
#define cs_currentpoint(gcref, x, y)					\
	cv_currentpoint(gc_ref(gcref), x, y)
extern void cv_flcurrentpoint();
#define cs_flcurrentpoint(gc, x, y)					\
	cv_flcurrentpoint(gc_ref(gc), x, y)
extern void cv_frcurrentpoint();
#define cs_frcurrentpoint(gc, x, y)					\
	cv_frcurrentpoint(gc_ref(gc), x, y)

#define cs_setrgb8color(gc, r, g, b, set_back)				\
	cv_setrgb8color(gc_ref(gc), r, g, b, set_back)

extern void cv_initgraphics();
#define cs_initgraphics(gc)						\
	cv_initgraphics(gc_ref(gc))

extern void renderPath();
#define cs_stroke(gc)							\
	renderPath(gc_ref(gc), CTX_SHADE_HOLLOW, -1)
#define cs_fill(gc)							\
	renderPath(gc_ref(gc), CTX_SHADE_SOLID, CTX_FILL_NONZERO)
#define cs_eofill(gc)							\
	renderPath(gc_ref(gc), CTX_SHADE_SOLID, CTX_FILL_EVENODD)

#define cs_create_cmap(visual, screen, alloc, cmap)			\
	cv_create_cmap(vis_ref(visual), screen, alloc, cmap)

#define cs_delete_overlay_from_active_list(cv)				\
	cv_delete_overlay_from_active_list(rcv_ref(cv))

#define cs_getcolormap(cv)						\
	((rcv_ref(cv))->colormap)
#define cs_setcolormap(cv, cmap)					\
	((rcv_ref(cv))->colormap = cmap)
#define cs_resolve_color_gc(pr, pg, pb, visual)				\
	cv_resolve_color_gc(pr, pg, pb, vis_ref(visual))

extern REF cv_concretefont();
#define cs_concretefont(psfont, gc)					\
	cv_concretefont((PSFONT)(entity_addr(psfont)), gc_ref(gc))

extern REF cv_frscalefont();
#define cs_frscalefont(psfont, scale)					\
	cv_frscalefont((PSFONT)(entity_addr(psfont)), scale)

extern void cv_setfont();
#define cs_setfont(gc, psfont)						\
	cv_setfont(gc_ref(gc), psfont)

extern char *cv_fontpath();
#define cs_fontpath()							\
	cv_fontpath()

extern REF cv_create_canvas_from_raster();
#define cs_create_canvas_from_raster(raster)				\
	cv_create_canvas_from_raster(raster)

extern void cv_set_memory_colormap();
#define cs_set_memory_colormap(raster)					\
	cv_set_memory_colormap(raster)

extern void cv_cleargraphicsdamagearea();
#define cs_cleargraphicsdamagearea(cv)					\
	cv_cleargraphicsdamagearea(ocv_ref(cv))

#define cs_get_transform(gc)						\
	gc_get_transform(gc_ref(gc))

extern void cv_cursorin();
#define cs_cursorin()							\
	cv_cursorin()

extern void cv_cursorout();
#define cs_cursorout()							\
	cv_cursorout()

extern void cv_cursorsmartout();
#define cs_cursorsmartout(raster)					\
	cv_cursorsmartout(raster)

extern void cv_cursorbboxout();
#define cs_cursorbboxout(raster, org, dim)				\
	cv_cursorbboxout(raster, org, dim)

extern void cv_setcurfgpixel();
#define cs_setcurfgpixel(gc, value)					\
	cv_setcurfgpixel(gc_ref(gc), (value))

extern void cv_setcurbgpixel();
#define cs_setcurbgpixel(gc, value)					\
	cv_setcurbgpixel(gc_ref(gc), (value))

#define cs_current_context(gc)						\
	gc_get_context(gc_ref(gc))

#define cs_currentfont(gc)						\
	gc_get_font(gc_ref(gc))

#define cs_currenttransfers(gc)						\
	gc_get_transfers(gc_ref(gc))

#define cs_currenthalftone(gc)						\
	(gc_ref(gc))->halftone

/* for VisibilityNotify events */
enum visibility {
    unobscured,
    partially_obscured,
    totally_obscured,
};

#define cs_set_winGravity(cv, dir) \
	canvas_set_winGravity(xncv_ref(cv), dir)
#define cs_set_bitGravity(cv, dir) \
	canvas_set_bitGravity(xncv_ref(cv), dir)

/* public macros for accessing pixmap formats */
#define cs_pxmapfmt_depth(fmt)		pxmapfmt_get_depth(fmt)
#define cs_pxmapfmt_bitsPerPixel(fmt)	pxmapfmt_get_bitsPerPixel(fmt)
#define cs_pxmapfmt_scanlinePad(fmt)	pxmapfmt_get_scanlinePad(fmt)

/* public macros for accessing visuals */
#define cs_set_visual(cv, visualref)					\
	canvas_set_visual(ocv_ref(cv), (visualref))
#define cs_visual_vid(visual)						\
	visual_get_vid(vis_ref(visual))
#define cs_visual_screen(visual)					\
	visual_get_screen(vis_ref(visual))
#define cs_visual_class(visual)						\
	visual_get_class(vis_ref(visual))
#define cs_visual_redMask(visual)					\
	visual_get_redMask(vis_ref(visual))
#define cs_visual_greenMask(visual)					\
	visual_get_greenMask(vis_ref(visual))
#define cs_visual_blueMask(visual)					\
	visual_get_blueMask(vis_ref(visual))
#define cs_visual_offsetRed(visual)					\
	visual_get_offsetRed(vis_ref(visual))
#define cs_visual_offsetGreen(visual)					\
	visual_get_offsetGreen(vis_ref(visual))
#define cs_visual_offsetBlue(visual)					\
	visual_get_offsetBlue(vis_ref(visual))
#define cs_visual_bitsPerRGBValue(visual)  				\
	visual_get_bitsPerRGBValue(vis_ref(visual))
#define cs_visual_ColormapEntries(visual)  				\
	visual_get_ColormapEntries(vis_ref(visual))
#define cs_visual_nplanes(visual)	   				\
	visual_get_nplanes(vis_ref(visual))

/* public macros for accessing depths */
#define cs_depth_depth(depth)		depth_get_depth(depth)
#define cs_depth_numVids(depth)		depth_get_numVids(depth)
#define cs_depth_vids(depth)		depth_get_vids(depth)

/* public macros for accessing the screen */
#define cs_scrn_myNum(scrn)		screene_get_myNum(scrn)
#define cs_scrn_width(scrn)		screene_get_width(scrn)
#define cs_scrn_height(scrn)		screene_get_height(scrn)
#define cs_scrn_mmWidth(scrn)		screene_get_mmWidth(scrn)
#define cs_scrn_mmHeight(scrn)		screene_get_mmHeight(scrn)
#define cs_scrn_scene(scrn)		screene_get_scene(scrn)
#define cs_scrn_numDepths(scrn)		screene_get_numDepths(scrn)
#define cs_scrn_rootDepth(scrn)		screene_get_rootDepth(scrn)
#define cs_scrn_allowedDepths(scrn)	screene_get_allowedDepths(scrn)
#define cs_scrn_numVisuals(scrn)	screene_get_numVisuals(scrn)
#define cs_scrn_visuals(scrn)		screene_get_visuals(scrn)
#define cs_scrn_pRootVisual(scrn)	screene_get_pRootVisual(scrn)
#define cs_scrn_minInstalledCmaps(scrn)	screene_get_minInstalledCmaps(scrn)
#define cs_scrn_maxInstalledCmaps(scrn)	screene_get_maxInstalledCmaps(scrn)
#define cs_scrn_defaultmap(scrn)	screene_get_defaultmap(scrn)
#define cs_scrn_staticmap(scrn)		screene_get_staticmap(scrn)
#define cs_scrn_installedmap(scrn)	screene_get_installedmap(scrn)
#define cs_scrn_whitePixel(scrn)	screene_get_whitePixel(scrn)
#define cs_scrn_blackPixel(scrn)	screene_get_blackPixel(scrn)

/* public macros for accessing screeninfo */
#define cs_scrninfo_imageByteOrder(scrninfo)				\
	scrninfo_get_imageByteOrder(scrninfo)
#define cs_scrninfo_bitmapScanlineUnit(scrninfo)			\
	scrninfo_get_bitmapScanlineUnit(scrninfo)
#define cs_scrninfo_bitmapScanlinePad(scrninfo)				\
	scrninfo_get_bitmapScanlinePad(scrninfo)
#define cs_scrninfo_bitmapBitOrder(scrninfo)				\
	scrninfo_get_bitmapBitOrder(scrninfo)
#define cs_scrninfo_numPixmapFormats(scrninfo)				\
	scrninfo_get_numPixmapFormats(scrninfo)
#define cs_scrninfo_formats(scrninfo)					\
	scrninfo_get_formats(scrninfo)
#define cs_scrninfo_numScreenes(scrninfo)				\
	scrninfo_get_numScreenes(scrninfo)


/* ----------------------------------------------------------------------
 * public function definitions
 */

/* context operators */

#ifndef cs_currentcanvasshape
SHAPE cs_currentcanvasshape();
#endif
#ifndef cs_currentpath
SHAPE cs_currentpath();
#endif
#ifndef cs_eocurrentpath
SHAPE cs_eocurrentpath();
#endif
#ifndef cs_currentfillrule
CTX_area_style cs_currentfillrule();
#endif
#ifndef cs_currentplanemask
unsigned long cs_currentplanemask();
#endif
#ifndef cs_setfillstyle
void cs_setfillstyle();
#endif
#define cs_newpath(gc)							\
	cv_newpath(gc_ref(gc))

/* gsave, grestore */

#ifndef cs_gsave
REF cs_gsave();
#endif
#ifndef cs_grestoreall
REF cs_grestoreall();
#endif

/* color */

#ifndef cs_currentcolor
struct packedcolor cs_currentcolor();
#endif
#ifndef cs_frrgbcolor
struct packedcolor cs_frrgbcolor();
#endif
#ifndef cs_frhsbcolor
struct packedcolor cs_frhsbcolor();
#endif

/* font operators */

#ifndef NEWS_ONLY
#ifndef cs_findfont
REF cs_findfont();
#endif
#endif
#ifndef cs_makefont
REF cs_makefont();
#endif
#ifndef cs_frmakefont
REF cs_frmakefont();
#endif

/* graphics operators */

#ifndef cs_erasepage
void cs_erasepage();
#endif
#ifndef cs_fill
void cs_fill();
#endif
#ifndef cs_rmoveto
void cs_rmoveto();
#endif
#ifndef cs_rlineto
void cs_rlineto();
#endif
#ifndef cs_setcolor
void cs_setcolor();
#endif
#ifndef cs_imagecanvas
void cs_imagecanvas();
#endif

/* transform operators */

#ifndef cs_screentocanvas
void cs_screentocanvas();
#endif

/* canvas operators */

#ifndef cs_create_screen_canvas
REF cs_create_screen_canvas();
#endif
#ifndef cs_create_orphan_canvas
REF cs_create_orphan_canvas();
#endif
#ifndef cs_bottomcanvas
REF cs_bottomcanvas();
#endif
#ifndef cs_locatecanvasinglobalroot
REF cs_locatecanvasinglobalroot();
#endif
#ifndef cs_locatecanvasinscene
REF cs_locatecanvasinscene();
#endif
#ifndef cs_nextcanvasforpoint
REF cs_nextcanvasforpoint();
#endif
#ifndef NEWS_ONLY
#ifndef cs_read_from_canvas
extern void cs_read_from_canvas();
#endif
#endif

/* damage operators */

#ifndef cs_firstdamaged
REF cs_firstdamaged();
#endif
#ifndef cs_getdamagearea
SHAPE cs_getdamagearea();
#endif
#ifndef NEWS_ONLY
#ifndef cs_getgraphicsdamagearea
SHAPE cs_getgraphicsdamagearea();
#endif
#ifndef cs_outerpart
SHAPE cs_outerpart();
#endif
#ifndef cs_innerpart
SHAPE cs_innerpart();
#endif
#endif
#ifndef cs_addtodamqueue
void cs_addtodamqueue();
#endif
#ifndef cs_cleardamagearea
void cs_cleardamagearea();
#endif
#ifndef cs_hasdamagearea
int cs_hasdamagearea();
#endif
extern void cv_extenddamage();
#define cs_extenddamage(gc, all)					\
	cv_extenddamage(gc, all, CTX_FILL_NONZERO)
#define cs_eoextenddamage(gc, all)					\
	cv_extenddamage(gc, all, CTX_FILL_EVENODD)

/* misc */

#ifndef cs_writeimage
int cs_writeimage();
#endif
#ifndef cs_readimage
REF cs_readimage();
#endif

/* screene functions */

#ifndef cs_initoutput
void cs_initoutput();
#endif
#ifndef cs_initdepth1
void cs_initdepth1();
#endif

#ifndef cs_clip
void cs_clip();
#endif
#ifndef cs_clippath
void cs_clippath();
#endif
#ifndef cs_closepath
void cs_closepath();
#endif
#ifndef cs_contrastswithcurrent
int cs_contrastswithcurrent();
#endif
#ifndef cs_copy_context
void cs_copy_context();
#endif
#ifndef cs_copy_context_into
void cs_copy_context_into();
#endif
#ifndef cs_create_canvas_from_image
void cs_create_canvas_from_image();
#endif
#ifndef cs_create_copy_of_context
REF cs_create_copy_of_context();
#endif
#ifndef cs_cstringbbox
void cs_cstringbbox();
#endif
#ifndef cs_currentflat
fract cs_currentflat();
#endif
#ifndef cs_currentmatrix
void cs_currentmatrix();
#endif
#ifndef cs_currentrasteropcode
CTX_rop cs_currentrasteropcode();
#endif
#ifndef cs_dashpath
void cs_dashpath();
#endif
#ifndef cs_definefont
REF cs_definefont();
#endif
#ifndef cs_eoclip
void cs_eoclip();
#endif
#ifndef cs_eoclipcanvas
void cs_eoclipcanvas();
#endif
#ifndef cs_eofill
void cs_eofill();
#endif
#ifndef cs_eofrshiftarea
int cs_eofrshiftarea();
#endif
#ifndef cs_findfilefont
REF cs_findfilefont();
#endif
#ifndef cs_flattenpath
void cs_flattenpath();
#endif
#ifndef cs_flcurveto
void cs_flcurveto();
#endif
#ifndef cs_fllineby
void cs_fllineby();
#endif
#ifndef cs_flrcurveto
void cs_flrcurveto();
#endif
#ifndef cs_flrlineby
void cs_flrlineby();
#endif
#ifndef cs_flrmoveto
void cs_flrmoveto();
#endif
#ifndef cs_frarc
void cs_frarc();
#endif
#ifndef cs_frarcn
void cs_frarcn();
#endif
#ifndef cs_flarc
void cs_flarc();
#endif
#ifndef cs_flarcn
void cs_flarcn();
#endif
#ifndef cs_flarcto
void cs_flarcto();
#endif
#ifndef cs_frarcto
void cs_frarcto();
#endif
#ifndef cs_frcashow
void cs_frcashow();
#endif
#ifndef cs_frcawidthshow
void cs_frcawidthshow();
#endif
#ifndef cs_frcstringwidth
void cs_frcstringwidth();
#endif
#ifndef cs_frcurrentgray
fract cs_frcurrentgray();
#endif
#ifndef cs_frcurrenthsbcolor
void cs_frcurrenthsbcolor();
#endif
#ifndef cs_frcurrentlinewidth
fract cs_frcurrentlinewidth();
#endif
#ifndef cs_frcurrentrgbcolor
void cs_frcurrentrgbcolor();
#endif
#ifndef cs_frcurveto
void cs_frcurveto();
#endif
#ifndef cs_frcwidthshow
void cs_frcwidthshow();
#endif
#ifndef cs_frlineby
void cs_frlineby();
#endif
#ifndef cs_frlineto
void cs_frlineto();
#endif
#ifndef cs_flrlineto
void cs_flrlineto();
#endif
#ifndef cs_frrcurveto
void cs_frrcurveto();
#endif
#ifndef cs_frrlineby
void cs_frrlineby();
#endif
#ifndef cs_frrlineto
void cs_frrlineto();
#endif
#ifndef cs_frrmoveto
void cs_frrmoveto();
#endif
#ifndef cs_frsetdash
void cs_frsetdash();
#endif
#ifndef cs_frsetgray
void cs_frsetgray();
#endif
#ifndef cs_frsethsbcolor
void cs_frsethsbcolor();
#endif
#ifndef cs_frsetlinewidth
void cs_frsetlinewidth();
#endif
#ifndef cs_frsetrgbcolor
void cs_frsetrgbcolor();
#endif
#ifndef cs_frshiftarea
int cs_frshiftarea();
#endif
#ifndef cs_frtransform
void cs_frtransform();
#endif
#ifndef cs_frtranslate
void cs_frtranslate();
#endif
#ifndef cs_get_psf_bbox
void cs_get_psf_bbox();
#endif
#ifndef cs_initEvents
void cs_initEvents();
#endif
#ifndef cs_kill_context
void cs_kill_context();
#endif
#ifndef cs_make_nonretained
void cs_make_nonretained();
#endif
#ifndef cs_masksetcanvasshape
void cs_masksetcanvasshape();
#endif
#ifndef cs_pathbbox
void cs_pathbbox();
#endif
#ifndef cs_remove
void cs_remove();
#endif
#ifndef cs_setcanvasshape
void cs_setcanvasshape();
#endif
#ifndef cs_setdash
void cs_setdash();
#endif
#ifndef cs_setlinecap
void cs_setlinecap();
#endif
#ifndef cs_setlinejoin
void cs_setlinejoin();
#endif
#ifndef cs_setmatrix
void cs_setmatrix();
#endif
#ifndef cs_setpath
void cs_setpath();
#endif
#ifndef cs_setrasteropcode
void cs_setrasteropcode();
#endif
#ifndef cv_unmap
void cv_unmap();
#endif
#ifndef cs_currentdash
void cs_currentdash();
#endif
#ifndef cs_canvastoscreen
void cs_canvastoscreen();
#endif
void cs_setvisinterest();
void cs_clrvisinterest();

/* text functions */

#ifndef NEWS_ONLY
void cs_ctextextent();
void cv_text_xshow();

#define cs_xcshow(gc, string, length, pt, charbits)			\
	(cv_text_xshow(gc_ref(gc), string,	\
		       length, pt, charbits))


void cv_text_ximageshow();
#define cs_xcimageshow(gc, string, length, pt, charbits)		\
	(cv_text_ximageshow(gc_ref(gc), string, \
			    length, pt, charbits))

#endif

void cv_text_show();
#define cs_cshow(gc, string, length)					\
	(cv_text_show(gc_ref(gc), string,	\
		      length, (struct text_modifiers *)NULL))
	

#define cs_cstringbbox(gc, string, length, min_pt, max_pt)		\
	(cv_text_stringbbox(gc_ref(gc),					\
			    (string), (length), (min_pt), (max_pt)))

#endif /* _cscript_h_ */
