/*
 * @(#)gdefs.h 2.1 90/01/18
 *
 */
/*
 *************************************************************
 *                                                           *
 *   Copyright (c) 1989, 1987 by Sun Microsystems, Inc.      *
 *                                                           *
 *   This program is protected by Federal Copyright Law.     *
 *   No part of this program may be reproduced, stored in    *
 *   a retrieval system, translated, transcribed, or         *
 *   transmitted, in any form or by any means manual,        *
 *   electric, electronic, electro-magnetic, mechanical,     *
 *   optical, or otherwise, without prior explicit written   *
 *   permission from Sun Microsystems, Inc.                  *
 *                                                           *
 *************************************************************
 */
#include <math.h>
#include	"frmath.h"


/*---------------------------------------------------------------
    	MATH
---------------------------------------------------------------*/

/*
    These macros are collected here for portability reasons
    and should be examined very carefully when moving the
    code to a different machine. The math functions of the
    raw machine should NOT be used, only the macros below.
*/


#define	math_Floor(x)    ((int32)floor(x))

#define	math_Ceiling(x)  ((int32)ceil(x))


/* translation of FLOATs to integers */

/*	The following round function should be used instead of the
 *	<rint> provided in the math library, which only works if your program
 *	is compiled with the -f68881 option.  And furthermore, there are
 *	inconsistencies in the result.		may 5/11/88
 *      Returns an integer
 */
#define math_iRound(x)        (((x) > 0) ? math_Floor((x)+0.5) : math_Ceiling((x)-0.5))

/* same as above, returns a FLOAT */
#define math_dRound(x)        (((x) > 0) ? floor((x)+0.5) : ceil((x)-0.5))


/* computing absolute values */

#define math_Abs(x)     (((x)>=0)?(x):(-(x)))

#define math_FloatAbs(x) (((x)>=0.0)?(x):(-(x)))

#define math_DoubleAbs(x) (((x)>=0.0)?(x):(-(x)))


/* trigonometric stuff */

#define math_Sin(x)     (sin(x))

#define math_Cos(x)     (cos(x))

#define math_Tan(x)     (tan(x))

#define math_Asin(x)    (asin(x))

#define math_Acos(x)    (acos(x))

#define math_Atan(x)    (atan(x))

#define math_Atan2(x)   (atan2(x))


/* miscellaneous others */

#define math_Sqrt(x)    (sqrt(x))

#define math_Square(x)   ((x)*(x))

/*---------------------------------------------------------------
    	pair XY: REPRESENTATION OF POINTS
---------------------------------------------------------------*/

/* In source coordinates */

typedef struct {
    FLOAT   x,y;
} pair_dXY;


typedef struct {
    FLOAT   x,y;
} pair_fXY;


/* The same in grid coordinates */

typedef struct {
    int32   x,y;
} pair_iXY;

/* The same in 16/16 fixed point coordinates */

typedef struct {
    fract   x,y;
} pair_frXY;

/*---------------------------------------------------------------
    	BOUNDING BOXES
---------------------------------------------------------------*/

/* In source coordinates */

typedef struct {
    FLOAT   lox,loy,hix,hiy;
} bbox_dBBox;

typedef struct {
    FLOAT   lox,loy,hix,hiy;
} bbox_fBBox;


/* The same for grid coordinates */

typedef struct {
    int32   lox,loy,hix,hiy;
} bbox_iBBox;

/* The same for 16/16 fixed point coordinates */

typedef struct {
    fract   lox,loy,hix,hiy;
} bbox_frBBox;

/*---------------------------------------------------------------
    	TRANSFORMATIONS
---------------------------------------------------------------*/

/*
    The transformation: a 2D transformation in homogeneous coordinates
*/

/* in FLOAT */

typedef	struct {
    FLOAT    a,b,c,d,dx,dy;
} trans_dTrans;

/* in fract */

typedef	struct {
    fract    a,b,c,d,dx,dy;
} trans_frTrans;
