/****
 *
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 * @(#)sh_SHAPE.h 23.3 90/06/19 Copyright 1988 Sun Micro
 *
 * SHAPE.H - SHAPE Object Information
 * The user does not actually know what the SHAPE object looks like.
 * It can only be accessed via the Xxx_Shape functions.
 *
 ****/
#ifndef	_SH_SHAPE
/*
 * Inside_Shape intersection codes
 */
#define SHAPE_OUTSIDE   (-1)
#define SHAPE_INSIDE    (0)
#define SHAPE_TOUCH     (1)
#define SHAPE_INITIAL   (2)

/*
 * Display_Shape/Rectangles options
 */
#define	SHAPE_DEVICE	1
#define	SHAPE_NOCLIP	2
#define	SHAPE_REPEAT	4
/* SHAPE_RECT_CLIP	8 */
#define	SHAPE_RECTLIST	16

#define	X_EOL	(-32767)	/* end of X list */
#define	Y_EOL	(-32768)	/* end of Y list */

/*
 * struct class_SHAPE - internal representation of SHAPE object
 */
struct class_SHAPE
  {
#include "sh_SHAPE.ah"
  };

#define	SHAPE_OP_Destroy	0
#define	SHAPE_OP_Init		1
#define	SHAPE_OP_Get		2
#define	SHAPE_OP_Set		3
#define	SHAPE_OP_Print		4
#define	SHAPE_OP_Close		5
#define	SHAPE_OP_Union		6
#define	SHAPE_OP_Differ		7
#define	SHAPE_OP_Intersect	8
#define	SHAPE_OP_Copy		9
#define	SHAPE_OP_Translate	10
#define	SHAPE_OP_Position	11
#define	SHAPE_OP_Inside		12
#define	SHAPE_OP_Read		13
#define	SHAPE_OP_Overlap	14
#define	SHAPE_OP_PointIn	15
#define	SHAPE_OP_Compress	16
#define	SHAPE_OP_Size		17
#define SHAPE_OP_Path		18
#define SHAPE_OP_IsRectEqual    19
#define SHAPE_OP_ellipse	20
#define SHAPE_OP_fellipse	21
#define	SHAPE_OPERS		22

/*
 * SHAPE operators.
 *
 * SHAPE Create_Shape(size)	create SHAPE object
 * Destroy_Shape(shape)		destroy shape
 * Print_Shape(shape)		print shape
 * Close_Shape(shape, ptr) 	close shape
 * Intersect_Shape(s1, s2, s3)	intersect shape
 * Union_Shape(s1, s2, s3)	union shape
 * Differ_Shape(s1, s2, s3)	difference shape
 * Overlap_Shape(s1, s2)	determine if shapes overlap
 * Translate_Shape(shape, x, y)	translate shape
 * Position_Shape(shape, x, y)	position shape
 * Box_Shape(s1, x, y, dx, dy)	make box shape
 * Read_Shape(shape, func, arg)	read rectangle of shape
 * PointIn_Shape(shape, x, y)	determine if point inside shape
 * Compress_Shape(shape)	compress shape
 * Size_Shape(shape, n)		resize shape
 * Path_Shape(shape, path, f)   make shape out of the path
 * IsRectEqual_Shape(shape, srect) test if shape is equivalent to srect
 * Ellipse_Shape                load shape with ellipse boundary description
 * Fellipse_Shape               load shape with filled ellipse description
 * SHAPE Write_Shape(rects,n,h)	create shape from list of rects
 *
 */
#define	Destroy_Shape(shape)	Destroy_Obj(shape)
#define	Print_Shape(s)		(*GetOper_Obj(s, SHAPE_OP_Print))(s)
#define	Close_Shape(s,p)	((SHAPE) (*GetOper_Obj(s, SHAPE_OP_Close))(s,p))
#define	Union_Shape(s1,s2,sd) \
	((SHAPE) (*GetOper_Obj(s1, SHAPE_OP_Union))(s1,s2,sd))
#define	Differ_Shape(s1,s2,sd) \
	((SHAPE) (*GetOper_Obj(s1, SHAPE_OP_Differ))(s1,s2,sd))
#define	Intersect_Shape(s1,s2, dst) \
	((SHAPE) (*GetOper_Obj(s1, SHAPE_OP_Intersect))(s1,s2,dst))
#define	Copy_Shape(s,d,x,y) \
	((SHAPE) (*GetOper_Obj(s, SHAPE_OP_Copy))(s,d,x,y))
#define	Position_Shape(s,x,y) \
	((SHAPE) (*GetOper_Obj(s, SHAPE_OP_Position))(s,x,y))
#define	Translate_Shape(s,x,y) \
	((SHAPE) (*GetOper_Obj(s, SHAPE_OP_Translate))(s,x,y))
#define Path_Shape(s, p, f) \
	((SHAPE)(*GetOper_Obj(s, SHAPE_OP_Path))(s, p, f))
#define	Size_Shape(s,n)	 \
	((SHAPE)(*GetOper_Obj(s, SHAPE_OP_Size))(s,n))
#define	Display_Shape(s,c,o) \
	((*GetOper_Obj(((CONTEXT) c)->CTX_RASTER, RAS_OP_Shape)) \
		(((CONTEXT) c)->CTX_RASTER, s, c, o))
#define	Display_Rectangles(s,c,o,n,xo,yo) \
	((*GetOper_Obj(((CONTEXT) c)->CTX_RASTER, RAS_OP_Shape)) \
		(((CONTEXT) c)->CTX_RASTER, s, c, o, n, xo, yo))
#define	Inside_Shape(s,o,d)	(*GetOper_Obj(s, SHAPE_OP_Inside))(s,o,d)
#define	Read_Shape(s,f,a)	(*GetOper_Obj(s, SHAPE_OP_Read))(s,f,a)
#define	Overlap_Shape(a,b) 	(*GetOper_Obj(a, SHAPE_OP_Overlap))(a,b)
#define	PointIn_Shape(s,x,y) ((int)((*GetOper_Obj(s, SHAPE_OP_PointIn))(s,x,y)))
#define	Compress_Shape(s) 	(*GetOper_Obj(s, SHAPE_OP_Compress))(s)
#define IsRectEqual_Shape(s,r)  (*GetOper_Obj(s, SHAPE_OP_IsRectEqual))(s,r)
#define Ellipse_Shape(s,x,y,dx,dy)\
        (*GetOper_Obj(s, SHAPE_OP_ellipse))(s,x,y,dx,dy)
#define Fellipse_Shape(s,x,y,dx,dy)\
        (*GetOper_Obj(s, SHAPE_OP_fellipse))(s,x,y,dx,dy)


/* UnionN_Shape(slist, n, sout) union a list of shapes into sout */
#define UnionN_Shape		sh_UnionN_Shape
/* (SHAPE) Write_Shape(rlist, n, hint) union list of rects into a new shape */
#define Write_Shape		sh_Write_Shape

#define Box_Shape               sh_Box_Shape

#define Static_Shape(s, t, e) {			\
	Class_Obj(s, CLASS_SHAPE);		\
	Set_Obj(s, OBJ_FLAGS, OBJ_NOKILLHDR);	\
        Init_Obj(s, t, e ); }

#define Static_InitShape(s) {			\
	Class_Obj(s, CLASS_SHAPE);		\
	Set_Obj(s, OBJ_FLAGS, OBJ_NOKILLHDR);	\
	s->shape_flags = 0;			\
	s->shape_xeol = X_EOL;			\
	s->SHAPE_SIZE = 0;			\
	s->SHAPE_YMIN = Y_EOL;			\
	s->shape_yeol = Y_EOL;  }

#define Static_InitBoxShape(s, x, y, w, h) {	\
	Class_Obj(s, CLASS_SHAPE);		\
	Set_Obj(s, OBJ_FLAGS, OBJ_NOKILLHDR);	\
	s->shape_flags = 0;			\
	s->shape_xeol = X_EOL;			\
	s->SHAPE_SIZE = SHAPE_RECT_SIZE;	\
	s->SHAPE_YMIN = y;			\
	s->SHAPE_YMAX = y + h;			\
	s->SHAPE_XMIN = x;			\
	s->SHAPE_XMAX = x + w;			\
	s->shape_yeol = Y_EOL; }


/* 
 * Shape Type.
 * 	SHAPE_GENERIC -- creates a new shape that describe the path.
 *	SHAPE_FIXSIZE -- Create a shape with specified size.
 *
 */

#define SHAPE_GENERIC	0
#define SHAPE_FIXSIZE	1

/*
 * Get_Shape(shape, attr)
 * Fast way to access shape attributes.
 * This macro depends on the "attr" argument being a compile time constant.
 */
#ifdef	Get_Shape
#undef	Get_Shape
#endif	/* Get_Shape */

#define	Get_Shape(s,a)	CAT(a,get)((SHAPE) (s))
#define	Set_Shape(s,a,v) CAT(a,set)((SHAPE) (s), s)

#define SHAPE_XMINget(s)	((s)->SHAPE_XMIN)
#define SHAPE_XMAXget(s)	((s)->SHAPE_XMAX)
#define SHAPE_YMAXget(s)	((s)->SHAPE_YMAX)
#define SHAPE_YMINget(s)	((s)->SHAPE_YMIN)
#define SHAPE_RECTget(s)	((s)->SHAPE_SIZE == SHAPE_RECT_SIZE)
#define SHAPE_EMPTYget(s)	((s)->SHAPE_SIZE == 0)
#define SHAPE_SIZEget(s) (((s)->SHAPE_SIZE == 0xff) ? \
	Get_Obj((s)->shape_obj, OBJ_SIZE) : (s)->SHAPE_SIZE)

#define SHAPE_XMINset(s,v)	((s)->SHAPE_XMIN = v)
#define SHAPE_XMAXset(s,v)	((s)->SHAPE_XMAX = v)
#define SHAPE_YMAXset(s,v)	((s)->SHAPE_YMAX = v)
#define SHAPE_YMINset(s,v)	((s)->SHAPE_YMIN = v)
#define SHAPE_SIZEset(s,v)	(*GetOper_Obj(s, SHAPE_OP_Size))(s,v)


#define	Bound_Shape(s, x1, y1, x2, y2) (	\
		((SHAPE) s)->SHAPE_XMIN = x1,	\
		((SHAPE) s)->SHAPE_YMIN = y1,	\
		((SHAPE) s)->SHAPE_XMAX = x2,	\
		((SHAPE) s)->SHAPE_YMAX = y2)



#define	Rect_Shape(s, x1, y1, x2, y2) \
	((((SHAPE) s)->SHAPE_SIZE != SHAPE_RECT_SIZE) ?	\
		Box_Shape(s, x1, y1, x2, y2) :	\
		(((SHAPE) s)->SHAPE_XMIN = x1,		\
		 ((SHAPE) s)->SHAPE_YMIN = y1,		\
		 ((SHAPE) s)->SHAPE_XMAX = x2,		\
		 ((SHAPE) s)->SHAPE_YMAX = y2))


#define BBox_Shape(s) do {			\
	Destroy_Obj((s)->shape_obj);		\
	(s)->SHAPE_SIZE = SHAPE_RECT_SIZE;	\
	(s)->shape_yeol = Y_EOL;		\
} while (0)



#define Destroy_Shape_Data(s) \
		 do { if (ShapeHasData(s)) \
			{ Destroy_Obj((s)->shape_obj); \
			  (s)->SHAPE_YMIN = Y_EOL; \
			  (s)->SHAPE_SIZE=0; } \
		    } while (0)

	  
#define	Temp_Shape(t,e)	((SHAPE) Temp_Obj(CLASS_SHAPE, t, e))
#define	Create_Shape(t,e) ((SHAPE) Create_Obj(CLASS_SHAPE, t, e))

extern  SHAPE   sh_Box_Shape();
extern  int     sh_UnionN_Shape();
extern  SHAPE	sh_Write_Shape();
#define	Empty_Shape()		Create_Shape(SHAPE_FIXSIZE, NULL)


#define	_SH_SHAPE
#endif	/* _SH_SHAPE */
