/****
 *
 * Copyright (c) 1989 by Sun Microsystems, Inc.
 * @(#)sh_cmap_grab.h 23.6 90/06/19 Copyright 1989 Sun Micro
 *
 * Colormap grabber header file.
 *
 *
 *
 * Macros defined here:
 *
 *  sh_cm_lock(infop)
 *  sh_cm_unlock(infop)
 *	lock and unlock colormap info page.  'infop' is the Grabbedcmap
 *	pointer returned by cmapgrab_grab()
 *
 *  sh_cmapgrab_cookie(infop)
 *	returns the colormap info page cookie used to map the info page.
 *	this is the value that should be returned to the requesting
 *	client.
 *
 ****/

#ifndef	_CMAP_GRAB
#define	_CMAP_GRAB

#include <sys/types.h>


/* Tell client how to load colormaps */

typedef	enum {
	  SUNWINDOWS_IOCTL=0,	/* use /dev/winxx SunWindows device driver */
	  HW_DEVICE_IOCTL=1,	/* use graphics hardware device driver */
	  HW_DEVICE_DIRECT=2,	/* go straight to HW */
	} cmap_load_method ;


#define	CMMAGIC	0x434d4150	/* CMAP */
#define	CMVERS	0

#define	MAX_CM_DEVNAME	40	/* better not have long device names */

#ifndef	_SH_MAIN
typedef	caddr_t	RASTER ;
#endif

/* info in shared memory */

typedef	struct _grabbedcmap {
    /* version 0 info starts here */
	long	cm_magic ;	/* "CMAP" = 0x434d4150 ("PAMC" on 386i) */
	long	cm_version ;	/* 0 */
	int	cm_count ;	/* change count */
	struct _grabbedcmap *cm_ptr ;	/* self-reference */
	u_long	cm_cmap ;	/* X11 colormap ID */

	u_long	cm_cookie ;	/* lock-page cookie */

	/* server-specific info, not meaningful to client */
	int	cm_sinfofd ;	/* fd of this file */
	int	cm_sdevfd ;	/* fd of framebuffer */
	u_long	*cm_slockp ;	/* pointer to lock page */
	u_long	*cm_sunlockp ;	/* pointer to unlock page */
	int	cm_slockcnt ;	/* server lock count, for nesting */
	int	cm_clientcnt ;	/* number of clients grabbing this cmap */
	int	cm_filesuffix ;	/* internal bookkeeping */
	RASTER	cm_shras ;	/* internal raster handle */

	char	cm_devname[MAX_CM_DEVNAME] ;		/* fb */
	char	cm_lockdevname[MAX_CM_DEVNAME] ;	/* lockpages */
	int	cm_default ;	/* is it the default cmap for screen */
	int	cm_installed ;	/* flag: colormap is installed in HW */
	cmap_load_method cm_load_method ;

	/* server's /dev/winxx used to set colormaps, default colormap
	   name ("xnews_default") */
	char	cm_sunwindows_def_devname[MAX_CM_DEVNAME] ;
	char	cm_sunwindows_def_cmapname[MAX_CM_DEVNAME] ;
	int	cm_sunwindows_def_cmapsize ;

	/* server's /dev/winxx used to set colormaps, installed colormap
	   name ("xnews_installed") */
	char	cm_sunwindows_inst_devname[MAX_CM_DEVNAME] ;
	char	cm_sunwindows_inst_cmapname[MAX_CM_DEVNAME] ;
	int	cm_sunwindows_inst_cmapsize ;
	

	int	cm_fbtype ;	/* reserved */
	int	cm_reallen ;	/* colormap hardware size */
	int	cm_start ;	/* first valid location */
	int	cm_len ;	/* number of valid locations */
	int	cm_hwloc[8] ;	/* reserved */

	long	cm_shadowofs ;	/* byte offset relative to cm_magic of
				   the start of the shadow colormap */
} Grabbedcmap ;


	/* followed by three arrays of u_shorts, length cm_len, for
	   red, green and blue and one array of bytes which are
	   the change flags for their respective colors:
	   
		rr
		rr
		rr
		:
		gg
		gg
		gg
		:
		bb
		bb
		bb
		:
		f
		f
		f
		:
		
	    Clients set a flag to non-zero to indicate that the color
	    has been changed.  The server clears these when the changes
	    have been seen.
	  */



/* Client-side info that can't be stored in shared memory because
   there may be more than one client. */


typedef	struct cmap_grab {
	Grabbedcmap	*cm_info ;	/* pointer to shared memory */
	struct cmap_grab *cm_next ;	/* for client use */
	caddr_t		cm_client ;	/* for client use */
	int		cm_count ;	/* last recorded change count */
	int		cm_cinfofd ;	/* fd of shared memory file */
	int		cm_cdevfd ;	/* fd of framebuffer */
	u_long		*cm_clockp ;	/* pointer to lock page */
	u_long		*cm_cunlockp ;	/* pointer to unlock page */
	int		cm_lockcount ;	/* lock count for nested locks */
} Cmap_Grab ;



#define CM_PAGESZ       (8*1024)
#define CM_LOCK(x)      (*(x)) = 1
#define CM_UNLOCK(x)    (*(x)) = 0

#ifndef SH_USER		/* system shapes */

#define	sh_cmapgrab_cookie(infop)	((infop)->cm_cookie)

#define	sh_cm_lock(infop)		\
    do {				\
    if( (infop)->cm_slockcnt++ == 0 )	\
	*(infop)->cm_slockp = 1 ;	\
    } while (0)

#define	sh_cm_unlock(infop)		\
    do {				\
    if( --(infop)->cm_slockcnt <= 0 )	\
	*(infop)->cm_sunlockp = 0 ;	\
    } while (0)


#define	sh_cmapgrab_install(infop)	\
	do {				\
	  infop->cm_installed = 1 ;	\
	  ++infop->cm_count ;		\
	} while(0) ;

#define	sh_cmapgrab_uninstall(infop)	\
	do {				\
	  infop->cm_installed = 0 ;	\
	  ++infop->cm_count ;		\
	} while(0) ;


#else	SH_USER		/* client */

#define	sh_cm_lock(infop)		\
    do {				\
    if( (infop)->cm_lockcount++ == 0 )	\
	*(infop)->cm_clockp = 1 ;	\
    } while (0)

#define	sh_cm_unlock(infop)		\
    do {				\
    if( --(infop)->cm_lockcount == 0 )	\
	*(infop)->cm_cunlockp = 0 ;	\
    } while (0)

#endif	SH_USER


Grabbedcmap	*sh_cmapgrab_grab() ;


#endif	_CMAP_GRAB
