/****
 *
 * Copyright (c) 1989 by Sun Microsystems, Inc.
 * @(#)sh_win_grab.h 23.2 90/06/19 Copyright 1989 Sun Micro
 *
 * WIN_GRAB.H Window Grabber
 *
 * NOTE:  The WGINFO structure is a public structure which is stored in
 * shared memory.  It needs to be both backward and forward compatible.
 * In particular, this means that nothing before the
 * "FLOATING INFO STARTS HERE" comment should ever be changed, although
 * additions are permissible.	-- falk
 *
 ****/

#ifndef _WGRAB

#include <sys/types.h>
#include "di/shapes.h"
#include "sh_SHAPE.h"
#include "sh_CTX.h"
#include "sh_RAS.h"
#include "sh_PATH.h"
#include "sh_TRANS.h"
#include "sh_CMAP.h"
#include "sh_scr_dev.h"
#include "sh_point.h"
#include "di/sh_SCR.h"
#include "di/shape.h"

#ifdef	WINDOWGRABBER

#ifndef	SH_RECT_FLAG
#define	SH_RECT_FLAG	1
#define	SH_SPECIAL_FLAG	2
#define	SH_WX_EMPTY	128
#endif	SH_RECT_FLAG

typedef Unsgn32   *WX_LOCKP_T;

typedef struct  wx_winfo {
  /* VERSION 0 INFO STARTS HERE */
	unsigned long w_flag;		/* cliplist flag */
	long          w_magic;		/* magic number, "GRAB"=0x47524142 */
	long          w_version;	/* version, currently 0 */
	WX_LOCKP_T    w_cunlockp;	/* used to be w_list */
					/* points to unlock page */

	/* server info, meaningless to client */
	int           w_sinfofd;	/* fd of wxinfo file */
	int           w_sdevfd;		/* fd of framebuffer */
	WX_LOCKP_T    w_slockp;		/* pointer to lockpage */
	int           w_srefcnt;	/* lock count for nested locks */

	/* client info */
	char          w_devname[20];	/* framebuffer device name */
	int           w_cinfofd;	/* fd of wxinfo file */
	int           w_cdevfd;		/* fd of framebuffer */
	int           w_crefcnt;	/* lock count for nested locks */

	Unsgn32 	w_cookie;	/* "cookie" for this window */

	Unsgn32 	w_filesuffix;	/* private to server */
	WX_LOCKP_T	w_clockp;       /* pointer to lock page */

	/* clipping info */
	ENTRY         *w_cclipptr;    	/* client virtual pointer to    */
				    	/* clip array.                  */
	u_int  	w_scliplen;     	/* server's size of cliplist    */
				    	/* in bytes.                    */
	/* shapes stuff */
	POINT_B2D     w_org;          	/* origin of window */
	POINT_B2D     w_dim;          	/* size of window */

	/* VERSION 1 INFO STARTS HERE */
	long    w_clipoff;      /* byte offset from wx_info to  */
				/* w_cliparray.                 */
	long    w_shapeoff;     /* byte offset from wx_info to  */
				/* w_shape.                     */
	long    w_clipseq;      /* clip sequence number         */
	long    w_ccliplen;     /* client's size of clip list   */
				/* in bytes.                    */
	ENTRY	*w_sclipptr;	/* server virtual pointer to	*/
				/* extended clip array		*/

  /* FLOATING INFO STARTS HERE, CLIENT DO NOT REFER DIRECTLY
     TO ANYTHING BELOW THIS LINE */
	Static_Obj(w_shape,struct class_SHAPE);
	ENTRY	w_cliparray;
} WXINFO ;

	

#define WX_PAGESZ	(8*1024)
#define WXINFO_OFF	(2*WX_PAGESZ)
#define WG_LOCK(x)	(*(int *)(x)) = 1
#define WG_UNLOCK(x)	(*(int *)((x)+(WX_PAGESZ/sizeof(int)))) = 0

#define MINSHMEMSIZE    (8*1024)
#define	WXCLIPSIZE(infop)	\
	(MINSHMEMSIZE-(int)((char *)&(infop)->w_cliparray-(char *)(infop)))

extern	Unsgn32 *sh_wg_dummy;

#define wg_init()		sh_wg_init()

#define wg_get_grab(ras)	sh_wg_get_grab(ras)

#ifdef SH_USER

#define wg_initras(ras)			\
	((ras)->ras_lock = sh_wg_dummy, \
	 (ras)->ras_infop = sh_wg_dummy+WXINFO_OFF/sizeof(Unsgn32))
#define wg_lock(ras)			\
    do {				\
    if ((((WXINFO*)((ras)->ras_infop))->w_crefcnt)++ == 0) { \
	WG_LOCK((ras)->ras_lock);		\
	if (*(Unsgn32 *)(ras)->ras_infop) \
	    sh_wg_get_clip(ras);	\
    }					\
    } while (0)
#define wg_unlock(ras)			\
    do {				\
    if (--(((WXINFO*)((ras)->ras_infop))->w_crefcnt) == 0)	\
	WG_UNLOCK((ras)->ras_lock);	\
    } while (0)

#define wg_set_grab(ras,val)	sh_wg_set_grab_usr(ras,val)
#define wg_set_clip(ras)

#define wg_raster_exists	0

#else	/* SH_USER (System) */

#define wg_initras(ras)		\
	(ras)->ras_lock = sh_wg_dummy
#define wg_lock(ras)			\
    do {				\
    if (((WXINFO *)(ras->ras_lock+WXINFO_OFF/sizeof(Unsgn32)))->w_srefcnt++ == 0)\
	WG_LOCK(ras->ras_lock);		\
    } while (0)
#define wg_unlock(ras)			\
    do {				\
    if (--((WXINFO *)(ras->ras_lock+WXINFO_OFF/sizeof(Unsgn32)))->w_srefcnt == 0)\
	WG_UNLOCK(ras->ras_lock);	\
    } while (0)
#define wg_set_grab(ras,val)	sh_wg_set_grab_sys(ras,val)
#define wg_set_clip(ras)		\
    do {				\
    if (ras->ras_lock != sh_wg_dummy)	\
	sh_wg_set_clip(ras);		\
    } while (0)
#define wg_lockall(ras)		sh_wg_lockall(ras)
#define wg_unlockall(ras)	sh_wg_unlockall(ras)

extern	caddr_t	sh_wg_wlist;
#define wg_raster_exists	sh_wg_wlist

#endif	/* SH_USER */

#else	/* WINDOWGRABBER */

#define wg_init()
#define wg_initras(r)
#define wg_set_clip(r)
#define wg_lock(r)
#define wg_unlock(r)
#define wg_get_grab(r)
#define wg_set_grab(r,v)
#define wg_lockall(r)
#define wg_unlockall(r)

#define wg_raster_exists	0

#endif	/* WINDOWGRABBER */

#define _WGRAB
#endif	/* _WGRAB */
