/*
 * @(#)canvas.h 23.2 90/06/19 SMI
 */

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

/*
 * canvas.h - canvas-related information used at the nucleus level and higher
 */

#ifndef _canvas_h
#define _canvas_h

#include <NeWS/PostScript.h>
#include <cscript/cs_canvas.h>
#include <X11/pixmap.h>
#include <NeWS/input.h>

/*
 * Used as values for eventsconsumed slot in a canvas.
 */
 
#define NoEvents        0
#define MatchedEvents   1
#define AllEvents       2

/*
 * Server canvas structure - used by NeWS and X11 code.
 *
 * X-specific information: background tile and pixel; border tile and
 * pixel; and overrideRedirect flag.  I think we decided we wanted the
 * window ID for all canvases at this level (?).
 */

struct xn_canvas {

    ROOTED_FIELDS 			/* cscript rooted canvas struct */

    EvConsCell *preInterest;		/* pre-child interest list for canvas */
    EvConsCell *postInterest;		/* post-child interest list for canvas*/

    /*
     * match masks for this canvas:
     *  Pre child masks, post child masks, and descendent masks
     */
    MatchMask preName,  preAction;
    MatchMask postName, postAction;
    MatchMask ancesPostName, ancesPostAction;

    /* background and border for X window */
    PixUnion	background;
    PixUnion	border;

    unsigned damage_in_q:1;		/* True iff a damage event is queued
					 * for this canvas */
    unsigned overrideRedirect:1;	/* Override Redirect */
    unsigned eventsconsumed:2;		/* Whether canvas will prevent */
					/* lower canvases from */
					/* receiving events */
    unsigned	borderIsPixel:1;	/* X border is a pixel value */
    unsigned	bgState:2;		/* X background: None, Pixel, Parent, Pixmap */
    unsigned	key_grab:1;
    unsigned	button_grab:1;
    unsigned	pad:7;
};

typedef struct xn_canvas *XNCANVAS;

#define xncv_eventsconsumed(cv)			\
	(((XNCANVAS)entity_addr(cv))->eventsconsumed)

#define xncv_seteventsconsumed(cv, ev)		\
	(((XNCANVAS)entity_addr(cv))->eventsconsumed = (ev))

#endif 	/* #ifndef _canvas_h */
