/*
 * @(#)inputdist.h 23.2 90/06/19 SMI
 */


/*
 * Input distribution header file.  This contains most of the macros
 * necessary for manipulating input match masks.
 */

#include <memory.h>		/* for mem* operators */

/*
 * these two values should be set dependent on architectural specifics
 */
#define MASKWIDTH	32	/* Match Mask width desired in bits */
#define BitsPerInt	32	/* unsigned int width in bits */


/*
 * Reserved bits:
 * OTHER is used to mark the begining bit, and NULLBIT is used to mark
 * the last bit in inputdist.c.  Don't make them anything other than
 * these values.
 */
#define OTHER		0		/* the "other" or misc. bit */
#define KBDBIT		1		/* the keyboard values */
#define NULLBIT		(MASKWIDTH - 1)	/* the NULL value bit */


/*
 * the MatchMask structure
 */
typedef struct matchmask {
        unsigned int match_bits[(MASKWIDTH + (BitsPerInt - 1)) / BitsPerInt];
} MatchMask;



/*
 * All of these routines are special cased for implementations where
 * the bit mask spans word boundaries.
 *
 * M_SET -
 *	set the specified bit
 *
 * M_CLR -
 *	clear the specified bit
 *
 * M_ISSET -
 *	return TRUE if the specified bit is set
 *
 * M_ZEROALL -
 *	zero the entire MatchMask
 *
 * M_SETALL -
 *	set all Match bits to 1
 *
 * M_BITOR -
 *	Do a bit or on the masks, and stuff into dest.  Functional
 * 	semantics (i.e. dest could be either of the original masks)
 *
 * M_EQUAL -
 *	set result to TRUE if the two bitmasks are bit-for-bit equivalent.
 *
 * cvPreInterest -
 *	set result to TRUE if the given canvas may have a pre-child interest
 *	in this event.
 *
 * cvPostInterest -
 *	set result to TRUE if the given canvas may have a post-child interest
 *	in this event.
 *
 * cvPostAncesInterest -
 *	set result to TRUE if the given canvas' ancestors may have a
 *	post-child interest in this event.
 *
 * interestMatch -
 *	set result to TRUE if the given interest may match this event.
 */

#if MASKWIDTH <= BitsPerInt	/* Mask width <= integer width */

#define M_SET(n, p)		((p)->match_bits[0] |= (1 << (n)))
#define M_CLR(n, p)		((p)->match_bits[0] &= ~(1 << (n)))
#define M_ISSET(n, p)		((p)->match_bits[0] & (1 << (n)))

#define M_ZEROALL(p) ((p)->match_bits[0] = 0)
#define M_SETALL(p) ((p)->match_bits[0] = ~0)

#define M_BITOR(dest, p1, p2)						\
	{(dest)->match_bits[0] = (p1)->match_bits[0] | (p2)->match_bits[0]; }

#define M_BITAND(dest, p1, p2)						\
	{(dest)->match_bits[0] = (p1)->match_bits[0] & (p2)->match_bits[0]; }

#define M_EQUAL(p1, p2, result)						\
	{(result) = ((p1)->match_bits[0] == (p2)->match_bits[0]); }

#define cvInterest(C, mask, result)					\
	result = ((((C)->preName.match_bits[0]				\
				& mask.match_bits[0])			\
				== mask.match_bits[0])			\
		||(((C)->postName.match_bits[0]				\
				& mask.match_bits[0])			\
				== mask.match_bits[0]))
	
#define cvPreInterest(C, E, result) {					\
	result = ((((C)->preName.match_bits[0]				\
		      & (E)->corpus.event.nameMask.match_bits[0])	\
		      ==(E)->corpus.event.nameMask.match_bits[0])	\
		&&							\
	          (((C)->preAction.match_bits[0]			\
		       & (E)->corpus.event.actionMask.match_bits[0])	\
		       ==(E)->corpus.event.actionMask.match_bits[0]));	\
}

#define cvPostInterest(C, E, result) {					\
	result = ((((C)->postName.match_bits[0]				\
		      & (E)->corpus.event.nameMask.match_bits[0])	\
		      ==(E)->corpus.event.nameMask.match_bits[0])	\
		&&							\
	          (((C)->postAction.match_bits[0]			\
		       & (E)->corpus.event.actionMask.match_bits[0])	\
		       ==(E)->corpus.event.actionMask.match_bits[0]));	\
}

#define cvPostAncesInterest(C, E, result) {				\
	result = ((((C)->ancesPostName.match_bits[0]			\
		      & (E)->corpus.event.nameMask.match_bits[0])	\
		      ==(E)->corpus.event.nameMask.match_bits[0])	\
		&&							\
	          (((C)->ancesPostAction.match_bits[0]			\
		       & (E)->corpus.event.actionMask.match_bits[0])	\
		       ==(E)->corpus.event.actionMask.match_bits[0]));	\
}

#define interestMatch(I, E, result) {					\
	result = ((((I)->corpus.event.nameMask.match_bits[0]		\
		      & (E)->corpus.event.nameMask.match_bits[0])	\
		      ==(E)->corpus.event.nameMask.match_bits[0])	\
		&&							\
	          (((I)->corpus.event.actionMask.match_bits[0]		\
		       & (E)->corpus.event.actionMask.match_bits[0])	\
		       ==(E)->corpus.event.actionMask.match_bits[0]));	\
}

#define XinterestMatch(I, E, result) {					\
	result = ((((I)->nameMask.match_bits[0]				\
		      & (E)->corpus.event.nameMask.match_bits[0])	\
		      ==(E)->corpus.event.nameMask.match_bits[0])	\
	    	&&							\
		  (((I)->actionMask.match_bits[0]			\
		      & (E)->corpus.event.actionMask.match_bits[0])	\
		      ==(E)->corpus.event.actionMask.match_bits[0]));	\
}


#else		/* Mask width greater than integer width */

#define M_SET(n, p)		((p)->match_bits[(n)/BitsPerInt] \
					|= (1 << ((n) % BitsPerInt)))
#define M_CLR(n, p)		((p)->match_bits[(n)/BitsPerInt] \
					&= ~(1 << ((n) % BitsPerInt)))
#define M_ISSET(n, p)		((p)->match_bits[(n)/BitsPerInt] \
					& (1 << ((n) % BitsPerInt)))

/* REMIND: would a while loop be faster? */
#define M_ZEROALL(p) ((void)memset((p), 0, sizeof(*(p))))
#define M_SETALL(p) ((void)memset((p), ~0, sizeof(*(p))))

#define M_BITOR(dest, p1, p2) {						\
	register int i = 0;						\
	while (i < (MASKWIDTH / BitsPerInt))				\
	    (dest)->match_bits[i] = (p1)->match_bits[i] | (p2)->match_bits[i];\
}

#define M_BITAND(dest, p1, p2) {					\
	register int i = 0;						\
	while (i < (MASKWIDTH / BitsPerInt))				\
	    (dest)->match_bits[i] = (p1)->match_bits[i] & (p2)->match_bits[i];\
}

#define M_EQUAL(p1, p2, result) {					\
	register int i = 0;						\
	while (i < (MASKWIDTH / BitsPerInt) &&				\
		((p1)->match_bits[0] == (p2)->match_bits[0])) i++;	\
	result = (i < (MASKWIDTH / BitsPerInt));			\
}

#define cvInterest(C, mask, result) {					\
	register int i, tmpres;						\
	i = 0;								\
	while ((i < (MASKWIDTH / BitsPerInt)) &&			\
	         ((((C)->preName.match_bits[i]				\
			& mask.match_bits[i]) == mask.match_bits[i]))) i++;\
	tmpres = (i < (MASKWIDTH / BitsPerInt));			\
	i = 0;								\
	while ((i < (MASKWIDTH / BitsPerInt)) &&			\
	         ((((C)->postName.match_bits[i]				\
			& mask.match_bits[i]) == mask.match_bits[i])))i++;\
	result = tmpres || (i < (MASKWIDTH / BitsPerInt));		\
}

#define cvPreInterest(C, E, result) {					\
	register int i = 0;						\
	while ((i < (MASKWIDTH / BitsPerInt)) &&			\
	         ((((C)->preName.match_bits[i]				\
		      & (E)->corpus.event.nameMask.match_bits[i])	\
		      ==(E)->corpus.event.nameMask.match_bits[i])	\
		&&							\
	          (((C)->preAction.match_bits[i]			\
		       & (E)->corpus.event.actionMask.match_bits[i])	\
		       ==(E)->corpus.event.actionMask.match_bits[i]))) i++;\
	result = (i < (MASKWIDTH / BitsPerInt));			\
}

#define cvPostInterest(C, E, result) {					\
	register int i = 0;						\
	while ((i < (MASKWIDTH / BitsPerInt)) &&			\
	         ((((C)->postName.match_bits[i]				\
		      & (E)->corpus.event.nameMask.match_bits[i])	\
		      ==(E)->corpus.event.nameMask.match_bits[i])	\
		&&							\
	          (((C)->postAction.match_bits[i]			\
		       & (E)->corpus.event.actionMask.match_bits[i])	\
		       ==(E)->corpus.event.actionMask.match_bits[i]))) i++;\
	result = (i < (MASKWIDTH / BitsPerInt));			\
}

#define cvPostAncesInterest(C, E, result) {				\
	register int i = 0;						\
	while ((i < (MASKWIDTH / BitsPerInt)) &&			\
	         ((((C)->ancesPostName.match_bits[i]			\
		      & (E)->corpus.event.nameMask.match_bits[i])	\
		      ==(E)->corpus.event.nameMask.match_bits[i])	\
		&&							\
	          (((C)->ancesPostAction.match_bits[i]		\
		       & (E)->corpus.event.actionMask.match_bits[i])	\
		       ==(E)->corpus.event.actionMask.match_bits[i]))) i++;\
	result = (i < (MASKWIDTH / BitsPerInt));			\
}

#define interestMatch(I, E, result) {					\
	register int i = 0;						\
	while ((i < (MASKWIDTH / BitsPerInt)) &&			\
	         ((((I)->corpus.event.nameMask.match_bits[i]		\
		      & (E)->corpus.event.nameMask.match_bits[i])	\
		      ==(E)->corpus.event.nameMask.match_bits[i])	\
		&&							\
	          (((I)->corpus.event.actionMask.match_bits[i]		\
		       & (E)->corpus.event.actionMask.match_bits[i])	\
		       ==(E)->corpus.event.actionMask.match_bits[i]))) i++;\
	result = (i < (MASKWIDTH / BitsPerInt));			\
}


#endif	/* MASKWIDTH <= BitsPerInt */


/*
 * interest backptr structure
 */
typedef struct {
    union {
	struct {
	    struct corpus	*interest;
	    unsigned		name:1;
	    unsigned		action:1;
	    unsigned		canvas:1;
	    unsigned		pad:5;
	} back;
	long size;
    } value;
} IntBackPtr;


/* some function defs */

void newCanvasMasks(/* REF */);
void buildCanvasMasks(/* REF, int, struct corpus * */);
void setEventMatchMask(/* struct corpus * */);
void reparentCanvasMasks(/* REF, REF */);
void removeInterestBackptr(/* struct corpus *, struct object * */);
void addInterestBackptr(/* struct corpus *, struct object * */);
void deleteInterestObj(/* IntBackPtr *, struct object * */);
void addInterestObj(/* IntBackPtr *, struct object * */);
void add_invalid_canvas(/* struct object *, int */);
