/*
 * @(#)nucleus.h 23.2 90/06/19 SMI
 */

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

/*
 * Definitions for nucleus level of the system.  Primarily
 * related to server interpreter processing
 */

#ifndef	_nucleus_h
#define _nucleus_h

/*
 * getref ( struct object *object_addr )
 *	Given the address of an object structure, access the
 *	REF fields that are the first set of fields in that
 *	structure.
 */
#define getref(object_addr)	(*(REF *)(object_addr))

/*
 * struct operator_def
 * 	The definition descriptor of a built-in operator
 */
struct operator_def {
    int         index;          /* The magic number for this operator (it's
                                   irrelevant for called operators) */
    char       *pname;          /* Its print name */
    short       args_used;      /* Minimum # of arguments that will be used */
    short       growth;         /* Minimum amount by which this operator will
                                 * cause the stack to grow (if it grows by more
                                 * than this amount, the operator will have to
                                 * check for stack overflow itself) */
    int         (*function) (); /* The function implementing this operator */
    int         pack_index;     /* If != zero, it's packable in exec. arrays */
    struct kwd_name *bindname;  /* Used for bindoverride */
};


/* 
 * Keyword name are represented by 16 bits of length followed by
 * the name character string.
 */
struct kwd_name {
	unsigned short namelen;		/* Name string length */
	char name[1];			/* 1st element of the name string */
};

#define MAX_KWD_LEN	65535		/* # chars that can fit in kwd_name */

struct kwd_hash_table {
	struct kwd_name *kwd;	/* Pointer to string data */
};

extern struct kwd_hash_table *kwd_hash_table;	/* Hash table base */
extern unsigned int kwd_hash_table_size;	/* Current hash table size */

struct kwd_hash_table *rehash(/*
	unsigned oldsize,	struct kwd_hash_table *oldtable,
	unsigned newsize,	struct kwd_hash_table *newtable */);

/*
 * Kwd name hash generator. Hash is the return value.
 * CAVEATS: macro effects: name is set to the end of the string, 
 * length is set to -1 and initial hash value must be zero. 
 */
#define kwd_hash_generator(hash, c)		(hash << 5) - hash + c
#define kwd_hash(hash, length, name) \
{ \
	while(--length >= 0) \
    		hash = kwd_hash_generator(hash, *name++); \
}

struct object add_kwd_name(/* char *name */);
struct object add_kwd_name_len(/* char *name, int len */);

/* 
 * define_operator is called to define a new intrinsic operator 
 */
#define define_operator(name, function, index, args, results) {      \
    struct object o;						     \
    static struct operator_def d = 				     \
		{index, name, args, results - args, function};       \
    o = add_kwd_name(d.pname);	 	     	     		     \
    d.bindname = (struct kwd_name *) entity_addr(o);		     \
    define_operator_obj.value.def = &d; 			     \
    (void) define_object_in_dictionary(o, define_operator_obj,       \
			system_dictionary, (ENTITY *)0);	     \
}
/*
 * define_magic
 * a function that implements a magic dict operation.
 */
#define define_magic(name, dict, function, index, args, results) {   \
    struct object o;						     \
    static struct operator_def d = 				     \
		{index, name, args, results - args, function}; 	     \
    o = add_kwd_name(d.pname);	 	     	     		     \
    d.bindname = (struct kwd_name *) entity_addr(o);		     \
    magic_variable_obj.value.def = &d; 				     \
    (void) define_object_in_dictionary(o, magic_variable_obj,	     \
				dict, (ENTITY *)0); 		     \
}

/*
 * define_object - define an object in dict. Create a keyword object.
 * Put keyword and object info into specified dictionary.
 */ 
#define define_object(dict, name, object) { \
    define_object_in_dictionary(add_kwd_name(name),		      \
					object, dict, (ENTITY *)0);	   \
}

extern struct object define_operator_name, 
 		     define_operator_obj, 
		     magic_variable_obj;

/* 
 * Macros used to call get and/or set functions for magic variables.
 */
#define	GET_MAGIC(ix)	(((ix)<<1)|0)
#define	SET_MAGIC(ix)	(((ix)<<1)|1)
#define	IS_SET_MAGIC(i)	(i&1)

/* 
 * system_dictionary:	pointer to the system dictionary
 * stdout_file:		%stdout psfile
 * stderr_file:		%stderr psfile, same psiofile as stdout in outbuf
 * stdin_file:		%stdin psfile, same fd, new psiofile in inbuf
 * dict_table:		
 */
extern DICT *system_dictionary;		/* The system dictionary */
extern struct corpus *stdout_file;	/* Used for stdout output */
extern struct corpus *stderr_file;	/* Used for stderr output */
extern struct corpus *stdin_file;	/* Used for stdin input */
extern DICT *dict_table[]; 		/* Maps types to corresponding "magic" 
					 * dictionaries */

/* 
 * For masks that can be greater than 32 bits:  
 */
extern void	add_selectable_file( /* fd, read_flag */ );
extern void	remove_selectable_file( /* fd, read_flag */ );

/*
 * Dictionary monitoring tools: operators in objects.c 
 *
 * REMIND: the operators need to be put in a debug file.
 */
#ifdef DICT_TRACE
/* Dictionary monitoring */
extern int dict_trace_fd;
extern int dict_stats_fd;
extern int dict_trace_flg;
extern int dict_stats_flg;

#define DICT_TRACE_REPORT(old, new, s)  if(dict_trace_flg)\
                                                dict_trace_report(old, new, s)
#define DICT_STAT_REPORT(type, d, v)    if(dict_stats_flg)\
                                                dict_stats_report(type, d, v)
#define ZERO_COLLIDE()	collide = 0
#define DECLARE_COLLIDE	int ZERO_COLLIDE();
#define INCR_COLLIDE()  ++collide
enum dict_stat_type {
        ds_null_type,
        ds_active_type,
        ds_print_type,
        ds_collide_type,
        ds_create_type,
        ds_statchange_type,
        ds_systemdict_type,
        ds_free_type
};
#else
#define DICT_TRACE_REPORT(old, new, s)
#define DICT_STAT_REPORT(type, d, v)
#define DECLARE_COLLIDE
#define INCR_COLLIDE()
#define ZERO_COLLIDE()
#endif	/* DICT_TRACE */


#endif	/* #ifndef _nucleus_h */
