/* @(#)psio.c 23.1 90/03/23 SMI	 */

/*
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

/*-
	input side of the stdio for NeWS


 */

/*
 * The psio package is essentially equivalent to stdio except that it has some
 * extensions for NeWS.  The extensions are that it supports reading from
 * strings, it supports the non-blocking escape required by the lwp mechanism,
 * it supports "reading ahead" past a protected part of the input buffer, it
 * supports a hook to the output file for automatic flushing, and it supports
 * more file descriptors than stdio.  [many stdio implementations only allow
 * one iob per fd; with read/write descriptors, two are needed]
 */

#include "psio.h"
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>

#include <sys/time.h>

#ifdef SYSV
#include <stropts.h>
#include <poll.h>
#define EWOULDBLOCK EAGAIN
#endif

#include <fcntl.h>
#include <errno.h>
    extern int  errno;

extern int  errno;
extern long lseek();
extern char * malloc();
extern char * realloc();

#ifdef SYSV
/*
 * Lseek call.
 */
#define	L_SET		0	/* absolute offset */
#define	L_INCR		1	/* relative to current offset */
#define	L_XTND		2	/* relative to end of file */
#endif

#define INITBUF(p) if ((p)->bufsiz == 0) psio_initbuf(p)

static PSFILE	psio_stdfile[3] = {
    {0, 0, 0, 0, PSREAD, 0, 0},
    {0, 0, 0, 0, PSWRITE | PSLINEBUF, 1, 0},
    {0, 0, 0, 0, PSWRITE | PSLINEBUF, 2, 0}
};
PSFILE     *psio_stdin = &psio_stdfile[0];
PSFILE     *psio_stdout = &psio_stdfile[1];
PSFILE     *psio_stderr = &psio_stdfile[2];
int         psio_bufsiz;
long	    psio_bufspace;
int	    psio_bufcount;

psio_initbuf(p)
    register PSFILE *p;
{
    if (p->bufsiz == 0) {
	if ((p->flag & PSSTRG) == 0 && (p->flag & PSWRITE)) {
	    struct stat statbuf;
	    register fd = psio_fileno(p);
	    if (fd == 2 || (fstat(fd, &statbuf) == 0 &&
			    (statbuf.st_mode & S_IFCHR) != 0))
		p->flag |= PSLINEBUF;
	}

	if (psio_bufsiz <= 0 || psio_bufsiz >= 16 * PSBUFSIZ)
	    psio_bufsiz = PSBUFSIZ;
	p->bufsiz = psio_bufsiz;
	p->ptr = p->base = (unsigned char *) malloc((unsigned)p->bufsiz);
	psio_bufspace += p->bufsiz;
	++psio_bufcount;
	p->flag |= PSMYBUF;
    }
}

psio_expandbuf(p, n)
    register PSFILE *p;
{
    register	new_bufsiz = p->bufsiz + n + PSBUFSIZ;
    register	old_offset = p->ptr - p->base;
    register	char *new_base;

    if (p->bufsiz && !(p->flag & PSMYBUF))
	return -1;
    if (new_bufsiz > PSMAXBUFSIZ)
	if ((new_bufsiz = PSMAXBUFSIZ) <= p->bufsiz) {
	    p->flag |= PSERR;
	    return -1;
	}
    if (p->bufsiz == 0) {
	old_offset = 0;
	new_base = malloc((unsigned)new_bufsiz);
	if (new_base)
	    ++psio_bufcount;
    } else
	new_base = realloc(p->base, (unsigned)new_bufsiz);
    if (new_base == (char *)NULL) {
	p->flag |= PSERR;
	return -1;
    }
    p->base = (unsigned char *)new_base;
    p->flag |= PSMYBUF;
    p->ptr = p->base + old_offset;
    if (p->flag & PSWRITE)	    /* p->cnt is already correct for PSREAD */
	p->cnt = (p->base + new_bufsiz) - p->ptr;
    psio_bufspace += (new_bufsiz - p->bufsiz);
    p->bufsiz = new_bufsiz;
    return (0);
}

psio_freebuf(p)
    register PSFILE *p;
{
    if (p->base && (p->flag & PSMYBUF)) {
	free(p->base);
	psio_bufspace -= p->bufsiz;
	--psio_bufcount;
	p->base = p->ptr = (unsigned char *)NULL;
	p->cnt = p->bufsiz = 0;
	p->flag &= ~PSMYBUF;
    }
}

#define psio_nbdeclare		register psio_old_flags, psio_is_nb

#define psio_nbsetup(p)							\
    if ((psio_is_nb = (!psio_dnimode(p) && psio_nonblock(p))) &&	\
	(psio_old_flags = fcntl(p->file, F_GETFL, 0)) < 0) return -1

#define psio_nbexec(ret, type, fd, buf, siz) {				\
    if (psio_is_nb && !(psio_old_flags & O_NDELAY)) {			\
	/* Set to be non-blocking */					\
	if (fcntl(fd, F_SETFL, psio_old_flags | O_NDELAY) < 0)		\
	    return -1;							\
    }									\
    ret = type(fd, buf, siz);						\
    if (psio_is_nb && !(psio_old_flags & O_NDELAY)) {			\
	register type_errno;						\
	/* Restore original flags */					\
	type_errno = errno;						\
	if (fcntl(fd, F_SETFL, psio_old_flags) < 0)			\
	    return -1;							\
	errno = type_errno;						\
    }									\
}

/* before calling psio_filbuf, either set or clear the PSBLOCKOK bit */

psio_filbuf(p)
    register PSFILE *p;
{
    register    avail;
    psio_nbdeclare;

    psio_clearblocked(p);
    if (p->flag & (PSSTRG | PSEOF | PSERR | PSWRITE)) {
	p->flag |= PSEOF;
	return -1;
    }
    psio_nbsetup(p);
    INITBUF(p);
    while (1) {
	avail = p->bufsiz - p->protected;
	if (avail <= 0)
	    return -1;
	p->ptr = p->base + p->protected;
	if (p->assoc.outputside && psio_needsflush(p->assoc.outputside)) {
	    psio_flush(p->assoc.outputside);
	    psio_clearoutblocked(p->assoc.outputside);
	}
	psio_nbexec(p->cnt, read, p->file, p->ptr, avail);
	if (p->cnt > 0)
	    return psio_getc(p);
	if (p->cnt == 0) {
	    p->flag |= PSEOF;
	    return -1;
	}
	if (errno == EINTR)
	    continue;
	if (errno == EWOULDBLOCK) {
	    if (psio_blockok(p)) {
		psio_wait(p->file, 1);
		continue;
	    } else {
		psio_setblocked(p);
		return -1;
	    }
	}
	p->flag |= PSERR;
	return -1;
    }
}

PSFILE     *
psio_open(name, mode)
    char *name;
    register char *mode;
{   
    register unsigned open_flags;
    switch (mode[0]) {
    case 'a':
	open_flags = O_WRONLY | O_APPEND;
	break;
    case 'w':
	open_flags = O_WRONLY | O_TRUNC | O_CREAT;
	break;
    case 'r':
	open_flags = O_RDONLY;
	break;
    default:
	return (PSFILE *)0;
    }   
    return psio_fdopen(open(name, open_flags, 0644), mode);
}  

PSFILE     *
psio_fdopen(fd, mode)
    char *mode;
{
    register PSFILE *f;
    if (fd < 0)
	return 0;
    if ((f = (PSFILE *) malloc(sizeof(PSFILE))) == (PSFILE *)NULL)
	return 0;
    bzero((char *)f, sizeof *f);
    f->flag = (mode && mode[0] != 'r') ? PSWRITE : PSREAD;
    f->file = fd;
    return f;
}

PSFILE     *
psio_sopen(string, length, mode)
    unsigned char *string;
    char *mode;
{
    register PSFILE *f;
    if ((f = (PSFILE *) malloc(sizeof(PSFILE))) == (PSFILE *)NULL)
	return 0;
    bzero((char *)f, sizeof *f);
    f->cnt = f->bufsiz = length;
    f->flag = PSSTRG | (mode[0] == 'w' ? PSWRITE : PSREAD);
    if (string)
	f->base = f->ptr = string;
    else if (length) {
	f->base = f->ptr = (unsigned char *) malloc((unsigned)length);
	if (f->base == (unsigned char *)NULL) {
	    free(f);
	    return 0;
	}
	psio_bufspace += length;
	++psio_bufcount;
	f->flag |= PSMYBUF;
    }
    f->file = -1;
    return f;
}

int
psio_close(f)
    register PSFILE *f;
{
    if (f == 0)
	return -1;
    if ((f->flag & PSSTRG) == 0) {
	if ((f->flag & PSWRITE) && f->bufsiz > 0)
	    psio_flush_nb(f);
	if ((f->flag & PSREAD) && f->assoc.outputside)
	    psio_close(f->assoc.outputside);
	errno = 0;
	if (f->file >= 0 && close(f->file) < 0)
	    if (errno != EBADF)
	        perror("close");
    }
    psio_freebuf(f);
    free((char *)f);
    return (0);
}

psio_nodropclose(inf, outf)
    register PSFILE *inf, *outf;
{
#ifdef DEBUG
    if (inf && outf && inf->file == outf->file
		&& (inf->flag & PSSTRG) == 0 && (inf->flag & PSWRITE)) {
	char *msg = "assertion failure nodropclose\n";
	write(2, msg, strlen(msg));
	abort();
    }
#endif
    if (inf && outf) {
	if(outf->file == inf->file) {
	    _psio_nodropclose(outf);
	    psio_fileno(inf) = -1;	/* So close isn't called below */
	}
	_psio_nodropclose(inf);
    }
    else if (inf)
	_psio_nodropclose(inf);
    else if (outf)
	_psio_nodropclose(outf);
}

_psio_nodropclose(f)
    register PSFILE *f;
{
    if ((f->flag & PSSTRG) == 0) {
	if ((f->flag & PSWRITE) && f->bufsiz > 0)
	    psio_flush_nb(f);
#ifdef USETLI
	if (t_close(f->file) < 0)
	    errno = 0;
	    if (close(f->file) < 0)
		if (errno != EBADF)
	            perror("close");
#else
	errno = 0;
	if (f->file >= 0 && close(f->file) < 0)
	    if (errno != EBADF)
	        perror("close");
#endif
    }
    psio_freebuf(f);
    f->flag |= PSEOF;
    f->base = 0;
    f->ptr = 0;
    f->cnt = 0;
    f->file = -1;
    return 0;
}

psio_read(dest, sz0, n, p)
    register unsigned char *dest;
    register PSFILE *p;
    int         sz0,
                n;
{
    register    ret = 0,
                sz,
                c;

#ifndef	sun386
    while (--n >= 0) {
	for (sz = sz0; --sz >= 0;) {
#else
    /*
     * The pre-decrement is now at the botom of the loop -
     * check for 0 up front so we don't send garbage.
     */
    if (sz0 <= 0)
       return;
    do { 
	sz = sz0;
	do {
#endif
	    if ((c = psio_getc(p)) < 0)
		return ret;
	    *dest++ = c;
#ifndef	sun386
	}
	ret++;
    }
#else
	} while (--sz > 0);
	ret++;
    } while (--n > 0);
#endif
    return ret;
}

psio_write(src, sz0, n, p)
    register unsigned char *src;
    register PSFILE *p;
    int         sz0,
                n;
{
    register    ret = 0,
                sz;

#ifndef	sun386
    while (--n >= 0) {
	for (sz = sz0; --sz >= 0;)
#else
    /*
     * The pre-decrement is now at the botom of the loop -
     * check for 0 up front so we don't send garbage.
     */
    if (sz0 <= 0)
	return;
    do {
	sz = sz0;
	do {
#endif
	    if (psio_putc(*src++, p) < 0)
		return ret;
#ifndef	sun386
	ret++;
    }
#else
	} while (--sz > 0);
	ret++;
    } while (--n > 0);
#endif
    return ret;
}

psio_ungetc(c, p)
    register PSFILE *p;
{
    if (c == EOF || (p->flag & PSREAD) == 0)
	return EOF;
    INITBUF(p);
    if (p->ptr <= p->base)
	if (p->ptr == p->base && p->cnt == 0)
	    ++p->ptr;
	else
	    return EOF;
    *--p->ptr = c;
    ++p->cnt;
    return c;
}

/* before calling psio_flush, either set or clear the PSBLOCKOK bit */

psio_flush(p)
    register PSFILE *p;
{
    psio_nbdeclare;

    psio_clearoutblocked(p);
    if (p->flag & (PSSTRG | PSEOF | PSERR | PSREAD)) {
	p->flag |= PSERR | PSEOF;
	return -1;
    }
    psio_nbsetup(p);
    INITBUF(p);
    if (p->ptr > p->base) {	/* Buffer contains output */
	register unsigned char *pos = p->base;
	register    left = p->ptr - p->base;

	while (left > 0) {
	    register    written;

	    psio_nbexec(written, write, p->file, (char *)pos, left);
	    if (written <= 0)
		switch (errno) {
		case EWOULDBLOCK:
		    if (psio_blockok(p)) {
			psio_wait(p->file, 0);
			continue;
		    } else {
			psio_setoutblocked(p);
			if (pos > p->base)
			    bcopy(pos, p->base, left);
			p->cnt = p->bufsiz - left;
			p->ptr = p->base + left;
			return -1;
		    }
		    break;
		case EINTR:
		    break;
		default:
		    p->flag |= PSERR | PSEOF;
		    return -1;
		}
	    else {
		left -= written;
		pos += written;
	    }
	}
    }
    if ((p->cnt = p->bufsiz) > PSKEEPBUFSIZ && (p->flag & PSMYBUF)) {
	psio_freebuf(p);
	psio_initbuf(p);	/* psio_flush must always leave a buffer */
    } else
	p->ptr = p->base;
    return 0;
}

psio_wait(fd, read_flag)
{
#ifdef USETLI
    struct pollfd pfd[1];
#else
    fd_set read_bits;
    fd_set write_bits;
#endif
    int n = 0;
    register int i;

    /*
     * Wait wait_tv.  If n != 0 then got interrupted and try again.
     * This routine can wait up to 2*wait_tv and as little as almost
     * no time.
     */
#ifndef USETLI
    FD_ZERO(&read_bits);
    FD_ZERO(&write_bits);
    if (read_flag)
	FD_SET(fd, &read_bits);
    else
	FD_SET(fd, &write_bits);
    n = select(fd + 1, &read_bits, &write_bits, (fd_set *)NULL,
		(struct timeval *)NULL);
#else
    pfd[0].fd = fd;
    pfd[0].events = read_flag ? POLLIN : POLLOUT;
    n = poll(pfd, 1, -1);
#endif
    return n;
}

psio_flushbuf(c, p)
    unsigned char c;
    register PSFILE *p;
{
    if (((p->flag & PSSTRG) && (p->flag & PSWRITE)) || psio_flush_nb(p) < 0)
	if (psio_error(p) || (p->cnt <= 0 && psio_expandbuf(p, 1) < 0))
	    return -1;
    psio_clearoutblocked(p);
    p->cnt--;
    return *p->ptr++ = c;
}
