#ifndef lint
static char sccsid[] = "@(#)psiofprntf.c 23.3 90/06/19 Copyright 1985 Sun Micro";
#endif

/*
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

/*-
	fprintf equivalent for psio

 */

#ifdef REF
#include <sys/types.h>
#include <ref/config.h>
#endif
#include "psio.h"
#include <varargs.h>
#include <ctype.h>

/* if the definition of a fract changes include/fract.h this should likely
 * change to match.
 */
typedef int fract;

#define MAX_PRECISION	20	/* shouldn't want any more, Note buf[] size */
#define MAX_FLDWIDTH	100	/* shouldn't want any more, Note buf[] size */
#define FLDBUFFER	5	/* enough room for sign, 0., etc. */

#define DEFAULT_PRECISION	6
#define DEFAULT_FRACT_PRECISION	4

static char	nullstr[]  = "(null)";
static char	upperhex[] = "0123456789ABCDEF";
static char	lowerhex[] = "0123456789abcdef";

/* VARARGS2 */
psio_doprnt(f, fmt, args)
    register PSFILE *f;
    register char *fmt;
    va_list     args;
{
    register    c;

    while (c = *fmt++)
	if (c != '%') {
	    psio_putc(c, f);
	}
	else {
	    register	width = 0;
	    register	precision = -1;
	    int		left_just = 0,
			force_sign = 0,
			alternate = 0,
			is_signed = 1,
			long_flag = 0;
	    char	pad_char = ' ';
	    char	arg_buf[MAX_FLDWIDTH + FLDBUFFER];
	    char	*end_arg_buf = &arg_buf [sizeof arg_buf];
	    char	*hex_table   = lowerhex;
	    register char	*arg_ptr = end_arg_buf;
	    register 	arg_len;

	another_flag:

	    switch (c = *fmt++) {
	    case '0':
		pad_char = '0';
		goto another_flag;
	    case '-':
		left_just++;
		goto another_flag;
	    case '+':
		force_sign++;
		goto another_flag;
	    case ' ':
		force_sign = 0;
		goto another_flag;
	    case '#':
		alternate++;
		goto another_flag;
	    default:
		break;
	    }

	    if (c == '*') {
		width = va_arg(args, int);
		if (width < 0) {
		    left_just++;
		    width = -width;
		}
		c = *fmt++;
	    } else
		while (isdigit(c)) {
		    width = width * 10 + (c - '0');
		    c = *fmt++;
		}

	    if (c == '.') {
		if ((c = *fmt++) == '*') {
		    precision = va_arg(args, int);
		    c = *fmt++;
		} else {
		    precision = 0;
		    while (isdigit(c)) {
			precision = precision * 10 + (c - '0');
			c = *fmt++;
		    }
		}
	    }

	another_conversion:

	    switch (c) {
	    case 'l':
		if (sizeof (int) != sizeof (long))
		    long_flag++;
		c = *fmt++;
		goto another_conversion;
	    case 's':{
		    register char *p;
		    arg_ptr = va_arg(args, char *);
		    if (arg_ptr == NULL)
			arg_ptr = nullstr;
		    for (p = arg_ptr; precision-- && *p; p++)
			;
		    arg_len = p - arg_ptr;
		}
		break;
	    case 'X':
		hex_table = upperhex;
	    case 'x':{
		    register char		*hex = hex_table;
		    register unsigned long	n;
		    if ((sizeof (int) != sizeof (long)) && long_flag)
			n = va_arg(args, unsigned long);
		    else
			n = (unsigned long) va_arg(args, unsigned int);
		    do {
			*--arg_ptr = hex[n & 0xF];
			n >>= 4;
		    } while (n != 0);
		}
		break;
	    case 'O':
	    case 'o':{
		    register unsigned long	n;
		    if ((sizeof (int) != sizeof (long)) && long_flag)
			n = va_arg(args, unsigned long);
		    else
			n = (unsigned long) va_arg(args, unsigned int);
		    do {
			*--arg_ptr = (n & 07) + '0';
			n >>= 3;
		    } while (n != 0);
		}
		break;
	    case 'c':
		*--arg_ptr = (char) va_arg(args, int);
		arg_len = 1;
		break;
	    case 'u':
	    case 'U':
		is_signed = 0;
	    case 'i':
	    case 'D':
	    case 'd':{
		    register unsigned long	n;
		    register 			is_negative = 0;
		    if ((sizeof (int) != sizeof (long)) && long_flag)
			n = va_arg(args, unsigned long);
		    else
			n = (unsigned long) va_arg(args, unsigned int);
		    if (is_signed && ((int) n) < 0) {
			n = -n;
			is_negative++;
		    }
		    do {
			*--arg_ptr = (n % 10) + '0';
			n /= 10;
		    } while (n != 0);
		    if (is_negative)
			*--arg_ptr = '-';
		    else if (force_sign && is_signed)
			*--arg_ptr = '+';
		}
		break;
	    case 'e':
	    case 'E':
	    case 'f':
	    case 'g':
	    case 'G':
		{
		    extern      gconvert();
		    double	fval;

		    fval = va_arg(args, double);
		    arg_ptr = arg_buf + 1;
		    if (precision < 0)
			precision = DEFAULT_PRECISION;
		    else if (precision > MAX_PRECISION)
			precision = MAX_PRECISION;
		    gconvert(fval, precision, c, alternate, arg_ptr);
		    if (*arg_ptr != '-' && force_sign)
			*--arg_ptr = '+';
		    arg_len = strlen (arg_ptr);
		}
		break;
	    case 'p':		/* fixed(31,16) */
		{
		    extern      frconvert();
		    register char *fp;

		    arg_ptr = arg_buf + 1;
		    if (precision < 0)
			precision = DEFAULT_FRACT_PRECISION;
		    else if (precision > MAX_PRECISION)
			precision = MAX_PRECISION;
		    frconvert(va_arg(args, fract), precision,
			      alternate, arg_ptr);
		    if (*arg_ptr != '-' && force_sign)
			*--arg_ptr = '+';
		    arg_len = strlen (arg_ptr);
		}
		break;
	    default:
		*--arg_ptr = c;
		arg_len = 1;
		break;
	    }

	    switch (c) {
	    case 'x':
	    case 'X':
	    case 'o':
	    case 'O':
	    case 'd':
	    case 'D':
	    case 'i':
	    case 'u':
	    case 'U':{
		    register char sign = '\0';
		    arg_len = end_arg_buf - arg_ptr;
		    if (precision < 0)
			precision = 1;
		    else if (precision > MAX_PRECISION)
			precision = MAX_PRECISION;
		    if (*arg_ptr == '+' || *arg_ptr == '-') {
			sign = *arg_ptr++;
			arg_len--;
		    }
		    if (precision == 0 && arg_len == 1 && *arg_ptr == '0') {
			arg_len = 0;
			break;
		    }
		    while (arg_len < precision) {
			*--arg_ptr = '0';
			arg_len++;
		    }
		    if (sign) {
			*--arg_ptr = sign;
			arg_len++;
		    }
		    if (alternate) {
			switch (c) {
			case 'x':
			case 'X':
			    *--arg_ptr = c;
			    arg_len++;
			case 'o':
			case 'O':
			    if (*arg_ptr != '0') {
				*--arg_ptr = '0';
				arg_len++;
			    }
			    break;
			}
		    }
		}
		break;
	    }

	    if (!left_just || pad_char == '0') {
		if (pad_char == '0' && c != 's' && c != '-' && c != '+')
		    if (*arg_ptr == '-' || *arg_ptr == '+') {
			psio_putc(*arg_ptr++, f);
			arg_len--;
			width--;
		    }
		while (width > arg_len) {
		    psio_putc(pad_char, f);
		    width--;
		}
	    }
	    while (--arg_len >= 0) {
		psio_putc(*arg_ptr++, f);
		width--;
	    }
	    while (--width >= 0)
		psio_putc(pad_char, f);
	}
    if (psio_linebuf(f) && memchr(f->base, '\n', f->cnt) != NULL) {
	psio_flush(f);
	psio_clearoutblocked(f);
    }
}

#if defined(mc68000) || defined(sparc) || defined(i386)
/* IEEE standard codes: 	0x7FF00000 00000000	+infinity
 *				0xFFF00000 00000000	-infinity
 *				0x7FFxxxxx xxxxxxxx	-NaN
 */
#define POSINF 0x7FF00000
#define NEGINF 0xFFF00000

/* in: 	n		double
 * out:	<return value>	0	numeric
 *			1	infinity
 *			2	-infinity
 *			3	NaN
 */
int rangetest(n)		/* NON-PORTABLE tests for infinity and NaN */
double n;
{
    register long *lo = (long *)(&n);

#if defined(i386)
    /* most significant word is the second long for the i386 */
    lo++;
#endif
    if ( (*lo & POSINF) == POSINF) {
	if ( (*lo & NEGINF) == NEGINF)
	    return 2;		/* -infinity */
	else if (*lo & ~POSINF)
	    return 3;		/* NaN */
	else
	    return 1;		/* +infinity */
    } else {
	return 0;		/* numeric */
    }
}
#endif	/* mc68000 || sparc || i386 */

#if defined(vax)

#include <signal.h>
#include <setjmp.h>

jmp_buf	env;

/*
 * fake out IEEE reports for the VAX which is not so careful about
 * the values of numbers. Others may wish this routine to act
 * differently (ie interpret or report the exceptions differently).
 */
float_exception(sig, code, scp)
	int	sig, code;
	struct sigcontext	*scp;
{
	switch (code) {
		case FPE_FLTOVF_TRAP:
		case FPE_FLTOVF_FAULT:
			longjmp(env, 1);	/* +infinity */
			break;          	/* keep lint happy */
		case FPE_FLTUND_TRAP:
		case FPE_FLTUND_FAULT:
			longjmp(env, 2);	/* -infinity */
			break;
		default:
			longjmp(env, 3);	/* NaN */
	}
}
#endif	/* vax */

static
gconvert(n, precision, method, alternate, buffer)
    double      n;
    char	method;
    char       *buffer;
{
    register    decexp = 0;
    register	digits;
    register char *bp = buffer;
    char        decbuf[MAX_FLDWIDTH + FLDBUFFER];
    register char *dp = decbuf;
#if defined(vax)
    char *hold_bp;
    int (*hold_func)();
    int val;
#endif

    if (n < 0) {
	*bp++ = '-';
	n = -n;
    }
#if defined(mc68000) || defined(sparc) || defined(i386)
    switch ( rangetest(n) )
#endif
#if defined(vax)
    hold_bp = bp;			/* hold in case we get an exception */
    signal(SIGFPE, float_exception);
    if (val = setjmp(env))		/* returns 0 first time */
    {
	bp = hold_bp;			/* reset the bp pointer if exception */
	signal(SIGFPE, hold_func);
    }
    switch ( val )			
#endif
    {
    case 1:		/* n == infinity */
	*bp++ = 'i';
	*bp++ = 'n';
	*bp++ = 'f';
	*bp++ = '\0';
	return;
    case 2:   		/* n == -infinity */
	*bp++ = '-';
	*bp++ = 'i';
	*bp++ = 'n';
	*bp++ = 'f';
	*bp++ = '\0';
	return;
    case 3:		/* n == NaN */
	*bp++ = 'N';
	*bp++ = 'a';
	*bp++ = 'N';
	*bp++ = '\0';
	return;
    default:
	break;
    } /* end switch */
    if (n) {
	while (n >= 10) {
	    n = n / 10;
	    decexp++;
	}
	while (n < 1) {
	    n = n * 10;
	    decexp--;
	}
    }
    if (method == 'g' || method == 'G') {
	if (precision <= 0)
	    precision = 1;
	digits = precision;
    } else {
	if (method == 'f') {
	    digits = decexp + precision + 1;
	    if (decexp > 0 && digits > MAX_FLDWIDTH)
		method = 'e';
	}
	if (method != 'f')
	    digits = precision + 1;
    }
    if (digits < 0)
	digits = 0;
    do {
	register    digit = n;
	*dp++ = digit + '0';
	n = (n - digit) * 10;
    } while (--digits >= 0 && n);
    if (digits < 0 && *--dp >= '5') {
	while (1) {
	    if (dp <= decbuf) {
		*dp++ = '1';
		decexp++;
		break;
	    }
	    if (dp[-1] != '9') {
		dp[-1]++;
		break;
	    }
	    dp--;
	}
    }
    while (dp > decbuf && dp[-1] == '0')
	dp--;
    if (dp == decbuf) {
	*dp++ = '0';
	decexp = 0;
    }
    *dp = 0;
    if (method == 'g' || method == 'G') {
	if (decexp < -4 || (decexp > 0 && decexp >= precision)) {
	    method += 'E' - 'G';
	    if (alternate)
		precision--;
	    else
		precision = (dp - decbuf) - 1;
	} else
	    precision -= decexp + 1;
    }
    dp = decbuf;
    if (method == 'e' || method == 'E') {
	*bp++ = *dp++;
	if (*dp || alternate || precision > 0) {
	    *bp++ = '.';
	    while (*dp) {
		*bp++ = *dp++;
		precision--;
	    }
	    while (--precision >= 0)
		*bp++ = '0';
	}
	*bp++ = method;
	if (decexp < 0) {
	    decexp = -decexp;
	    *bp++ = '-';
	} else
	    *bp++ = '+';
	if (decexp >= 100) {
	    *bp++ = decexp / 100 + '0';
	    decexp %= 100;
	}
	*bp++ = decexp / 10 + '0';
	decexp %= 10;
	*bp++ = decexp + '0';
    }
    else if (decexp < 0) {
	if (decexp < -1 || method == 'f')
	    *bp++ = '0';
	if (precision > 0 || alternate) {
	    *bp++ = '.';
	    while (++decexp < 0 && precision > 0) {
		*bp++ = '0';
		precision--;
	    }
	    while (*dp && precision > 0) {
		*bp++ = *dp++;
		precision--;
	    }
	    while ((method == 'f' || alternate) && --precision >= 0)
		*bp++ = '0';
	}
    }
    else {
	while (decexp >= 0 && *dp) {
	    *bp++ = *dp++;
	    decexp--;
	}
	while (decexp >= 0) {
	    *bp++ = '0';
	    decexp--;
	}
	if (method == 'f' && precision > 0)
	    alternate++;
	if (alternate || *dp) {
	    *bp++ = '.';
	    while (*dp) {
		*bp++ = *dp++;
		precision--;
	    }
	    while (alternate && --precision >= 0)
		*bp++ = '0';
	}
    }
    *bp++ = 0;
#if defined(vax)
    signal(SIGFPE, hold_func);
#endif
}

static char *
decconvert(bp, num)
    register char *bp;
    register    num;
{
    if (!num)
	return bp;
    if (num >= 10)
	bp = decconvert(bp, num / 10);
    *bp++ = num % 10 + '0';
    return bp;
}

static
frconvert(n, precision, alternate, buffer)
    register fract	n;
    char		*buffer;
{
    register    decexp, ndigit;
    register char *bp = buffer, *dp;
    char        decbuf[MAX_FLDWIDTH + FLDBUFFER];
    if (n < 0) {
	*bp++ = '-';
	n = -n;
    }
    dp = decconvert(decbuf, ((unsigned) n) >> 16);
    decexp = dp - decbuf - 1;
    ndigit = precision;
    do {
	n = (n & 0xFFFF) * 10;
	*dp++ = (n >> 16) + '0';
    } while (--ndigit >= 0 && n);
    if (ndigit < 0 && *--dp >= '5') {
	while (1) {
	    if (dp <= decbuf) {
		*dp++ = '1';
		decexp++;
		break;
	    }
	    if (dp[-1] != '9') {
		dp[-1]++;
		break;
	    }
	    dp--;
	}
    }
    while (dp > decbuf && dp[-1] == '0')
	dp--;
    if (dp == decbuf) {
	*dp++ = '0';
	decexp = 0;
    }
    *dp = 0;
    dp = decbuf;
    while (decexp >= 0 && *dp) {
	*bp++ = *dp++;
	decexp--;
    }
    while (decexp >= 0) {
	*bp++ = '0';
	decexp--;
    }
    if (((n || *dp) && precision > 0) || alternate) {
	if (dp == decbuf)
	    *bp++ = '0';
	*bp++ = '.';
	while (decexp < -1 && precision > 0) {
	    *bp++ = '0';
	    precision--;
	    decexp++;
	}
	while (*dp && precision > 0) {
	    *bp++ = *dp++;
	    precision--;
	}
	if (alternate && bp[-1] == '.')
	    *bp++ = '0';
    }
    *bp++ = 0;
}

/* VARARGS2 */
psio_fprintf(f, fmt, va_alist)
    PSFILE     *f;
    char       *fmt;
    va_dcl
{
    psio_doprnt(f, fmt, &va_alist);
}

/* VARARGS1 */
psio_printf(fmt, va_alist)
    char       *fmt;
    va_dcl
{
    psio_doprnt(psio_stdout, fmt, &va_alist);
}

psio_sprintf(buf, fmt, va_alist)
    char       *buf;
    char       *fmt;
    va_dcl
{
    PSFILE	strfile;

    strfile.cnt = strfile.bufsiz = PSMAXBUFSIZ;
    strfile.ptr = strfile.base = (unsigned char *)buf;
    strfile.flag = PSSTRG | PSWRITE;
    strfile.file = -1;
    strfile.protected = 0;
    psio_doprnt(&strfile, fmt, &va_alist);
}
