/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static char sccsid[] = "@(#)pstoc.c 23.2 90/06/19 Copyright 1987 Sun Micro";
#endif

/*
 * Copyright (c) 1985 by Sun Microsystems, Inc. 
 */

/*-
	Receiving typed messages from postscript

	pstoc.c, Sun Jun 15 13:41:00 1986

 */

#ifdef PORTABLE
#include <sys/types.h>
#include <ref/config.h>
#else
#include <sys/time.h>
#endif
#include <fcntl.h>
#include <errno.h>
#include <varargs.h>
#include "encoding.h"
#include "pstoctable.h"
#include "psio.h"

#ifdef SYSV
#define EWOULDBLOCK EAGAIN
#endif

extern char *malloc();

#define formtag(ptr)		((short) ((ptr)[1] << 8) + (ptr)[2])
#define comparetag(ptr,tag)	((ptr)[0] == enc_tag && \
				 formtag(ptr) == (short) (tag))
#define INITBUF(p) if ((p)->bufsiz == 0) psio_initbuf(p)
#define FLUSH_OUTPUTSIDE(p)						\
    if (psio_getassoc(p) && psio_needsflush(psio_getassoc(p)))		\
	(psio_setblockok(psio_getassoc(p)),				\
	    psio_flush(psio_getassoc(p)))

#define PSRESET(p) if ((p)->protected) _psreset(p)


/* VARARGS */
pscanf(f, fmt, va_alist)
    register struct psiobuf *f;
    register char *fmt;
    va_dcl
{
    va_list     args;
    unsigned char dtype;
    va_start(args);
    FLUSH_OUTPUTSIDE(f);
    while (*fmt) {
	register    c;
	register    temp;
	register    len;
	c = *fmt++;
	if (c == '*') {
	    len = va_arg(args, int);
	    c = *fmt++;
	}
	else
	    len = 99999;
	if ((dtype = ttype[c]) == 0)
	    continue;
	c = psio_getc(f);
	if (dtype == enc_string + 1) {
	    *va_arg(args, char *) = c;
	    continue;
	}
#define case4(tag) case tag: case tag+1: case tag+2: case tag+3
#define case16(tag) case4(tag):case4(tag+4):case4(tag+8):case4(tag+12)
#define case32(tag) case16(tag):case16(tag+16)
	switch (c) {
	    /*
	     * An encoded integer: the bottom two bits of the opcode = (the
	     * number of bytes in the number)-1;  the next two bits = the
	     * number of bytes that are to follow the binary point
	     */
    case16(enc_int):
	    {
		register    n;
		temp = c;
		c = psio_getc(f);
		n = (char) c;
		while (temp & 3) {
		    --temp;
		    n = (n << 8) | psio_getc(f);
		}
		switch (dtype) {
		case enc_int:
		    *va_arg(args, int *) = n >> ((temp >> 2) & 3) * 8;
		    break;
		case enc_int + 1:
		    c = (2 - ((temp >> 2) & 3)) * 8;
		    if (c > 0) {
			*va_arg(args, int *) = n << c;
		    }
		    else if (c < 0) {
			*va_arg(args, int *) = n >> -c;
		    }
		    else {
			*va_arg(args, int *) = n;
		    }
		    break;
		case enc_IEEEfloat:
		    *va_arg(args, float *) = ((double) n) / (1 << (((temp >> 2) & 3) * 8));
		    break;
		case enc_IEEEdouble:
		    *va_arg(args, double *) = ((double) n) / (1 << (((temp >> 2) & 3) * 8));
		    break;
		default:
		    return -1;
		}
		break;
	case16(enc_short_string):
		temp = c & 017;
		goto read_encoded_string;
	case enc_IEEEfloat:
		{
		    union {
			float       f;
			long        n;
		    }           u;
		    u.n = psio_getc(f);
		    u.n = (u.n << 8) | psio_getc(f);
		    u.n = (u.n << 8) | psio_getc(f);
		    u.n = (u.n << 8) | psio_getc(f);
#if defined(vax)
		    /* This call is dangerous, the first instance of
		     * u is treated as a long and the second instance as
		     * float. The conversion is returned in the second u.
		     * If the following routine is modified, on must be
		     * careful to isolate the values.
		     */
		    ieee_vax_float(&u.n, &u.f);
#endif
		    switch (dtype) {
		    case enc_int:
			*va_arg(args, int *) = u.f;
			break;
		    case enc_IEEEfloat:
			*va_arg(args, float *) = u.f;
			break;
		    case enc_IEEEdouble:
			*va_arg(args, double *) = u.f;
			break;
		    default:
			return -1;
		    }
		}
		break;
	case4(enc_string):
		temp = 0;
		{
		    register    i = c & 3;
		    do {
			temp = (temp << 8) | psio_getc(f);
		    } while (--i >= 0);
		}
	read_encoded_string:
		if (dtype != enc_string)
		    return -1;
		{
		    register char *p = va_arg(args, char *);
		    while (--temp >= 0 && len > 0)
			*p++ = psio_getc(f), len--;
		    if (len == 0) {
			while (temp-- >= 0)
			    (void) psio_getc(f);
		    }
		    else
			*p++ = 0;
		}
		break;
	    }
	case enc_syscommon:{
		register char *p = va_arg(args, char *);
		while (c != '\n' && c != ' ' && len-- > 0) {
		    *p++ = c;
		    c = psio_getc(f);
		}
		if (len == 0) {
		    while (c != '\n' && c != ' ')	/* flush remainder */
			c = psio_getc(f);
		}
		else
		    *p = '\0';
		break;
	    }
	default:
	    return -1;
	}
    }
    va_end(args);
    return 0;
}

static
_psfilbuf(p, blockok)
    register struct psiobuf *p;
{
    register char *free_buffer;
    register	bytes_read;
    register    avail;
    int fcntl_flags;
    int read_errno;
    extern int errno;
    if (p->cnt == 0 && p->bufsiz > PSKEEPBUFSIZ)
	psio_freebuf(p);
    INITBUF(p);
    while ((avail = p->bufsiz - p->cnt) <= 0) {
	if (psio_expandbuf(p, 0) < 0)
	    return -1;
    }
    if (p->ptr > p->base) {
	if (p->cnt > 0)
	    bcopy((char *)p->ptr, (char *)p->base, p->cnt);
	p->ptr = p->base;
    }
    free_buffer = (char *) p->base + p->cnt;
    if (!blockok) {
	/* Remember original flags */
	if ((fcntl_flags = fcntl(p->file, F_GETFL, 0)) < 0)
	    return -1;
    }
    while (1) {
	if (!blockok && !(fcntl_flags & O_NDELAY)) {
	    /* Set to be non-blocking */
	    if (fcntl(p->file, F_SETFL, fcntl_flags | O_NDELAY) < 0)
		return -1;
	}
	bytes_read = read(p->file, free_buffer, avail);
	read_errno = errno;
	if (!blockok && !(fcntl_flags & O_NDELAY)) {
	    /* Restore original flags */
	    if (fcntl(p->file, F_SETFL, fcntl_flags) < 0)
		return -1;
	}
	if (bytes_read > 0) {
	    p->cnt += bytes_read;
	    return 0;
	}
	if (bytes_read == 0) {
	    p->flag |= PSEOF;
	    return -1;
	}
	if (read_errno == EINTR) {
	    continue;
	} else if (read_errno == EWOULDBLOCK) {
	    if (blockok)
		psio_wait(p->file, 1);
	    else
		return -1;
	    continue;
	} else {
	    p->flag |= PSERR;
	    p->cnt = 0;
	    return -1;
	}
    }
}

static
_psreset(p)
    register struct psiobuf *p;
{
    if (p->protected) {
	if (p->cnt > 0) {
	    bcopy((char *)p->ptr, (char *)p->base + p->protected, p->cnt);
	    p->cnt += p->protected;
	}
	else
	    p->cnt = p->protected;
	p->ptr = p->base;
	p->protected = 0; 
    }
}

static
_psgetoplength(ptr)
    register unsigned char *ptr;
{
    register char c;
    c = oplength[*ptr];
    if (c <= 0) {
	switch (c) {
	    case -2:
	    case -3:
	    case -4:
	    case -5:
	    {
		register len = 0;
		while (++c < 0)
		    len = len << 8 + *++ptr;
		return (len + 2);
	    }
	    default:
		return 1;
	}
    }
    else
	return c;
}

/* For the following routines, here are the entry/exit conditions (the
 * state of the PostScriptInput structure between procedure calls):
 *
 *	p->base always points to the beginning of the input buffer.
 *	p->bufsiz is always the size of the input buffer.
 *	p->protected indicates the number of characters at the beginning
 *		of the buffer which were skipped over from the last call
 *		to ps_checkfor(method = PSIO_WAIT_TAG).  All other calls
 *		currently leave p->protected = 0.  After psio_checkfor
 *		leaves protected > 0, some I/O is done temporarily by
 *		pscanf from the place in the buffer where the tag was
 *		found by ps_checkfor.  The next call to one of these functions
 *		will rejoin the buffer by calling _psreset.
 *	p->ptr points to the current byte to be read.  If ps_checkfor has
 *		found some tag for synchronous input, then p->ptr will point
 *		to the first yet-to-be-read character beyond the protected
 *		portion.  In all other cases, p->ptr will point to the first
 *		yet-to-be-read character from the beginning of the buffer.
 *	p->cnt points to the number of valid characters following p->ptr.
 *		Again, this may or may not be all of the characters in the
 *		buffer depending on a recent call to ps_checkfor.  Generally,
 *		p->cnt + p->protected == all of the yet-to-be-read characters
 *		in the entire buffer.
 *
 * Possible scenarios:
 *
 *	p->base		p->protected	p->ptr		p->cnt	p->bufsiz
 *	|ccccccccccccccc|...............|ccccccccccccccc|.......|
 * ps_checkfor found a tag at p->base+p->protected and I/O is continuing
 * at p->ptr via pscanf.  None of the other ps_??? functions have been
 * called since.
 *
 *	p->protected
 *	p->base		p->ptr			p->cnt		p->bufsiz
 *	|...............|ccccccccccccccccccccccc|...............|
 * Input is currently going on at p->ptr (either by pscanf or by the
 * ps_??? functions).  ps_checkfor has not been called recently.
 */

/* are we looking at a particular tag? */
ps_lookingat(p, tag)
    register struct psiobuf *p;
{
    INITBUF(p);
    FLUSH_OUTPUTSIDE(p);
    PSRESET(p);
    while (p->cnt < 3) {
	if (p->cnt > 0 && p->ptr[0] != enc_tag)
	    return 0;
	if (_psfilbuf(p, 1) < 0)
	    return -1;
    }
    if (comparetag(p->ptr, tag)) {
	p->ptr += 3;
	p->cnt -= 3;
	return 1;
    }
    return 0;
}

/* are we looking at a particular tag? */
ps_currenttag(p, method, tagbuf)
    register struct psiobuf *p;
    register method;
    register *tagbuf;
{
    INITBUF(p);
    FLUSH_OUTPUTSIDE(p);
    PSRESET(p);
    while (p->cnt < 3) {
	if (p->cnt > 0 && p->ptr[0] != enc_tag)
	    return 0;
	if (_psfilbuf(p, 1) < 0)
	    return -1;
    }
    if (p->ptr[0] != enc_tag)
	return 0;
    *tagbuf = formtag(p->ptr);
    if (method == PSIO_GET_TAG) {
	p->ptr += 3;
	p->cnt -= 3;
    }
    return 1;
}

/* Is a particular tag or any event waiting in the input */
ps_checkfor(p, method, tag)
    register struct psiobuf *p;
    register method, tag;
{
    register unsigned char *examine;
    register bytes_needed, bytes_examined;
    INITBUF(p);
    FLUSH_OUTPUTSIDE(p);
    PSRESET(p);
    if (p->ptr > p->base) {
	if (p->cnt > 0)
	    bcopy((char *)p->ptr, (char *)p->base, p->cnt);
	p->ptr = p->base;
    }
    bytes_examined = 0;
    bytes_needed = (method == PSIO_CHECK_INPUT) ? 1 : 3;
    while (1) {
	while (p->cnt - bytes_examined < bytes_needed) {
	    if (_psfilbuf(p, (method == PSIO_WAIT_TAG)) < 0)
		return (psio_error(p) || psio_eof(p)) ? -1 : 0;
	}
	examine = p->base + bytes_examined;
	if (method == PSIO_CHECK_INPUT || comparetag(examine, tag)) {
	    if (method == PSIO_WAIT_TAG) {
		p->protected = bytes_examined;
		p->ptr = examine + 3;
		p->cnt -= (bytes_examined + 3);
	    }
	    return 1;
	}
	bytes_examined += _psgetoplength (examine);
    }
}

/* wait for some particular tag, included for binary compatibility */
ps_waitfor(p, tag)
    struct psiobuf *p;
{
    return ps_checkfor(p, PSIO_WAIT_TAG, tag);
}

/* skip over some event in the buffer */
ps_skip(p)
    register struct psiobuf *p;
{
    register eventsize = 1;
    register sizeknown = 0;
    register used = 0;
    INITBUF(p);
    FLUSH_OUTPUTSIDE(p);
    PSRESET(p);
    while (eventsize) {
	if (p->cnt > 1 && !sizeknown) {
	    eventsize = _psgetoplength(p->ptr);
	    sizeknown = 1;
	}
	if (sizeknown) {
	    if (eventsize > p->cnt)
		used = p->cnt;
	    else
		used = eventsize;
	    p->cnt -= used;
	    p->ptr += used;
	    eventsize -= used;
	}
	if (eventsize) {
	    if (_psfilbuf(p, 1) < 0)
		return -1;
	}
    }
    return 1;
}

#if defined(vax)

/* What IEEE single precision floating point looks like on a Vax */
struct	ieee_single {
	unsigned int	mant: 23;
	unsigned int	exp     : 8;
	unsigned int	sign    : 1;
};

/* Vax single precision floating point */
struct	vax_single {
	unsigned int	mant1 : 7;
	unsigned int	exp       : 8;
	unsigned int	sign      : 1;
	unsigned int	mant2 : 16;
};

#define VAX_SNG_BIAS	0x81
#define IEEE_SNG_BIAS	0x7f

static struct sgl_limits {
	struct vax_single s;
	struct ieee_single ieee;
} sgl_limits[2] = {
	{{ 0x7f, 0xff, 0x0, 0xffff },	/* Max Vax */
	{ 0x0, 0xff, 0x0 }},		/* Max IEEE */
	{{ 0x0, 0x0, 0x0, 0x0 },	/* Min Vax */
	{ 0x0, 0x0, 0x0 }}		/* Min IEEE */
};

/* Thsi routine is typically called with lp, and rfp pointing to
 * the same location (as a union). You can not modify part of rfp
 * and expect lp not to be changed.
 */
ieee_vax_float(lp, rfp)
	register long *lp;
	register float *rfp;
{
	struct ieee_single is;
	struct vax_single *vsp;
	struct sgl_limits *lim;
	int i;


	is = *((struct ieee_single *)lp);
	vsp = (struct vax_single *)rfp;
	for (i = 0, lim = sgl_limits;
		i < sizeof(sgl_limits)/sizeof(struct sgl_limits);
		i++, lim++) {
		if ((is.exp == lim->ieee.exp) &&
			(is.mant == lim->ieee.mant)) {
			*vsp = lim->s;
			goto returnit;
		}
	}
	vsp->exp = is.exp - IEEE_SNG_BIAS + VAX_SNG_BIAS;
	vsp->mant2 = is.mant;
	vsp->mant1 = (is.mant >> 16);
returnit:
	vsp->sign = is.sign;
}

#endif
